@@ -13,6 +13,31 @@ def get_dict():
     return _dict
 
 
+AFC_ACQCORE_NR_PHY_TRIGGER = 8
+AFC_ACQCORE_NR_LOG_TRIGGER = 24
+
+SI_BPM_POLY_X_COEFF = [
+    8574000.0,   4728000.0,  4036000.0,  2814000.0, 9673000.0,
+    4015000.0,  10560000.0,  9858000.0, 86840000.0, 3947000.0,
+    5277000.0, 228500000.0, -1140000.0, 95490000.0, 24360000.0
+]
+SI_BPM_POLY_Y_COEFF = [
+    8574331.0,   4727847.0,  4035990.0,  2814060.0, 9673411.0,
+    4015438.0,  10564885.0,  9858212.0, 86840956.0, 3946578.0,
+    5276864.0, 228461777.0, -1139796.0, 95491966.0, 24361950.0
+]
+SI_BPM_POLY_SUM_COEFF = [
+    1.0, -1.3977e-05, -0.43378, -1.0575, -1.3979e-05,
+    2.6356, 3.7618, 12.457, -0.43378, 3.7614, 83.478,
+    -584.08, -1.0575, 12.459, -584.05, 6267.9
+]
+SI_BPM_POLY_Q_COEFF = [
+    0.925787421325340, 0.993974314425180, 1.0992213606011,
+    0.068300275370276, 0.046848344301059, 3.3136235284580,
+    0.092273210587639,-1.764922087231000, 2.9203104725574
+]
+
+
 _li_diags = [
     # Screens
     ['LA-BI:PRF1:CAM:AcquireTime', 0.001, 0],
@@ -107,8 +132,23 @@ _amcfpgaevrs = [
     'IA-20RaBPM:TI-AMCFPGAEVR:',
     'IA-20RaBPMTL:TI-AMCFPGAEVR:',
     ]
-_amcfpgaevr_amcs = ['AMC2', 'AMC3', 'AMC4', 'AMC5', 'AMC6', 'AMC7']
 _amcfpgaevr_propts = [
+    ['DevEnbl-Sel', 1, 0.0],
+    ['RTMFreqPropGain-SP', 1, 0.0],
+    ['RTMFreqIntgGain-SP', 128, 0.0],
+    ['RTMPhasePropGain-SP', 100, 0.0],
+    ['RTMPhaseIntgGain-SP', 1, 0.0],
+    ['RTMPhaseNavg-SP', 0, 0.0],
+    ['RTMPhaseDiv-SP', 0, 0.0],
+    ['AFCFreqPropGain-SP', 1, 0.0],
+    ['AFCFreqIntgGain-SP', 1500, 0.0],
+    ['AFCPhasePropGain-SP', 10, 0.0],
+    ['AFCPhaseIntgGain-SP', 1, 0.0],
+    ['AFCPhaseNavg-SP', 7, 0.0],
+    ['AFCPhaseDiv-SP', 1, 0.0],
+]
+_amcfpgaevr_amcs = ['AMC3', 'AMC6']
+_amcfpgaevr_amcs_propts = [
     ['State-Sel', 1, 0.0],
     ['Src-Sel', 3, 0.0],
     ['Dir-Sel', 0, 0.0],
@@ -120,8 +160,12 @@ _amcfpgaevr_propts = [
     ]
 _amcfpgaevr_pvs = list()
 for dev in _amcfpgaevrs:
+    # for AFC and RTM freq loops
+    for ppt, val, dly in _amcfpgaevr_propts:
+        _amcfpgaevr_pvs.append([dev+ppt, val, dly])
+    # for AMCs
     for amc in _amcfpgaevr_amcs:
-        for ppt, val, dly in _amcfpgaevr_propts:
+        for ppt, val, dly in _amcfpgaevr_amcs_propts:
             _amcfpgaevr_pvs.append([dev+amc+ppt, val, dly])
 
 _bpms = [
@@ -348,14 +392,9 @@ _bpms = [
     'SI-01M1:DI-BPM',
     ]
 _bpm_propts = [
-    [':INFOClkProp-Sel', 0, 0.0],
     [':ADCSi57xOe-Sel', 1, 0.0],
     [':ADCClkSel-Sel', 1, 0.0],
     [':INFOADCRate-SP', 1, 0.0],
-    [':ADCAD9510PllFunc-SP', 1, 0.0],
-    [':ADCAD9510Outputs-SP', 31, 0.0],
-    [':ADCAD9510ClkSel-Sel', 1, 0.0],
-    [':ADCAD9510CpCurrent-Sel', 7, 0.0],
     [':ADCTrigDir-Sel', 0, 0.0],
     [':ADCTrigTerm-Sel', 0, 0.0],
     [':RFFEPidSpAC-SP', 45.0, 0.0],
@@ -369,253 +408,21 @@ _bpm_propts = [
     [':RFFEPidBDTd-SP', 2.0, 0.0],
     [':RFFEHeaterAC-SP', 0.0, 0.0],
     [':RFFEHeaterBD-SP', 0.0, 0.0],
-    [':ADC0RstModes-Sel', 1, 0.0],
-    [':ADC1RstModes-Sel', 1, 0.0],
-    [':ADC2RstModes-Sel', 1, 0.0],
-    [':ADC3RstModes-Sel', 1, 0.0],
-    [':TRIGGER0Dir-Sel', 1, 0.0],
-    [':TRIGGER0DirPol-Sel', 1, 0.0],
-    [':TRIGGER0RcvLen-SP', 1, 0.0],
-    [':TRIGGER0TrnLen-SP', 1, 0.0],
-    [':TRIGGER1Dir-Sel', 1, 0.0],
-    [':TRIGGER1DirPol-Sel', 1, 0.0],
-    [':TRIGGER1RcvLen-SP', 1, 0.0],
-    [':TRIGGER1TrnLen-SP', 1, 0.0],
-    [':TRIGGER2Dir-Sel', 0, 0.0],
-    [':TRIGGER2DirPol-Sel', 1, 0.0],
-    [':TRIGGER2RcvLen-SP', 1, 0.0],
-    [':TRIGGER2TrnLen-SP', 1, 0.0],
-    [':TRIGGER3Dir-Sel', 0, 0.0],
-    [':TRIGGER3DirPol-Sel', 1, 0.0],
-    [':TRIGGER3RcvLen-SP', 1, 0.0],
-    [':TRIGGER3TrnLen-SP', 1, 0.0],
-    [':TRIGGER4Dir-Sel', 1, 0.0],
-    [':TRIGGER4DirPol-Sel', 1, 0.0],
-    [':TRIGGER4RcvLen-SP', 1, 0.0],
-    [':TRIGGER4TrnLen-SP', 1, 0.0],
-    [':TRIGGER5Dir-Sel', 1, 0.0],
-    [':TRIGGER5DirPol-Sel', 1, 0.0],
-    [':TRIGGER5RcvLen-SP', 1, 0.0],
-    [':TRIGGER5TrnLen-SP', 1, 0.0],
-    [':TRIGGER6Dir-Sel', 1, 0.0],
-    [':TRIGGER6DirPol-Sel', 1, 0.0],
-    [':TRIGGER6RcvLen-SP', 1, 0.0],
-    [':TRIGGER6TrnLen-SP', 1, 0.0],
-    [':TRIGGER7Dir-Sel', 1, 0.0],
-    [':TRIGGER7DirPol-Sel', 1, 0.0],
-    [':TRIGGER7RcvLen-SP', 1, 0.0],
-    [':TRIGGER7TrnLen-SP', 1, 0.0],
-    [':TRIGGER0RcvInSel-SP', 0, 0.0],
-    [':TRIGGER0RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER0TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER0TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER1RcvInSel-SP', 0, 0.0],
-    [':TRIGGER1RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER1TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER1TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER2RcvInSel-SP', 0, 0.0],
-    [':TRIGGER2RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER2TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER2TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER3RcvInSel-SP', 0, 0.0],
-    [':TRIGGER3RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER3TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER3TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER4RcvInSel-SP', 0, 0.0],
-    [':TRIGGER4RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER4TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER4TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER5RcvInSel-SP', 0, 0.0],
-    [':TRIGGER5RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER5TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER5TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER6RcvInSel-SP', 0, 0.0],
-    [':TRIGGER6RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER6TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER6TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER7RcvInSel-SP', 0, 0.0],
-    [':TRIGGER7RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER7TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER7TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER8RcvInSel-SP', 0, 0.0],
-    [':TRIGGER8RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER8TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER8TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER9RcvInSel-SP', 0, 0.0],
-    [':TRIGGER9RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER9TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER9TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER10RcvInSel-SP', 0, 0.0],
-    [':TRIGGER10RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER10TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER10TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER11RcvInSel-SP', 0, 0.0],
-    [':TRIGGER11RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER11TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER11TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER12RcvInSel-SP', 0, 0.0],
-    [':TRIGGER12RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER12TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER12TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER13RcvInSel-SP', 0, 0.0],
-    [':TRIGGER13RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER13TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER13TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER14RcvInSel-SP', 0, 0.0],
-    [':TRIGGER14RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER14TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER14TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER15RcvInSel-SP', 0, 0.0],
-    [':TRIGGER15RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER15TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER15TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER16RcvInSel-SP', 0, 0.0],
-    [':TRIGGER16RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER16TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER16TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER17RcvInSel-SP', 0, 0.0],
-    [':TRIGGER17RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER17TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER17TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER18RcvInSel-SP', 1, 0.0],
-    [':TRIGGER18RcvSrc-Sel', 1, 0.0],
-    [':TRIGGER18TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER18TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER19RcvInSel-SP', 0, 0.0],
-    [':TRIGGER19RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER19TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER19TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER20RcvInSel-SP', 0, 0.0],
-    [':TRIGGER20RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER20TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER20TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER21RcvInSel-SP', 0, 0.0],
-    [':TRIGGER21RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER21TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER21TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER22RcvInSel-SP', 0, 0.0],
-    [':TRIGGER22RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER22TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER22TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER23RcvInSel-SP', 0, 0.0],
-    [':TRIGGER23RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER23TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER23TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM0RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM0RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM0TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM0TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM1RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM1RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM1TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM1TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM2RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM2RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM2TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM2TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM3RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM3RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM3TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM3TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM4RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM4RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM4TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM4TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM5RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM5RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM5TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM5TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM6RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM6RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM6TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM6TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM7RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM7RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM7TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM7TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM8RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM8RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM8TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM8TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM9RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM9RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM9TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM9TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM10RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM10RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM10TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM10TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM11RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM11RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM11TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM11TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM12RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM12RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM12TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM12TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM13RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM13RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM13TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM13TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM14RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM14RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM14TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM14TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM15RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM15RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM15TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM15TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM16RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM16RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM16TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM16TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM17RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM17RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM17TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM17TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM18RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM18RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM18TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM18TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM19RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM19RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM19TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM19TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM20RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM20RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM20TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM20TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM21RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM21RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM21TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM21TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM22RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM22RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM22TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM22TrnSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM23RcvInSel-SP', 0, 0.0],
-    [':TRIGGER_PM23RcvSrc-Sel', 0, 0.0],
-    [':TRIGGER_PM23TrnOutSel-SP', 0, 0.0],
-    [':TRIGGER_PM23TrnSrc-Sel', 0, 0.0],
-    [':ACQChannel-Sel', 0, 0.0],
-    [':ACQSamplesPre-SP', 100, 0.0],
-    [':ACQSamplesPost-SP', 50, 0.0],
-    [':ACQTrigger-Sel', 1, 0.0],
-    [':ACQShots-SP', 1, 0.0],
-    [':ACQTriggerRep-Sel', 0, 0.0],
-    [':ACQBPMMode-Sel', 1, 0.0],
-    [':TbTTagEn-Sel', 1, 0.0],
-    [':TbTTagDly-SP', 0.0, 0.0],
+    # [':ADC0RstModes-Sel', 1, 0.0],
+    # [':ADC1RstModes-Sel', 1, 0.0],
+    # [':ADC2RstModes-Sel', 1, 0.0],
+    # [':ADC3RstModes-Sel', 1, 0.0],
+    [':TbTPhaseSyncDly-SP', 0.0, 0.0],
     [':TbTDataMaskEn-Sel', 1, 0.0],
     [':TbTDataMaskSamplesBeg-SP', 0.0, 0.0],
     [':TbTDataMaskSamplesEnd-SP', 0.0, 0.0],
-    [':RFFEAtt-SP', 31.5, 0.0],
     [':PosKsum-SP', 1, 0.0],
     [':PosXOffset-SP', 0, 0.0],
     [':PosYOffset-SP', 0, 0.0],
     [':PosQOffset-SP', 0, 0.0],
-    [':ACQTriggerHwDly-SP', 0, 0.0],
-    [':INFOClkFreq-SP', 220910069.0, 0.0],
+    [':GENTriggerHwDly-SP', 0, 0.0],
+    [':PMTriggerHwDly-SP', 0, 0.0],
+    # [':INFOClkFreq-SP', 220910069.0, 0.0],
     [':INFOHarmonicNumber-SP', 864, 0.0],
     [':INFOTbTRate-SP', 382, 0.0],
     [':INFOFOFBRate-SP', 8786, 0.0],
@@ -626,15 +433,12 @@ _bpm_propts = [
     [':PosKq-SP', 21389980, 0.0],
     [':ADCAD9510ADiv-SP', 0, 0.0],
     [':ADCAD9510BDiv-SP', 382, 0.0],
-    [':ADCAD9510Prescaler-SP', 0, 0.0],
     [':ADCAD9510RDiv-SP', 120, 0.0],
     [':ADCSi57xFreq-SP', 220910069.0, 0.0],
-    [':SwMode-Sel', 1, 0.0],
     [':SwDivClk-SP', 8786, 0.0],
-    [':SwDly-SP', 29, 0.0],
-    [':SwTagEn-Sel', 0, 0.0],
-    [':SwDataMaskEn-Sel', 0, 0.0],
-    [':SwDataMaskSamples-SP', 1910, 0.0],
+    [':SwDeswapDly-SP', 29, 0.0],
+    [':FOFBDataMaskEn-Sel', 0, 0.0],
+    [':FOFBDataMaskSamples-SP', 1910, 0.0],
     [':SwDirGainA-SP', 1.0, 0.0],
     [':SwDirGainB-SP', 1.0, 0.0],
     [':SwDirGainC-SP', 1.0, 0.0],
@@ -643,7 +447,30 @@ _bpm_propts = [
     [':SwInvGainB-SP', 1.0, 0.0],
     [':SwInvGainC-SP', 1.0, 0.0],
     [':SwInvGainD-SP', 1.0, 0.0],
+    [':GEN_PolyXArrayCoeff-SP', SI_BPM_POLY_X_COEFF, 0.0],
+    [':GEN_PolyYArrayCoeff-SP', SI_BPM_POLY_Y_COEFF, 0.0],
+    [':GEN_PolySUMArrayCoeff-SP', SI_BPM_POLY_SUM_COEFF, 0.0],
+    [':GEN_PolyQArrayCoeff-SP', SI_BPM_POLY_Q_COEFF, 0.0],
+    [':PM_PolyXArrayCoeff-SP', SI_BPM_POLY_X_COEFF, 0.0],
+    [':PM_PolyYArrayCoeff-SP', SI_BPM_POLY_Y_COEFF, 0.0],
+    [':PM_PolySUMArrayCoeff-SP', SI_BPM_POLY_SUM_COEFF, 0.0],
+    [':PM_PolyQArrayCoeff-SP', SI_BPM_POLY_Q_COEFF, 0.0],
 ]
+for phy_trig in range(AFC_ACQCORE_NR_PHY_TRIGGER):
+    _bpm_propts.extend([
+        [f':TRIGGER{phy_trig}Dir-Sel', 0, 0.0],
+        [f':TRIGGER{phy_trig}DirPol-Sel', 0, 0.0],
+        [f':TRIGGER{phy_trig}RcvLen-SP', 0, 0.0],
+        [f':TRIGGER{phy_trig}TrnLen-SP', 0, 0.0],
+    ])
+for acq_core in ['_GEN', '_PM']:
+    for log_trig in range(AFC_ACQCORE_NR_LOG_TRIGGER):
+        _bpm_propts.extend([
+            [f':TRIGGER{acq_core}{log_trig}RcvInSel-SP', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}RcvSrc-Sel', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}TrnOutSel-SP', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}TrnSrc-Sel', 0, 0.0],
+        ])
 _bpm_pvs = list()
 for dev in _bpms:
     for ppt, val, dly in _bpm_propts:
@@ -1413,6 +1240,50 @@ _tune_pvs = [
     ['SI-Glob:DI-TuneProc-V:FiltType-Sel', 0, 0],
     ]
 
+_fofbctrls = [
+    'IA-01RaBPM:BS-FOFBCtrl',
+    'IA-02RaBPM:BS-FOFBCtrl',
+    'IA-03RaBPM:BS-FOFBCtrl',
+    'IA-04RaBPM:BS-FOFBCtrl',
+    'IA-05RaBPM:BS-FOFBCtrl',
+    'IA-06RaBPM:BS-FOFBCtrl',
+    'IA-07RaBPM:BS-FOFBCtrl',
+    'IA-08RaBPM:BS-FOFBCtrl',
+    'IA-09RaBPM:BS-FOFBCtrl',
+    'IA-10RaBPM:BS-FOFBCtrl',
+    'IA-11RaBPM:BS-FOFBCtrl',
+    'IA-12RaBPM:BS-FOFBCtrl',
+    'IA-13RaBPM:BS-FOFBCtrl',
+    'IA-14RaBPM:BS-FOFBCtrl',
+    'IA-15RaBPM:BS-FOFBCtrl',
+    'IA-16RaBPM:BS-FOFBCtrl',
+    'IA-17RaBPM:BS-FOFBCtrl',
+    'IA-18RaBPM:BS-FOFBCtrl',
+    'IA-19RaBPM:BS-FOFBCtrl',
+    'IA-20RaBPM:BS-FOFBCtrl',
+]
+_fofb_propts = list()
+for phy_trig in range(AFC_ACQCORE_NR_PHY_TRIGGER):
+    _fofb_propts.extend([
+        [f':TRIGGER{phy_trig}Dir-Sel', 0, 0.0],
+        [f':TRIGGER{phy_trig}DirPol-Sel', 0, 0.0],
+        [f':TRIGGER{phy_trig}RcvLen-SP', 0, 0.0],
+        [f':TRIGGER{phy_trig}TrnLen-SP', 0, 0.0],
+    ])
+for acq_core in ['_LAMP', '_SYSID']:
+    for log_trig in range(AFC_ACQCORE_NR_LOG_TRIGGER):
+        _fofb_propts.extend([
+            [f':TRIGGER{acq_core}{log_trig}RcvInSel-SP', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}RcvSrc-Sel', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}TrnOutSel-SP', 0, 0.0],
+            [f':TRIGGER{acq_core}{log_trig}TrnSrc-Sel', 0, 0.0],
+        ])
+_fofb_pvs = list()
+for dev in _fofbctrls:
+    for ppt, val, dly in _fofb_propts:
+        _fofb_pvs.append([dev+ppt, val, dly])
+
+
 # When using this type of configuration to set the machine,
 # the list of PVs should be processed in the same order they are stored
 # in the configuration. The second numeric parameter in the pair is the
@@ -1421,6 +1292,6 @@ _tune_pvs = [
 _template_dict = {
     'pvs':
         _li_diags + _amcfpgaevr_pvs + _bpm_pvs + _scrn_pvs + _vlightcam_pvs +
-        _dcct_pvs + _slit_pvs + _tune_pvs
+        _dcct_pvs + _slit_pvs + _tune_pvs + _fofb_pvs
         # + _ict_pvs  # commented out becaus IOC is down.
         }
