@@ -1,37 +1,72 @@
 """Epics Devices and Device Application."""
 
-import time as _time
-import operator as _opr
 import math as _math
+import operator as _opr
+import time as _time
 from functools import partial as _partial
 
-from epics.ca import ChannelAccessGetFailure as _ChannelAccessGetFailure, \
-    CASeverityException as _CASeverityException
 import numpy as _np
+from epics.ca import CASeverityException as _CASeverityException, \
+    ChannelAccessGetFailure as _ChannelAccessGetFailure
 
 from ..envars import VACA_PREFIX as _VACA_PREFIX
-from ..epics import PV as _PV, CONNECTION_TIMEOUT as _CONN_TIMEOUT, \
-    GET_TIMEOUT as _GET_TIMEOUT
-from ..simul import SimPV as _PVSim
-from ..simul import Simulation as _Simulation
+from ..epics import CONNECTION_TIMEOUT as _CONN_TIMEOUT, \
+    GET_TIMEOUT as _GET_TIMEOUT, PV as _PV
 from ..namesys import SiriusPVName as _SiriusPVName
+from ..simul import SimPV as _PVSim, Simulation as _Simulation
 
 _DEF_TIMEOUT = 10  # s
 _TINY_INTERVAL = 0.050  # s
 
 
 class Device:
-    """Epics Device."""
+    """Epics Device.
+
+    Parameters
+    ----------
+        devname: str
+            Device name, to be used as PVs prefix.
+        props2init: ('all', None or (tuple, list)), optional
+            Define which PVs will be connected in the class instantiation. If
+            equal to 'all', all properties listed in PROPERTIES_DEFAULT will
+            be initialized. If None or empty iterable, no property will be
+            initialized. If iterable of strings, only the properties listed
+            will be initialized. In this last option, it is possible to add
+            properties that are not in PROPERTIES_DEFAULT. The other properties
+            will be created whenever they are needed. Defaults to 'all'.
+        auto_monitor: bool, optional
+            Whether to automatically monitor PVs for changes. Used for PVs
+            that do not end with '-Mon' or 'Data'. Defaults to True.
+        auto_monitor_mon: bool, optional
+            Whether to automatically monitor '-Mon' or 'Data' PVs for changes.
+            Defaults to False (to avoid overloading the client). Set to False
+            when using PV.get_timevars() to know when a PV has been updated.
+
+    """
 
     CONNECTION_TIMEOUT = _CONN_TIMEOUT
     GET_TIMEOUT = _GET_TIMEOUT
-    _properties = ()
+    PROPERTIES_DEFAULT = ()
 
-    def __init__(self, devname, properties, auto_mon=False):
+    PROPERTY_SEP = ':'
+
+    def __init__(
+            self, devname, props2init='all', auto_monitor=True,
+            auto_monitor_mon=False):
         """."""
-        self._properties = properties[:]
-        self._auto_mon = auto_mon
-        self._devname, self._pvs = self._create_pvs(devname)
+        self._devname = _SiriusPVName(devname) if devname else devname
+        self._auto_monitor = auto_monitor
+        self._auto_monitor_mon = auto_monitor_mon
+
+        if isinstance(props2init, str) and props2init.lower() == 'all':
+            propties = self.PROPERTIES_DEFAULT
+        elif not props2init:
+            propties = []
+        elif isinstance(props2init, (list, tuple)):
+            propties = props2init
+        else:
+            raise ValueError('Wrong value for props2init.')
+        self._pvs = {prpt: self._create_pv(prpt) for prpt in propties}
 
     @property
     def devname(self):
@@ -39,9 +74,22 @@ class Device:
         return self._devname
 
     @property
-    def properties(self):
-        """Return device properties."""
-        return self._properties
+    def properties_in_use(self):
+        """Return properties that were already added to the PV list."""
+        return tuple(sorted(self._pvs.keys()))
+
+    @property
+    def properties_added(self):
+        """Return properties that were added to the PV list that are not in
+        PROPERTIES_DEFAULT."""
+        return tuple(sorted(
+            set(self.properties_in_use) - set(self.PROPERTIES_DEFAULT)))
+
+    @property
+    def properties_all(self):
+        """Return all properties of the device, connected or not."""
+        return tuple(sorted(
+            set(self.PROPERTIES_DEFAULT + self.properties_in_use)))
 
     @property
     def simulators(self):
@@ -83,7 +131,15 @@ class Device:
         """Set auto_monitor state of individual PVs."""
         if pvname not in self._pvs:
             return False
-        self._pvs[pvname].auto_monitor = int(value)
+        pvobj = self._pvs[pvname]
+        try:
+            # TODO verify need of int
+            pvobj.auto_monitor = int(value)
+        except (_ChannelAccessGetFailure, _CASeverityException):
+            # exceptions raised in a Virtual Circuit Disconnect (192)
+            # event. If the PV IOC goes down, for example.
+            print('Could not set auto_monitor of {}'.format(pvobj.pvname))
+            return False
         return True
 
     def update(self):
@@ -93,6 +149,10 @@ class Device:
 
     def pv_object(self, propty):
         """Return PV object for a given device property."""
+        if propty not in self._pvs:
+            pvobj = self._create_pv(propty)
+            pvobj.wait_for_connection(Device.CONNECTION_TIMEOUT)
+            self._pvs[propty] = pvobj
         return self._pvs[propty]
 
     def pv_ctrlvars(self, propty):
@@ -100,10 +160,9 @@ class Device:
         return self._pvs[propty].get_ctrlvars()
 
     def pv_attribute_values(self, attribute):
-        """Return property-value dict of a given attribute for all PVs."""
+        """Return pvname-value dict of a given attribute for all PVs."""
         attributes = dict()
-        for propty in self._properties:
-            pvobj = self._pvs[propty]
+        for pvobj in self._pvs.values():
             attributes[pvobj.pvname] = getattr(pvobj, attribute)
         return attributes
 
@@ -127,7 +186,7 @@ class Device:
 
     def __getitem__(self, propty):
         """Return value of property."""
-        pvobj = self._pvs[propty]
+        pvobj = self.pv_object(propty)
         try:
             value = pvobj.get(timeout=Device.GET_TIMEOUT)
         except (_ChannelAccessGetFailure, _CASeverityException):
@@ -139,27 +198,25 @@ class Device:
 
     def __setitem__(self, propty, value):
         """Set value of property."""
-        pvobj = self._pvs[propty]
-        pvobj.value = value
+        pvobj = self.pv_object(propty)
+        try:
+            pvobj.value = value
+        except (_ChannelAccessGetFailure, _CASeverityException):
+            print('Could not set value of {}'.format(pvobj.pvname))
 
     # --- private methods ---
+    def _create_pv(self, propty):
+        pvname = self._get_pvname(propty)
+        auto_monitor = self._auto_monitor
+        if pvname.endswith(('-Mon', 'Data')):
+            auto_monitor = self._auto_monitor_mon
+        in_sim = _Simulation.pv_check(pvname)
+        pvclass = _PVSim if in_sim else _PV
+        return pvclass(
+            pvname, auto_monitor=auto_monitor,
+            connection_timeout=Device.CONNECTION_TIMEOUT)
 
-    def _create_pvs(self, devname):
-        if devname:
-            devname = _SiriusPVName(devname)
-
-        pvs = dict()
-        for propty in self._properties:
-            pvname = self._get_pvname(devname, propty)
-            auto_monitor = self._auto_mon or not pvname.endswith('-Mon')
-            in_sim = _Simulation.pv_check(pvname)
-            pvclass = _PVSim if in_sim else _PV
-            pvs[propty] = pvclass(
-                pvname, auto_monitor=auto_monitor,
-                connection_timeout=Device.CONNECTION_TIMEOUT)
-        return devname, pvs
-
-    def _wait(self, propty, value, timeout=_DEF_TIMEOUT, comp='eq'):
+    def _wait(self, propty, value, timeout=None, comp='eq'):
         """."""
         def comp_(val):
             boo = comp(self[propty], val)
@@ -170,122 +227,82 @@ class Device:
         if isinstance(comp, str):
             comp = getattr(_opr, comp)
 
-        if comp_(value):
-            return True
-
-        ntrials = int(timeout/_TINY_INTERVAL)
-        for _ in range(ntrials):
+        if not isinstance(timeout, str) and timeout != 'never':
+            timeout = _DEF_TIMEOUT if timeout is None else timeout
+            timeout = 0 if timeout <= 0 else timeout
+        t0_ = _time.time()
+        while not comp_(value):
+            if isinstance(timeout, str) and timeout == 'never':
+                pass
+            else:
+                if _time.time() - t0_ > timeout:
+                    return False
             _time.sleep(_TINY_INTERVAL)
-            if comp_(value):
-                return True
-        return False
+        return True
 
     def _wait_float(
-            self, propty, value, rel_tol=0.0, abs_tol=0.1,
-            timeout=_DEF_TIMEOUT):
+            self, propty, value, rel_tol=0.0, abs_tol=0.1, timeout=None):
         """Wait until float value gets close enough of desired value."""
-        func = _partial(_math.isclose, abs_tol=abs_tol, rel_tol=rel_tol)
+        isc = _np.isclose if isinstance(value, _np.ndarray) else _math.isclose
+        func = _partial(isc, abs_tol=abs_tol, rel_tol=rel_tol)
         return self._wait(propty, value, comp=func, timeout=timeout)
 
-    def _get_pvname(self, devname, propty):
-        if devname:
-            func = devname.substitute
-            pvname = func(prefix=_VACA_PREFIX, propty=propty)
+    def _wait_set(self, props_values, timeout=None, comp='eq'):
+        timeout = _DEF_TIMEOUT if timeout is None else timeout
+        t0_ = _time.time()
+        for propty, value in props_values.items():
+            timeout_left = max(0, timeout - (_time.time() - t0_))
+            if timeout_left == 0:
+                return False
+            if not self._wait(propty, value, timeout=timeout_left, comp=comp):
+                return False
+        return True
+
+    def _get_pvname(self, propty):
+        dev = self._devname
+        pref = _VACA_PREFIX + ('-' if _VACA_PREFIX else '')
+        if isinstance(dev, _SiriusPVName) and \
+                dev.is_standard(name_type='devname'):
+            ppt = dev.propty
+            pvname = dev.substitute(prefix=_VACA_PREFIX, propty=ppt + propty)
+        elif dev:
+            pvname = pref + dev + self.PROPERTY_SEP + propty
         else:
-            pvname = _VACA_PREFIX + ('-' if _VACA_PREFIX else '') + propty
+            pvname = pref + propty
         return pvname
 
     def _enum_setter(self, propty, value, enums):
-        if hasattr(enums, '_fields'):
-            enums = enums._fields
-        if value is None:
-            return None
-        elif isinstance(value, str) and value in enums:
-            self[propty] = enums.index(value)
-        elif 0 <= int(value) < len(enums):
+        value = self._enum_selector(value, enums)
+        if value is not None:
             self[propty] = value
 
-
-class ProptyDevice(Device):
-    """Device with a prefix property name."""
-
-    def __init__(self, devname, propty_prefix, properties):
-        """."""
-        self._propty_prefix = propty_prefix
-        # call base class constructor
-        super().__init__(devname, properties=properties)
-
-    def _get_pvname(self, devname, propty):
-        if devname:
-            func = devname.substitute
-            pvname = func(propty=self._propty_prefix + propty)
-        else:
-            pvname = self._propty_prefix + propty
-        return pvname
+    @staticmethod
+    def _enum_selector(value, enums):
+        if value is None:
+            return
+        if hasattr(enums, '_fields'):
+            enums = enums._fields
+        if isinstance(value, str) and value in enums:
+            return enums.index(value)
+        elif 0 <= int(value) < len(enums):
+            return value
 
 
-# NOTE: This class is temporary. It should become deprecated once all
-# devices names are in accordance with Sirius naming system
-class DeviceNC(Device):
-    """Non-compliant Devices.
-
-    This device class is to be used for those devices whose
-    names and PVs are not compliant to the Sirius naming system.
-    """
-    DEVSEP = ':'
-
-    def _create_pvs(self, devname):
-        pvs = dict()
-        devname = devname or ''
-        for propty in self._properties:
-            pvname = devname + self.DEVSEP + propty
-            auto_monitor = not pvname.endswith('-Mon')
-            pvs[propty] = _PV(pvname, auto_monitor=auto_monitor)
-        return devname, pvs
-
-
-class DeviceApp(Device):
-    """Application Device.
-
-    This kind of device groups properties of other devices.
-    """
-
-    def __init__(self, properties, devname=None, auto_mon=False):
-        """."""
-        self._devname_app = devname
-
-        # call base class constructor
-        super().__init__(None, properties=properties, auto_mon=auto_mon)
-
-    @property
-    def devname(self):
-        """Return application device name."""
-        return self._devname_app
-
-
-class Devices:
+class DeviceSet:
     """."""
 
-    def __init__(self, devname, devices):
+    def __init__(self, devices, devname=''):
         """."""
-        self._devname = devname
         self._devices = devices
+        self._devname = _SiriusPVName(devname)
 
-        self._properties = []
-        for dev in self._devices:
-            if dev is not None:
-                self._properties += dev.properties
+    _enum_selector = staticmethod(Device._enum_selector)
 
     @property
     def devname(self):
-        """Return device name."""
+        """Name of the Device set. May be empty in some cases."""
         return self._devname
 
-    @property
-    def properties(self):
-        """Return device properties."""
-        return self._properties
-
     @property
     def simulators(self):
         """Return list of simulators."""
@@ -373,12 +390,14 @@ class Devices:
 
     def _wait_devices_propty(
             self, devices, propty, values, comp='eq',
-            timeout=_DEF_TIMEOUT, return_prob=False):
+            timeout=None, return_prob=False):
         """Wait for devices property to reach value(s)."""
         if isinstance(comp, str):
             comp = getattr(_opr, comp)
         dev2val = self._get_dev_2_val(devices, values)
 
+        timeout = _DEF_TIMEOUT if timeout is None else timeout
+        tini = _time.time()
         for _ in range(int(timeout/_TINY_INTERVAL)):
             okdevs = set()
             for k, v in dev2val.items():
@@ -387,14 +406,18 @@ class Devices:
                     boo = _np.all(boo)
                 if boo:
                     okdevs.add(k)
+                if _time.time() - tini > timeout:
+                    break
             list(map(dev2val.__delitem__, okdevs))
             if not dev2val:
                 break
+            if _time.time() - tini > timeout:
+                break
             _time.sleep(_TINY_INTERVAL)
 
         allok = not dev2val
         if return_prob:
-            return allok, [dev.devname+':'+propty for dev in dev2val]
+            return allok, [dev.pv_object(propty).pvname for dev in dev2val]
         return allok
 
     def _get_dev_2_val(self, devices, values):
