@@ -4,7 +4,7 @@ import numpy as _np
 
 from ..search import BPMSearch as _BPMSearch
 from ..util import ClassProperty as _classproperty
-from .device import Device as _Device, Devices as _Devices
+from .device import Device as _Device, DeviceSet as _DeviceSet
 
 
 class BaseOrbitIntlk:
@@ -88,6 +88,7 @@ class BaseOrbitIntlk:
         self._oper = {
             'mean': self._mean,
             'diff': self._diff,
+            'min': min,
         }
 
     @staticmethod
@@ -114,7 +115,7 @@ class BaseOrbitIntlk:
             func = self._oper[operation]
             val = func(dval, uval)
             data_values.append(val)
-        return data_values
+        return _np.array(data_values)
 
     @staticmethod
     def _mean(var1, var2):
@@ -126,125 +127,124 @@ class BaseOrbitIntlk:
 
 
 class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
-    """This device group the orbit interlock  PVs from one BPM."""
+    """This device group the orbit interlock PVs from one BPM."""
 
-    _properties = (
-        # ==============================================================
-        # Basic properties
-        'PosX-Mon', 'PosY-Mon', 'Sum-Mon',
+    PROPERTIES_DEFAULT = (
         # ==============================================================
         # General
-        # +++++++
+        # **************************************************************
         # General interlock enable:
         'IntlkEn-Sel', 'IntlkEn-Sts',
-        # General interlock clear:
-        'IntlkClr-Cmd',  # maybe -Cmd?
+        # General interlock reset:
+        'IntlkClr-Cmd',
         # Minimum sum threshold enable:
-        # Habilita interlock de órbita apenas quando threshold da soma
-        # ultrapassar o valor em "IntlkLmtMinSum-SP"
+        # Enable orbit interlock only when sum is higher than IntlkLmtMinSum-RB
         'IntlkMinSumEn-Sel', 'IntlkMinSumEn-Sts',
-        # Minimum sum threshold (em contagens da Soma da taxa FAcq):
+        # Minimum sum threshold (sum counts in FAcq rate):
         'IntlkLmtMinSum-SP', 'IntlkLmtMinSum-RB',
-        # Status Instantâneo:
-        # Interlock instântaneo, dificilmente será detectado com
-        # a implementação atual do gateware
+        # Instantaneous interlock, difficult to be checked in the current
+        # gateware implementation
         'Intlk-Mon',
-        # Latch do interlock, limpo apenas acionando-se a PV "Clr"
-        # correspondente
+        # Latch interlock, clean only when respective "Clr" PV is triggered
         'IntlkLtc-Mon',
         # ===============================================================
-        # Position (interlock de posição)
-        # +++++++++++++++++++++++++++++++++++++
+        # Position Interlock
         # ***************************************************************
-        # Condição para interlock de posição:
+        # Condition for position interlock:
         #   thres_min > (pos BPM downstream + pos BPM upstream)/2 or
         #   thres_max < (pos BPM downstream + pos BPM upstream)/2
-        # BPMs são agrupados 2 a 2 seguindo a ordem do feixe:
+        # BPMs are grouped 2 by 2 following the order seen by the beam:
         # - M1/M2
         # - C1-1/C1-2
         # - C2/C3-1
         # - C3-2/C4
-        # BPM upstream é sempre o "primeiro" BPM da dupla acima e BPM
-        # downstream é sempre o "segundo" BPM da dupla.
+        # upstream BPM is always the "first" BPM of the pairs above and
+        # downstream BPM is always the "second" BPM of the pair.
         # ***************************************************************
         # Position interlock enable:
         'IntlkPosEn-Sel', 'IntlkPosEn-Sts',
         # Position interlock clear:
         'IntlkPosClr-Cmd',
-        # Thresholds (em nm da taxa Monit1):
+        # Thresholds (nm, FAcq rate):
         'IntlkLmtPosMaxX-SP', 'IntlkLmtPosMaxX-RB',
         'IntlkLmtPosMinX-SP', 'IntlkLmtPosMinX-RB',
         'IntlkLmtPosMaxY-SP', 'IntlkLmtPosMaxY-RB',
         'IntlkLmtPosMinY-SP', 'IntlkLmtPosMinY-RB',
-        # Todos os interlocks são mascarados pelo "Enable"
-        # Status Instantâneo:
-        'IntlkPosLower-Mon', 'IntlkPosUpper-Mon',  # X ou Y
+        # All interlocks are masked by the "Enable" state
+        # Instantaneous Status:
         'IntlkPosLowerX-Mon', 'IntlkPosUpperX-Mon',  # X
         'IntlkPosLowerY-Mon', 'IntlkPosUpperY-Mon',  # Y
-        # Status Latch, limpo apenas acionando-se a PV "Clr" correspondente:
-        'IntlkPosLowerLtc-Mon', 'IntlkPosUpperLtc-Mon',
+        # Latch Status, clean only when respective "Clr" PV is triggered
         'IntlkPosLowerLtcX-Mon', 'IntlkPosUpperLtcX-Mon',
         'IntlkPosLowerLtcY-Mon', 'IntlkPosUpperLtcY-Mon',
+        # Position measure from orbit interlock core
+        'IntlkPosX-Mon',
+        'IntlkPosY-Mon',
         # =============================================================
-        # Angular (interlock de ângulo)
-        # +++++++++++++++++++++++++++++
+        # Angle Interlock
         # *************************************************************
-        # Condição para interlock de ângulo:
-        #   thres_min > (posição BPM downstream - posição BPM upstream) or
-        #   thres_max < (posição BPM downstream - posição BPM upstream)
-        # BPMs são agrupados 2 a 2 seguindo a ordem do feixe:
+        # Condition for angle interlock:
+        #   thres_min > (pos BPM downstream - pos BPM upstream) or
+        #   thres_max < (pos BPM downstream - pos BPM upstream)
+        # BPMs are grouped 2 by 2 following the order seen by the beam:
         # - M1/M2
         # - C1-1/C1-2
         # - C2/C3-1
         # - C3-2/C4
-        # BPM upstream é sempre o "primeiro" BPM da dupla acima e BPM
-        # downstream é sempre o "segundo" BPM da dupla.
+        # upstream BPM is always the "first" BPM of the pairs above and
+        # downstream BPM is always the "second" BPM of the pair.
         # ************************************************************
-        # Angulation interlock enable:
+        # Angle interlock enable:
         'IntlkAngEn-Sel', 'IntlkAngEn-Sts',
-        # Angulation interlock clear:
+        # Angle interlock clear:
         'IntlkAngClr-Cmd',
-        # Thresholds (em rad.nm da taxa FAcq).
-        # Thresholds devem ser calculados como ângulo (em rad)
-        #  entre os 2 BPMs adjacentes * distância (em nm) entre eles):
+        # Thresholds (rad.nm, FAcq rate).
+        # Thresholds must be calculated as angle (rad)
+        # between the 2 adjacent BPMs * distance (nm) between them:
         'IntlkLmtAngMaxX-SP', 'IntlkLmtAngMaxX-RB',
         'IntlkLmtAngMinX-SP', 'IntlkLmtAngMinX-RB',
         'IntlkLmtAngMaxY-SP', 'IntlkLmtAngMaxY-RB',
         'IntlkLmtAngMinY-SP', 'IntlkLmtAngMinY-RB',
-        # Todos os interlocks são mascarados pelo "Enable"
-        # Status Instantâneo:
-        'IntlkAngLower-Mon', 'IntlkAngUpper-Mon',
+        # All interlocks are masked by the "Enable" state
+        # Intantaneous Status
         'IntlkAngLowerX-Mon', 'IntlkAngUpperX-Mon',  # X
         'IntlkAngLowerY-Mon', 'IntlkAngUpperY-Mon',  # Y
-        # Status Latch, limpo apenas acionando-se a PV "Clr" correspondente:
-        'IntlkAngLowerLtc-Mon', 'IntlkAngUpperLtc-Mon',
+        # Latch Status, clean only when respective "Clr" PV is triggered
         'IntlkAngLowerLtcX-Mon', 'IntlkAngUpperLtcX-Mon',
         'IntlkAngLowerLtcY-Mon', 'IntlkAngUpperLtcY-Mon',
+        # Angle measure from orbit interlock core
+        'IntlkAngX-Mon',
+        'IntlkAngY-Mon',
         # ============================================================
         )
 
-    def __init__(self, devname):
+    def __init__(self, devname, props2init='all'):
         """Init."""
         # call base class constructor
         BaseOrbitIntlk.__init__(self)
         if devname not in self.BPM_NAMES:
             raise ValueError(devname + ' is no a valid BPM name.')
-        _Device.__init__(self, devname, properties=BPMOrbitIntlk._properties)
+        _Device.__init__(self, devname, props2init=props2init)
 
     @property
-    def posx(self):
-        """Position X, Monit rate."""
-        return self['PosX-Mon'] * self.CONV_NM2UM
+    def intlkposx(self):
+        """Orbit interlock core Position X."""
+        return self['IntlkPosX-Mon'] * self.CONV_NM2UM
 
     @property
-    def posy(self):
-        """Position Y, Monit rate."""
-        return self['PosY-Mon'] * self.CONV_NM2UM
+    def intlkposy(self):
+        """Orbit interlock core Position Y."""
+        return self['IntlkPosY-Mon'] * self.CONV_NM2UM
 
     @property
-    def possum(self):
-        """Sum, Monit rate."""
-        return self['Sum-Mon']
+    def intlkangx(self):
+        """Orbit interlock core Angle X."""
+        return self['IntlkAngX-Mon'] * self.CONV_NM2UM
+
+    @property
+    def intlkangy(self):
+        """Orbit interlock core Angle Y."""
+        return self['IntlkAngY-Mon'] * self.CONV_NM2UM
 
     @property
     def pair_down_up_bpms(self):
@@ -280,26 +280,26 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
     # --- minimum sum threshold ---
 
     @property
-    def minsumthres_enable(self):
+    def minsum_enable(self):
         """
         Minimum sum threshold enable.
 
-        If enabled, generate orbit interlock only when sum threshold exceeds
-        value in 'minsumthres' property.
+        If enabled, generate orbit interlock only when sum exceeds
+        value in 'minsum_thres' property.
         """
         return self['IntlkMinSumEn-Sts']
 
-    @minsumthres_enable.setter
-    def minsumthres_enable(self, value):
+    @minsum_enable.setter
+    def minsum_enable(self, value):
         self['IntlkMinSumEn-Sel'] = int(value)
 
     @property
-    def minsumthres(self):
+    def minsum_thres(self):
         """Minimum sum threshold [sum count, FAcq rate]."""
         return self['IntlkLmtMinSum-RB']
 
-    @minsumthres.setter
-    def minsumthres(self, value):
+    @minsum_thres.setter
+    def minsum_thres(self, value):
         self['IntlkLmtMinSum-SP'] = int(value)
 
     # --- position interlock ---
@@ -319,57 +319,41 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
         return True
 
     @property
-    def pos_thresminx(self):
+    def pos_x_min_thres(self):
         """Minimum X position threshold."""
         return self['IntlkLmtPosMinX-RB']
 
-    @pos_thresminx.setter
-    def pos_thresminx(self, value):
+    @pos_x_min_thres.setter
+    def pos_x_min_thres(self, value):
         self['IntlkLmtPosMinX-SP'] = value
 
     @property
-    def pos_thresmaxx(self):
+    def pos_x_max_thres(self):
         """Maximum X position threshold."""
         return self['IntlkLmtPosMaxX-RB']
 
-    @pos_thresmaxx.setter
-    def pos_thresmaxx(self, value):
+    @pos_x_max_thres.setter
+    def pos_x_max_thres(self, value):
         self['IntlkLmtPosMaxX-SP'] = value
 
     @property
-    def pos_thresminy(self):
+    def pos_y_min_thres(self):
         """Minimum Y position threshold."""
         return self['IntlkLmtPosMinY-RB']
 
-    @pos_thresminy.setter
-    def pos_thresminy(self, value):
+    @pos_y_min_thres.setter
+    def pos_y_min_thres(self, value):
         self['IntlkLmtPosMinY-SP'] = value
 
     @property
-    def pos_thresmaxy(self):
+    def pos_y_max_thres(self):
         """Maximum Y position threshold."""
         return self['IntlkLmtPosMaxY-RB']
 
-    @pos_thresmaxy.setter
-    def pos_thresmaxy(self, value):
+    @pos_y_max_thres.setter
+    def pos_y_max_thres(self, value):
         self['IntlkLmtPosMaxY-SP'] = value
 
-    @property
-    def pos_inst_lower(self):
-        """
-        Instantaneous position interlock set when either X or Y
-        minimum thresholds are exceeded.
-        """
-        return self['IntlkPosLower-Mon']
-
-    @property
-    def pos_inst_upper(self):
-        """
-        Instantaneous position interlock set when either X or Y
-        maximum thresholds are exceeded.
-        """
-        return self['IntlkPosUpper-Mon']
-
     @property
     def pos_inst_lower_x(self):
         """
@@ -402,22 +386,6 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
         """
         return self['IntlkPosUpperY-Mon']
 
-    @property
-    def pos_latch_lower(self):
-        """
-        Latch position interlock set when either X or Y
-        minimum thresholds are exceeded.
-        """
-        return self['IntlkPosLowerLtc-Mon']
-
-    @property
-    def pos_latch_upper(self):
-        """
-        Latch position interlock set when either X or Y
-        maximum thresholds are exceeded.
-        """
-        return self['IntlkPosUpperLtc-Mon']
-
     @property
     def pos_latch_lower_x(self):
         """
@@ -467,57 +435,41 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
         return True
 
     @property
-    def ang_thresminx(self):
+    def ang_x_min_thres(self):
         """Minimum X angulation threshold."""
         return self['IntlkLmtAngMinX-RB']
 
-    @ang_thresminx.setter
-    def ang_thresminx(self, value):
+    @ang_x_min_thres.setter
+    def ang_x_min_thres(self, value):
         self['IntlkLmtAngMinX-SP'] = value
 
     @property
-    def ang_thresmaxx(self):
+    def ang_x_max_thres(self):
         """Maximum X angulation threshold."""
         return self['IntlkLmtAngMaxX-RB']
 
-    @ang_thresmaxx.setter
-    def ang_thresmaxx(self, value):
+    @ang_x_max_thres.setter
+    def ang_x_max_thres(self, value):
         self['IntlkLmtAngMaxX-SP'] = value
 
     @property
-    def ang_thresminy(self):
+    def ang_y_min_thres(self):
         """Minimum Y angulation threshold."""
         return self['IntlkLmtAngMinY-RB']
 
-    @ang_thresminy.setter
-    def ang_thresminy(self, value):
+    @ang_y_min_thres.setter
+    def ang_y_min_thres(self, value):
         self['IntlkLmtAngMinY-SP'] = value
 
     @property
-    def ang_thresmaxy(self):
+    def ang_y_max_thres(self):
         """Maximum Y angulation threshold."""
         return self['IntlkLmtAngMaxY-RB']
 
-    @ang_thresmaxy.setter
-    def ang_thresmaxy(self, value):
+    @ang_y_max_thres.setter
+    def ang_y_max_thres(self, value):
         self['IntlkLmtAngMaxY-SP'] = value
 
-    @property
-    def ang_inst_lower(self):
-        """
-        Instantaneous angulation interlock set when either X or Y
-        minimum thresholds are exceeded.
-        """
-        return self['IntlkAngLower-Mon']
-
-    @property
-    def ang_inst_upper(self):
-        """
-        Instantaneous angulation interlock set when either X or Y
-        maximum thresholds are exceeded.
-        """
-        return self['IntlkAngUpper-Mon']
-
     @property
     def ang_inst_lower_x(self):
         """
@@ -550,22 +502,6 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
         """
         return self['IntlkAngUpperY-Mon']
 
-    @property
-    def ang_latch_lower(self):
-        """
-        Latch angulation interlock set when either X or Y
-        minimum thresholds are exceeded.
-        """
-        return self['IntlkAngLowerLtc-Mon']
-
-    @property
-    def ang_latch_upper(self):
-        """
-        Latch angulation interlock set when either X or Y
-        maximum thresholds are exceeded.
-        """
-        return self['IntlkAngUpperLtc-Mon']
-
     @property
     def ang_latch_lower_x(self):
         """
@@ -599,7 +535,7 @@ class BPMOrbitIntlk(BaseOrbitIntlk, _Device):
         return self['IntlkAngUpperLtcY-Mon']
 
 
-class OrbitInterlock(BaseOrbitIntlk, _Devices):
+class OrbitInterlock(BaseOrbitIntlk, _DeviceSet):
     """Orbit Interlock device."""
 
     TIMEOUT = 10
@@ -609,31 +545,52 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         SI = 'SI-Fam:DI-BPM'
         ALL = (SI, )
 
-    def __init__(self, devname=None):
+    def __init__(self, devname=None, props2init='all'):
         """Init."""
         if devname is None:
             devname = self.DEVICES.SI
         if devname not in self.DEVICES.ALL:
             raise ValueError('Wrong value for devname')
         BaseOrbitIntlk.__init__(self)
-        devs = [BPMOrbitIntlk(dev) for dev in self.BPM_NAMES]
-        _Devices.__init__(self, devname, devs)
+        devs = [
+            BPMOrbitIntlk(dev, props2init=props2init)
+            for dev in self.BPM_NAMES]
+        _DeviceSet.__init__(self, devs, devname=devname)
 
     # --- general interlock ---
 
-    def cmd_gen_enable(self, timeout=TIMEOUT):
+    @property
+    def gen_enable(self):
+        """General interlock enable.
+
+        Returns:
+            enbl (numpy.ndarray, 160):
+                enable state for each BPM.
+        """
+        return _np.array([b.gen_enable for b in self._devices])
+
+    def set_gen_enable(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set enable state for BPM general interlock."""
+        self._set_devices_propty(self.devices, 'IntlkEn-Sel', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkEn-Sts', value, timeout=timeout,
+            return_prob=return_prob)
+
+    def cmd_gen_enable(self, timeout=TIMEOUT, return_prob=False):
         """Enable all BPM general interlock."""
         for dev in self.devices:
             dev.gen_enable = 1
         return self._wait_devices_propty(
-            self.devices, 'IntlkEn-Sts', 1, timeout=timeout)
+            self.devices, 'IntlkEn-Sts', 1, timeout=timeout,
+            return_prob=return_prob)
 
-    def cmd_gen_disable(self, timeout=TIMEOUT):
+    def cmd_gen_disable(self, timeout=TIMEOUT, return_prob=False):
         """Disable all BPM general interlock."""
         for dev in self.devices:
             dev.gen_enable = 0
         return self._wait_devices_propty(
-            self.devices, 'IntlkEn-Sts', 0, timeout=timeout)
+            self.devices, 'IntlkEn-Sts', 0, timeout=timeout,
+            return_prob=return_prob)
 
     def cmd_reset_gen(self):
         """Reset all BPM general interlock."""
@@ -663,50 +620,99 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
 
     # --- minimum sum threshold ---
 
-    def cmd_minsumthres_enable(self, timeout=TIMEOUT):
+    @property
+    def minsum_enable(self):
+        """Minimum sum threshold enable.
+
+        Returns:
+            enbl (numpy.ndarray, 160):
+                enable state for each BPM.
+        """
+        return _np.array([b.minsum_enable for b in self._devices])
+
+    def set_minsum_enable(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set enable state for BPM minimum sum interlock."""
+        self._set_devices_propty(self.devices, 'IntlkMinSumEn-Sel', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkMinSumEn-Sts', value, timeout=timeout,
+            return_prob=return_prob)
+
+    def cmd_minsum_enable(self, timeout=TIMEOUT, return_prob=False):
         """Enable all BPM minimum sum threshold."""
         for dev in self.devices:
-            dev.minsumthres_enable = 1
+            dev.minsum_enable = 1
         return self._wait_devices_propty(
-            self.devices, 'IntlkMinSumEn-Sts', 1, timeout=timeout)
+            self.devices, 'IntlkMinSumEn-Sts', 1, timeout=timeout,
+            return_prob=return_prob)
 
-    def cmd_minsumthres_disable(self, timeout=TIMEOUT):
+    def cmd_minsum_disable(self, timeout=TIMEOUT, return_prob=False):
         """Disable all BPM minimum sum threshold."""
         for dev in self.devices:
-            dev.minsumthres_enable = 0
+            dev.minsum_enable = 0
         return self._wait_devices_propty(
-            self.devices, 'IntlkMinSumEn-Sts', 0, timeout=timeout)
+            self.devices, 'IntlkMinSumEn-Sts', 0, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def minsumthres(self):
+    def minsum_thres(self):
         """Minimum sum thresholds.
 
         Returns:
             thres (numpy.ndarray, 160): min.sum threshold for each BPM.
         """
-        return _np.array([b.minsumthres for b in self._devices])
+        return _np.array([b.minsum_thres for b in self._devices])
 
-    @minsumthres.setter
-    def minsumthres(self, value):
+    def set_minsum_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set minimum sum thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.minsumthres = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtMinSum-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtMinSum-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     # --- position interlock ---
 
-    def cmd_pos_enable(self, timeout=TIMEOUT):
+    @property
+    def pos_enable(self):
+        """Position interlock enable.
+
+        Returns:
+            enbl (numpy.ndarray, 160):
+                enable state for each BPM.
+        """
+        return _np.array([b.pos_enable for b in self._devices])
+
+    def set_pos_enable(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set enable state for BPM position interlock."""
+        self._set_devices_propty(self.devices, 'IntlkPosEn-Sel', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkPosEn-Sts', value, timeout=timeout,
+            return_prob=return_prob)
+
+    def cmd_pos_enable(self, timeout=TIMEOUT, return_prob=False):
         """Enable all BPM position interlock."""
         for dev in self.devices:
             dev.pos_enable = 1
         return self._wait_devices_propty(
-            self.devices, 'IntlkPosEn-Sts', 1, timeout=timeout)
+            self.devices, 'IntlkPosEn-Sts', 1, timeout=timeout,
+            return_prob=return_prob)
 
-    def cmd_pos_disable(self, timeout=TIMEOUT):
+    def cmd_pos_disable(self, timeout=TIMEOUT, return_prob=False):
         """Disable all BPM position interlock."""
         for dev in self.devices:
             dev.pos_enable = 0
         return self._wait_devices_propty(
-            self.devices, 'IntlkPosEn-Sts', 0, timeout=timeout)
+            self.devices, 'IntlkPosEn-Sts', 0, timeout=timeout,
+            return_prob=return_prob)
 
     def cmd_reset_pos(self):
         """Reset all BPM position interlock."""
@@ -715,88 +721,112 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         return True
 
     @property
-    def pos_thresminx(self):
+    def pos_x_min_thres(self):
         """Minimum x position thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 min. x position threshold for each BPM.
         """
-        return _np.array([b.pos_thresminx for b in self._devices])
+        return _np.array([b.pos_x_min_thres for b in self._devices])
 
-    @pos_thresminx.setter
-    def pos_thresminx(self, value):
+    def set_pos_x_min_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set minimum x position thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.pos_thresminx = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtPosMinX-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtPosMinX-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def pos_thresmaxx(self):
+    def pos_x_max_thres(self):
         """Maximum x position thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 max. x position threshold for each BPM.
         """
-        return _np.array([b.pos_thresmaxx for b in self._devices])
+        return _np.array([b.pos_x_max_thres for b in self._devices])
 
-    @pos_thresmaxx.setter
-    def pos_thresmaxx(self, value):
+    def set_pos_x_max_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set maximum x position thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.pos_thresmaxx = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtPosMaxX-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtPosMaxX-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def pos_thresminy(self):
+    def pos_y_min_thres(self):
         """Minimum y position thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 min. y position threshold for each BPM.
         """
-        return _np.array([b.pos_thresminy for b in self._devices])
+        return _np.array([b.pos_y_min_thres for b in self._devices])
 
-    @pos_thresminy.setter
-    def pos_thresminy(self, value):
+    def set_pos_y_min_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set minimum y position thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.pos_thresminy = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtPosMinY-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtPosMinY-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def pos_thresmaxy(self):
+    def pos_y_max_thres(self):
         """Maximum y position thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 max. y position threshold for each BPM.
         """
-        return _np.array([b.pos_thresmaxy for b in self._devices])
+        return _np.array([b.pos_y_max_thres for b in self._devices])
 
-    @pos_thresmaxy.setter
-    def pos_thresmaxy(self, value):
+    def set_pos_y_max_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set maximum y position thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.pos_thresmaxy = value[idx]
-
-    @property
-    def pos_inst_lower(self):
-        """Instantaneous position interlock set when either X or Y
-        minimum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.pos_inst_lower for b in self._devices])
-
-    @property
-    def pos_inst_upper(self):
-        """Instantaneous position interlock set when either X or Y
-        maximum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.pos_inst_upper for b in self._devices])
+        self._set_devices_propty(self.devices, 'IntlkLmtPosMaxY-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtPosMaxY-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
     def pos_inst_lower_x(self):
@@ -838,26 +868,6 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         """
         return _np.array([b.pos_inst_upper_y for b in self._devices])
 
-    @property
-    def pos_latch_lower(self):
-        """Latch position interlock set when either X or Y
-        minimum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.pos_latch_lower for b in self._devices])
-
-    @property
-    def pos_latch_upper(self):
-        """Latch position interlock set when either X or Y
-        maximum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.pos_latch_upper for b in self._devices])
-
     @property
     def pos_latch_lower_x(self):
         """Latch position interlock set when X
@@ -900,19 +910,38 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
 
     #  --- angulation interlock ---
 
-    def cmd_ang_enable(self, timeout=TIMEOUT):
+    @property
+    def ang_enable(self):
+        """Angle interlock enable.
+
+        Returns:
+            enbl (numpy.ndarray, 160):
+                enable state for each BPM.
+        """
+        return _np.array([b.ang_enable for b in self._devices])
+
+    def set_ang_enable(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set enable state for BPM angulation interlock."""
+        self._set_devices_propty(self.devices, 'IntlkAngEn-Sel', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkAngEn-Sts', value, timeout=timeout,
+            return_prob=return_prob)
+
+    def cmd_ang_enable(self, timeout=TIMEOUT, return_prob=False):
         """Enable all BPM angulation interlock."""
         for dev in self.devices:
             dev.ang_enable = 1
         return self._wait_devices_propty(
-            self.devices, 'IntlkAngEn-Sts', 1, timeout=timeout)
+            self.devices, 'IntlkAngEn-Sts', 1, timeout=timeout,
+            return_prob=return_prob)
 
-    def cmd_ang_disable(self, timeout=TIMEOUT):
+    def cmd_ang_disable(self, timeout=TIMEOUT, return_prob=False):
         """Disable all BPM angulation interlock."""
         for dev in self.devices:
             dev.ang_enable = 0
         return self._wait_devices_propty(
-            self.devices, 'IntlkAngEn-Sts', 0, timeout=timeout)
+            self.devices, 'IntlkAngEn-Sts', 0, timeout=timeout,
+            return_prob=return_prob)
 
     def cmd_reset_ang(self):
         """Reset all BPM angulation interlock."""
@@ -921,88 +950,112 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         return True
 
     @property
-    def ang_thresminx(self):
+    def ang_x_min_thres(self):
         """Minimum x angulation thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 min. x angulation threshold for each BPM.
         """
-        return _np.array([b.ang_thresminx for b in self._devices])
+        return _np.array([b.ang_x_min_thres for b in self._devices])
 
-    @ang_thresminx.setter
-    def ang_thresminx(self, value):
+    def set_ang_x_min_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set minimum x angle thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.ang_thresminx = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtAngMinX-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtAngMinX-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def ang_thresmaxx(self):
+    def ang_x_max_thres(self):
         """Maximum x angulation thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 max. x angulation threshold for each BPM.
         """
-        return _np.array([b.ang_thresmaxx for b in self._devices])
+        return _np.array([b.ang_x_max_thres for b in self._devices])
 
-    @ang_thresmaxx.setter
-    def ang_thresmaxx(self, value):
+    def set_ang_x_max_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set maximum x angle thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.ang_thresmaxx = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtAngMaxX-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtAngMaxX-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def ang_thresminy(self):
+    def ang_y_min_thres(self):
         """Minimum y angulation thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 min. y angulation threshold for each BPM.
         """
-        return _np.array([b.ang_thresminy for b in self._devices])
+        return _np.array([b.ang_y_min_thres for b in self._devices])
 
-    @ang_thresminy.setter
-    def ang_thresminy(self, value):
+    def set_ang_y_min_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set minimum y angle thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.ang_thresminy = value[idx]
+        self._set_devices_propty(self.devices, 'IntlkLmtAngMinY-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtAngMinY-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
-    def ang_thresmaxy(self):
+    def ang_y_max_thres(self):
         """Maximum y angulation thresholds.
 
         Returns:
             thres (numpy.ndarray, 160):
                 max. y angulation threshold for each BPM.
         """
-        return _np.array([b.ang_thresmaxy for b in self._devices])
+        return _np.array([b.ang_y_max_thres for b in self._devices])
 
-    @ang_thresmaxy.setter
-    def ang_thresmaxy(self, value):
+    def set_ang_y_max_thres(self, value, timeout=TIMEOUT, return_prob=False):
+        """Set maximum y angle thresholds.
+
+        Args:
+            value (numpy.ndarray|float): Values for minimum sum.
+            timeout (float, optional): timeout to wait. Defaults to TIMEOUT.
+
+        Returns:
+            bool: True if set was successful.
+
+        """
         value = self._handle_thres_input(value)
-        for idx, dev in enumerate(self.devices):
-            dev.ang_thresmaxy = value[idx]
-
-    @property
-    def ang_inst_lower(self):
-        """Instantaneous angulation interlock set when either X or Y
-        minimum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.ang_inst_lower for b in self._devices])
-
-    @property
-    def ang_inst_upper(self):
-        """Instantaneous angulation interlock set when either X or Y
-        maximum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.ang_inst_upper for b in self._devices])
+        self._set_devices_propty(self.devices, 'IntlkLmtAngMaxY-SP', value)
+        return self._wait_devices_propty(
+            self.devices, 'IntlkLmtAngMaxY-RB', value, timeout=timeout,
+            return_prob=return_prob)
 
     @property
     def ang_inst_lower_x(self):
@@ -1044,26 +1097,6 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         """
         return _np.array([b.ang_inst_upper_y for b in self._devices])
 
-    @property
-    def ang_latch_lower(self):
-        """Latch angulation interlock set when either X or Y
-        minimum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.ang_latch_lower for b in self._devices])
-
-    @property
-    def ang_latch_upper(self):
-        """Latch angulation interlock set when either X or Y
-        maximum thresholds are exceeded.
-
-        Returns:
-            intlk (numpy.ndarray, 160): interlock status for each BPM.
-        """
-        return _np.array([b.ang_latch_upper for b in self._devices])
-
     @property
     def ang_latch_lower_x(self):
         """Latch angulation interlock set when X
@@ -1104,32 +1137,6 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
         """
         return _np.array([b.ang_latch_upper_y for b in self._devices])
 
-    @property
-    def slow_orbit(self):
-        """Slow orbit vectors.
-
-        Returns:
-            orbx (numpy.ndarray, 160): Horizontal Orbit.
-            orby (numpy.ndarray, 160): Vertical Orbit.
-
-        """
-        orbx, orby = [], []
-        for bpm in self._devices:
-            orbx.append(bpm.posx)
-            orby.append(bpm.posy)
-        orbx = _np.array(orbx)
-        orby = _np.array(orby)
-        return orbx, orby
-
-    @property
-    def possum(self):
-        """Sum vector, at Monit rate.
-
-        Returns:
-            possum (numpy.ndarray, 160): Sum vector, at Monit rate.
-        """
-        return _np.array([b.possum for b in self._devices])
-
     @property
     def position(self):
         """Position vectors.
@@ -1141,25 +1148,23 @@ class OrbitInterlock(BaseOrbitIntlk, _Devices):
             posx (numpy.ndarray, 160): Horizontal Position.
             posy (numpy.ndarray, 160): Vertical Position.
         """
-        orbx, orby = self.slow_orbit
-        posx = _np.array(self.calc_intlk_metric(orbx, metric='pos'))
-        posy = _np.array(self.calc_intlk_metric(orby, metric='pos'))
+        posx = _np.array([b.intlkposx for b in self._devices])
+        posy = _np.array([b.intlkposy for b in self._devices])
         return posx, posy
 
     @property
-    def angulation(self):
-        """Angulation vectors.
+    def angle(self):
+        """Angle vectors.
 
-        Angulation at each BPM is defined as:
+        Angle at each BPM is defined as:
             (posição BPM downstream - posição BPM upstream)
 
         Returns:
-            angx (numpy.ndarray, 160): Horizontal Angulation.
-            angy (numpy.ndarray, 160): Vertical Angulation.
+            angx (numpy.ndarray, 160): Horizontal Angle.
+            angy (numpy.ndarray, 160): Vertical Angle.
         """
-        orbx, orby = self.slow_orbit
-        angx = _np.array(self.calc_intlk_metric(orbx, metric='ang'))
-        angy = _np.array(self.calc_intlk_metric(orby, metric='ang'))
+        angx = _np.array([b.intlkangx for b in self._devices])
+        angy = _np.array([b.intlkangy for b in self._devices])
         return angx, angy
 
     def _handle_thres_input(self, value):
