@@ -10,8 +10,10 @@ from ..pwrsupply.csdev import Const as _Const, \
     MAX_WFMSIZE_FBP as _MAX_WFMSIZE_FBP, \
     MAX_WFMSIZE_OTHERS as _MAX_WFMSIZE_OTHERS
 from ..pwrsupply.psctrl.pscstatus import PSCStatus as _PSCStatus
+from ..magnet.factory import NormalizerFactory as _NormFactory
 
 from .device import Device as _Device
+from .timing import Trigger as _Trigger
 
 
 class _PSDev(_Device):
@@ -33,15 +35,21 @@ class _PSDev(_Device):
         'CycleType-Sel', 'CycleType-Sts',
         'CycleNrCycles-SP', 'CycleNrCycles-RB',
         'Wfm-SP', 'Wfm-RB', 'WfmRef-Mon', 'Wfm-Mon',
+        'ScopeDuration-SP', 'ScopeDuration-RB',
+        'ScopeFreq-SP', 'ScopeFreq-RB',
         'CycleFreq-SP', 'CycleFreq-RB',
         'CycleAmpl-SP', 'CycleAmpl-RB',
         'CycleOffset-SP', 'CycleOffset-RB',
         'CycleAuxParam-SP', 'CycleAuxParam-RB',
         'CycleEnbl-Mon',
     )
+    _properties_fbp = _properties_magps + (
+        'SOFBMode-Sel', 'SOFBMode-Sts',
+        'IDFFMode-Sel', 'IDFFMode-Sts',
+        )
     _properties_fc = (
         'AlarmsAmp-Mon', 'OpMode-Sel', 'OpMode-Sts',
-        'CurrLoopKp-RB', 'CurrLoopKp-SP', 'CurrLoopTi-RB', 'CurrLoopTi-SP',
+        'CurrLoopKp-RB', 'CurrLoopKp-SP', 'CurrLoopKi-RB', 'CurrLoopKi-SP',
         'CurrLoopMode-Sts', 'CurrLoopMode-Sel',
         'CurrGain-RB', 'CurrGain-SP', 'CurrOffset-RB', 'CurrOffset-SP',
         'Current-RB', 'Current-SP', 'Current-Mon', 'CurrentRef-Mon',
@@ -49,12 +57,17 @@ class _PSDev(_Device):
         'TestWavePeriod-RB', 'TestWavePeriod-SP',
         'Voltage-RB', 'Voltage-SP', 'Voltage-Mon',
         'VoltGain-RB', 'VoltGain-SP', 'VoltOffset-RB', 'VoltOffset-SP',
-        'InvRespMatRow-SP', 'InvRespMatRow-RB',
+        'InvRespMatRowX-SP', 'InvRespMatRowX-RB',
+        'InvRespMatRowY-SP', 'InvRespMatRowY-RB',
         'FOFBAccGain-SP', 'FOFBAccGain-RB',
         'FOFBAccFreeze-Sel', 'FOFBAccFreeze-Sts',
         'FOFBAccClear-Cmd',
         'FOFBAccSatMax-SP', 'FOFBAccSatMax-RB',
         'FOFBAccSatMin-SP', 'FOFBAccSatMin-RB',
+        'FOFBAcc-Mon',
+        'FOFBAccDecimation-SP', 'FOFBAccDecimation-RB',
+        'FOFBAccFilter-SP', 'FOFBAccFilter-RB',
+        'FOFBAccFilterGain-SP', 'FOFBAccFilterGain-RB',
     )
     _properties_pulsed = (
         'Voltage-SP', 'Voltage-RB', 'Voltage-Mon',
@@ -69,7 +82,7 @@ class _PSDev(_Device):
         'CCoilVVoltage-SP', 'CCoilVVoltage-RB', 'CCoilVVoltage-Mon',
     )
 
-    def __init__(self, devname):
+    def __init__(self, devname, auto_monitor_mon=False, props2init='all'):
         """."""
         devname = _SiriusPVName(devname)
 
@@ -80,7 +93,7 @@ class _PSDev(_Device):
         # power supply type and magnetic function
         (self._pstype, self._psmodel, self._magfunc,
          self._strength_propty, self._strength_units,
-         self._is_linac, self._is_pulsed, self._is_fc,
+         self._is_linac, self._is_pulsed, self._is_fc, self._is_fbp,
          self._is_magps) = _PSDev.get_device_type(devname)
 
         # set attributes
@@ -89,12 +102,24 @@ class _PSDev(_Device):
          self._strength_mon_propty,
          properties) = self._set_attributes_properties(devname)
 
-        # call base class constructor
-        super().__init__(devname, properties=properties)
+        if props2init == 'all':
+            props2init = properties
+        super().__init__(
+            devname, props2init=props2init, auto_monitor_mon=auto_monitor_mon)
 
         # private attribute with strength setpoint pv object
         self._strength_sp_pv = self.pv_object(self._strength_sp_propty)
 
+        try:
+            name = devname.substitute(dis='MA')
+            if name.dev == 'B1B2' or (name.sec == 'BO' and name.dev == 'B'):
+                maname = name.substitute(idx='')
+            else:
+                maname = name
+            self._normalizer = _NormFactory.create(maname)
+        except:
+            self._normalizer = None
+
     @property
     def pstype(self):
         """Return type of magnet(s) excited by power supply device."""
@@ -130,6 +155,11 @@ class _PSDev(_Device):
         """Return True if device is a Sirius magnet power supply."""
         return self._is_pulsed
 
+    @property
+    def normalizer(self):
+        """Return Normalizer object for current and strength conversions."""
+        return self._normalizer
+
     @property
     def strength_property(self):
         """Return Strength name."""
@@ -238,10 +268,11 @@ class _PSDev(_Device):
         is_linac = devname.sec.endswith('LI')
         is_pulsed = devname.dis == 'PU'
         is_fc = devname.dev == 'FCH' or devname.dev == 'FCV'
-        is_magps = not is_linac and not is_pulsed and not is_fc
+        is_fbp = psmodel == 'FBP'
+        is_magps = not is_linac and not is_pulsed and not is_fc and not is_fbp
         return (pstype, psmodel, magfunc,
                 strength_propty, strength_units,
-                is_linac, is_pulsed, is_fc, is_magps)
+                is_linac, is_pulsed, is_fc, is_fbp, is_magps)
 
     # --- private methods ---
 
@@ -260,6 +291,8 @@ class _PSDev(_Device):
                 properties += _PSDev._properties_pulsed_nlkckr
         elif self._is_fc:
             properties += _PSDev._properties_fc
+        elif self._is_fbp:
+            properties += _PSDev._properties_fbp
         else:
             properties += _PSDev._properties_magps
 
@@ -555,6 +588,26 @@ class PowerSupply(_PSDev):
         self._enum_setter(
             'WfmUpdateAuto-Sel', value, self.WFMUPDATEAUTO)
 
+    @property
+    def scope_freq(self):
+        """Scope frequency [Hz]."""
+        return self['ScopeFreq-RB']
+
+    @scope_freq.setter
+    def scope_freq(self, value):
+        """Set scope frequency [Hz]."""
+        self['ScopeFreq-SP'] = float(value)
+
+    @property
+    def scope_duration(self):
+        """Scope duration [s]."""
+        return self['ScopeDuration-RB']
+
+    @scope_duration.setter
+    def scope_duration(self, value):
+        """Set scope duration [s]."""
+        self['ScopeDuration-SP'] = float(value)
+
     def cmd_slowref(self, timeout=_PSDev._default_timeout):
         """."""
         self['OpMode-Sel'] = self.OPMODE_SEL.SlowRef
@@ -603,18 +656,18 @@ class PowerSupplyPU(_PSDev):
             TS_INJ_SEPTF,
             SI_INJ_DPKCKR, SI_INJ_NLKCKR,
             SI_PING_H, SI_PING_V,
-        )
+            )
 
     _properties_timing = ('Delay-SP', 'Delay-RB', 'DelayRaw-SP', 'DelayRaw-RB')
 
-    def __init__(self, devname):
+    def __init__(self, devname, props2init='all'):
         """."""
         # check if device exists
         if devname not in PowerSupplyPU.DEVICES.ALL:
             raise NotImplementedError(devname)
 
         # call base class constructor
-        super().__init__(devname)
+        super().__init__(devname, props2init=props2init)
 
         # create timing device
         self._dev_timing = self._create_timing_device()
@@ -705,7 +758,7 @@ class PowerSupplyPU(_PSDev):
     @property
     def properties(self):
         """Return device properties."""
-        return self._properties + self._dev_timing.properties
+        return self.properties_in_use + self._dev_timing.properties_in_use
 
     @property
     def pvnames(self):
@@ -716,7 +769,7 @@ class PowerSupplyPU(_PSDev):
     @property
     def interlock_ok(self):
         """Return whether all interlocks are in Ok state."""
-        intlks = [p for p in self._properties if 'Intlk' in p]
+        intlks = [p for p in self.properties_in_use if 'Intlk' in p]
         is_ok = True
         for ilk in intlks:
             is_ok &= self[ilk] == 1
@@ -771,8 +824,7 @@ class PowerSupplyPU(_PSDev):
     def _create_timing_device(self):
         """."""
         devname = self._devname.substitute(dis='TI')
-        device = _Device(devname, PowerSupplyPU._properties_timing)
-        return device
+        return _Trigger(devname, props2init=PowerSupplyPU._properties_timing)
 
 
 class PowerSupplyFC(_PSDev):
@@ -875,13 +927,13 @@ class PowerSupplyFC(_PSDev):
         self['CurrLoopKp-SP'] = value
 
     @property
-    def currloop_ti(self):
-        """Current control loop Ti parameter."""
-        return self['CurrLoopTi-RB']
+    def currloop_ki(self):
+        """Current control loop Ki parameter."""
+        return self['CurrLoopKi-RB']
 
-    @currloop_ti.setter
-    def currloop_ti(self, value):
-        self['CurrLoopTi-SP'] = value
+    @currloop_ki.setter
+    def currloop_ki(self, value):
+        self['CurrLoopKi-SP'] = value
 
     @property
     def currloop_mode(self):
@@ -910,13 +962,22 @@ class PowerSupplyFC(_PSDev):
         return self._wait('OpMode-Sts', mode, timeout=timeout)
 
     @property
-    def invrespmat_row(self):
-        """Correction coefficient value."""
-        return self['InvRespMatRow-RB']
+    def invrespmat_row_x(self):
+        """Horizontal correction coefficient value."""
+        return self['InvRespMatRowX-RB']
 
-    @invrespmat_row.setter
-    def invrespmat_row(self, value):
-        self['InvRespMatRow-SP'] = _np.array(value, dtype=float)
+    @invrespmat_row_x.setter
+    def invrespmat_row_x(self, value):
+        self['InvRespMatRowX-SP'] = _np.array(value, dtype=float)
+
+    @property
+    def invrespmat_row_y(self):
+        """Vertical correction coefficient value."""
+        return self['InvRespMatRowY-RB']
+
+    @invrespmat_row_y.setter
+    def invrespmat_row_y(self, value):
+        self['InvRespMatRowY-SP'] = _np.array(value, dtype=float)
 
     @property
     def fofbacc_gain(self):
@@ -954,7 +1015,88 @@ class PowerSupplyFC(_PSDev):
     def fofbacc_satmin(self, value):
         self['FOFBAccSatMin-SP'] = value
 
+    @property
+    def fofbacc_mon(self):
+        """FOFB accumulator."""
+        return self['FOFBAcc-Mon']
+
+    @property
+    def fofbacc_decimation(self):
+        """FOFB accumulator decimation."""
+        return self['FOFBAccDecimation-RB']
+
+    @fofbacc_decimation.setter
+    def fofbacc_decimation(self, value):
+        self['FOFBAccDecimation-SP'] = value
+
     def cmd_fofbacc_clear(self):
         """Command to clear FOFB accumulator."""
         self['FOFBAccClear-Cmd'] = 1
         return True
+
+    @property
+    def fofbacc_filter(self):
+        """FOFB accumulator filter."""
+        return self['FOFBAccFilter-RB']
+
+    @fofbacc_filter.setter
+    def fofbacc_filter(self, value):
+        self['FOFBAccFilter-SP'] = value
+
+    @property
+    def fofbacc_filter_gain(self):
+        """FOFB accumulator filter gain."""
+        return self['FOFBAccFilterGain-RB']
+
+    @fofbacc_filter_gain.setter
+    def fofbacc_filter_gain(self, value):
+        self['FOFBAccFilterGain-SP'] = value
+
+
+class PowerSupplyFBP(PowerSupply):
+    """FBP Power Supply Device."""
+
+    SOFBMODE_SEL = _Const.DsblEnbl
+    SOFBMODE_STS = _Const.DsblEnbl
+    IDFFMODE_SEL = _Const.DsblEnbl
+    IDFFMODE_STS = _Const.DsblEnbl
+
+    @property
+    def sofbmode(self):
+        """SOFB mode status."""
+        return self['SOFBMode-Sts']
+
+    @property
+    def idffmode(self):
+        """IDFF mode status."""
+        return self['IDFFMode-Sts']
+
+    def cmd_sofbmode_enable(self, timeout=_PSDev._default_timeout):
+        """Command to enable SOFBMode. Send command and wait."""
+        return self._cmd_sofbmode(
+            timeout, self.SOFBMODE_SEL.Enbl, self.SOFBMODE_STS.Enbl)
+
+    def cmd_sofbmode_disable(self, timeout=_PSDev._default_timeout):
+        """Command to disable SOFBMode. Send command and wait."""
+        return self._cmd_sofbmode(
+            timeout, self.SOFBMODE_SEL.Dsbl, self.SOFBMODE_STS.Dsbl)
+
+    def _cmd_sofbmode(self, timeout, state_sel, state_sts):
+        self['SOFBMode-Sel'] = state_sel
+        return self._wait(
+            'SOFBMode-Sts', state_sts, timeout=timeout)
+
+    def cmd_idffmode_enable(self, timeout=_PSDev._default_timeout):
+        """Command to enable IDFFMode. Send command and wait."""
+        return self._cmd_idffmode(
+            timeout, self.IDFFMODE_SEL.Enbl, self.IDFFMODE_STS.Enbl)
+
+    def cmd_idffmode_disable(self, timeout=_PSDev._default_timeout):
+        """Command to disable IDFFMode. Send command and wait."""
+        return self._cmd_idffmode(
+            timeout, self.IDFFMODE_SEL.Dsbl, self.IDFFMODE_STS.Dsbl)
+
+    def _cmd_idffmode(self, timeout, state_sel, state_sts):
+        self['IDFFMode-Sel'] = state_sel
+        return self._wait(
+            'IDFFMode-Sts', state_sts, timeout=timeout)
