@@ -5,10 +5,10 @@ import numpy as _np
 
 from mathphys.functions import get_namedtuple as _get_namedtuple
 
-from .device import DeviceNC as _DeviceNC, Devices as _Devices
+from .device import Device as _Device, DeviceSet as _DeviceSet
 
 
-class RFGen(_DeviceNC):
+class RFGen(_Device):
     """Wrap the basic features of the RF Generator used in Sirius.
 
     The Generator is the R&S SMA100A Signal Generator.
@@ -66,7 +66,7 @@ class RFGen(_DeviceNC):
         SPARE = 'RF-SI-Gen'
         ALL = (AS, SPARE)
 
-    _properties = (
+    PROPERTIES_DEFAULT = (
         'GeneralFreq-SP', 'GeneralFreq-RB',
 
         'FreqFExeSweep-Cmd',  # execute single sweep
@@ -90,7 +90,7 @@ class RFGen(_DeviceNC):
         'FreqFDwellTime-RB', 'FreqFDwellTime-SP',
         )
 
-    def __init__(self, devname=None):
+    def __init__(self, devname=None, props2init='all'):
         """."""
         if devname is None:
             devname = RFGen.DEVICES.AS
@@ -98,9 +98,7 @@ class RFGen(_DeviceNC):
         # check if device exists
         if devname not in RFGen.DEVICES.ALL:
             raise NotImplementedError(devname)
-
-        # call base class constructor
-        super().__init__(devname, properties=RFGen._properties)
+        super().__init__(devname, props2init=props2init)
 
     @property
     def is_phase_continuous(self):
@@ -123,10 +121,11 @@ class RFGen(_DeviceNC):
             return
         npoints = int(delta/delta_max) + 2
         freq_span = _np.linspace(freq0, value, npoints)[1:]
-        self._pvs['GeneralFreq-SP'].put(freq_span[0], wait=False)
+        pvo = self.pv_object('GeneralFreq-SP')
+        pvo.put(freq_span[0], wait=False)
         for freq in freq_span[1:]:
             _time.sleep(1.0)
-            self._pvs['GeneralFreq-SP'].put(freq, wait=False)
+            pvo.put(freq, wait=False)
         self['GeneralFreq-SP'] = value
 
     def set_frequency(self, value, tol=1, timeout=10):
@@ -564,8 +563,16 @@ class RFGen(_DeviceNC):
         return ampl
 
 
-class ASLLRF(_DeviceNC):
+class ASLLRF(_Device):
     """AS LLRF."""
+
+    PROPERTIES_INTERLOCK = (
+        'FIMLLRF1-Sel', 'FIMLLRF1-Sts', 'IntlkAll-Mon',
+        'FIMManual-Sel', 'FIMManual-Sts',
+        'IntlkManual-Sel', 'IntlkManual-Sts', 'IntlkReset-Cmd',
+        'Inp1Intlk-Mon', 'Inp2Intlk-Mon',
+    )
+
     VoltIncRates = _get_namedtuple('VoltIncRates', (
         'vel_0p01', 'vel_0p03', 'vel_0p1', 'vel_0p25', 'vel_0p5', 'vel_1p0',
         'vel_2p0', 'vel_4p0', 'vel_6p0', 'vel_8p0', 'vel_10p0', 'vel_15p0',
@@ -577,31 +584,75 @@ class ASLLRF(_DeviceNC):
     class DEVICES:
         """Devices names."""
 
-        BO = 'BR-RF-DLLRF-01'
-        SI = 'SR-RF-DLLRF-01'
-        ALL = (BO, SI)
+        BO = 'RA-RaBO01:RF-LLRF'
+        SIA = 'RA-RaSIA01:RF-LLRF'
+        SIB = 'RA-RaSIB01:RF-LLRF'
+        ALL = (BO, SIA, SIB)
 
-    _properties = (
-        'PL:REF', 'PL:REF:S', 'SL:REF:PHS', 'SL:INP:PHS',
-        'mV:AL:REF-SP', 'mV:AL:REF-RB', 'SL:REF:AMP', 'SL:INP:AMP',
-        'DTune-SP', 'DTune-RB', 'TUNE:DEPHS',
-        'RmpPhsBot-SP', 'RmpPhsBot-RB', 'RmpPhsTop-SP', 'RmpPhsTop-RB',
-        'RmpEnbl-Sel', 'RmpEnbl-Sts', 'RmpReady-Mon',
-        'FF:ON', 'FF', 'FF:S', 'FF:POS', 'FF:POS:S',
-        'FF:GAIN:CELL2', 'FF:GAIN:CELL2:S', 'FF:GAIN:CELL4', 'FF:GAIN:CELL4:S',
-        'FF:DEADBAND', 'FF:DEADBAND:S', 'FF:CELL2', 'FF:CELL4', 'FF:ERR',
-        'AMPREF:INCRATE', 'AMPREF:INCRATE:S',
-        'PHSREF:INCRATE', 'PHSREF:INCRATE:S',
-        )
-
-    def __init__(self, devname):
+    def __new__(cls, devname, props2init='all'):
         """."""
         # check if device exists
         if devname not in ASLLRF.DEVICES.ALL:
             raise NotImplementedError(devname)
 
-        # call base class constructor
-        super().__init__(devname, properties=ASLLRF._properties)
+        if 'SI' in devname:
+            obj = _SILLRF(devname, props2init)
+        elif 'BO' in devname:
+            obj = _BOLLRF(devname, props2init)
+
+        obj.system_nickname = cls.get_system_nickname(devname)
+        return obj
+
+    @staticmethod
+    def get_system_nickname(devname):
+        """."""
+        if devname == ASLLRF.DEVICES.BO:
+            return "BO"
+        if devname == ASLLRF.DEVICES.SIA:
+            return "SI-A"
+        if devname == ASLLRF.DEVICES.SIB:
+            return "SI-B"
+        return ""
+
+
+class _BaseLLRF(_Device):
+    """Base LLRF."""
+
+    PROPERTIES_DEFAULT = ASLLRF.PROPERTIES_INTERLOCK + (
+        'SL-Sel', 'SL-Sts',
+        'PLRef-RB', 'PLRef-SP', 'SLRefPhs-Mon', 'SLInpPhs-Mon',
+        'ALRef-SP', 'ALRef-RB', 'SLRefAmp-Mon', 'SLInpAmp-Mon',
+        'Detune-SP', 'Detune-RB', 'TuneDephs-Mon',
+        'RmpPhsBot-SP', 'RmpPhsBot-RB', 'RmpPhsTop-SP', 'RmpPhsTop-RB',
+        'RmpEnbl-Sel', 'RmpEnbl-Sts', 'RmpReady-Mon',
+        'AmpIncRate-RB', 'AmpIncRate-SP',
+        'PhsIncRate-RB', 'PhsIncRate-SP',
+        'AmpRefMin-RB', 'AmpRefMin-SP', 'PhsRefMin-RB', 'PhsRefMin-SP',
+        'CondEnbl-Sts', 'CondEnbl-Sel', 'CondDuty-RB', 'CondDuty-SP',
+        'CondDutyCycle-Mon', 'PhShCav-SP', 'PhShCav-RB',
+        'SLPILim-SP', 'SLPILim-RB', 'SLKI-SP', 'SLKI-RB', 'SLKP-SP', 'SLKI-RB',
+    )
+
+    def __init__(self, devname, props2init='all'):
+        """."""
+        # check if device exists
+        if devname not in ASLLRF.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    @property
+    def slow_loop_state(self):
+        """Slow loop state."""
+        return self['SL-Sts']
+
+    @slow_loop_state.setter
+    def slow_loop_state(self, value):
+        self['SL-Sel'] = int(value)
+
+    def set_slow_loop_state(self, value, timeout=None):
+        """Wait for slow loop state to reach `value`."""
+        self.slow_loop_state = value
+        return self._wait('SL-Sts', value, timeout=timeout)
 
     @property
     def is_cw(self):
@@ -622,6 +673,15 @@ class ASLLRF(_DeviceNC):
         """Ramp ready."""
         return self['RmpReady-Mon']
 
+    def set_rmp_enable(self, value, timeout=None, wait_ready=False):
+        """Set ramp enable."""
+        self.rmp_enable = value
+        if not self._wait('RmpEnbl-Sts', value, timeout=timeout):
+            return False
+        if wait_ready:
+            return self._wait('RmpReady-Mon', value, timeout=timeout)
+        return True
+
     @property
     def phase_top(self):
         """."""
@@ -643,174 +703,368 @@ class ASLLRF(_DeviceNC):
     @property
     def phase_mon(self):
         """."""
-        return self['SL:INP:PHS']
+        return self['SLInpPhs-Mon']
 
     @property
     def phase_ref(self):
         """."""
-        return self['SL:REF:PHS']
+        return self['SLRefPhs-Mon']
+
+    @property
+    def phase_sp(self):
+        """."""
+        return self['PLRef-SP']
 
     @property
     def phase(self):
         """."""
-        return self['PL:REF']
+        return self['PLRef-RB']
 
     @phase.setter
     def phase(self, value):
-        self['PL:REF:S'] = self._wrap_phase(value)
+        self['PLRef-SP'] = self._wrap_phase(value)
 
     @property
     def phase_incrate_str(self):
         """."""
-        return self.PhsIncRates._fields[self['PHSREF:INCRATE']]
+        return ASLLRF.PhsIncRates._fields[self['PhsIncRate-RB']]
 
     @property
     def phase_incrate(self):
         """."""
-        return self['PHSREF:INCRATE']
+        return self['PhsIncRate-RB']
 
     @phase_incrate.setter
     def phase_incrate(self, value):
-        if int(value) in self.PhsIncRates:
-            self['PHSREF:INCRATE:S'] = int(value)
+        self._enum_setter('PhsIncRate-SP', value, ASLLRF.PhsIncRates)
 
     def set_phase(self, value, tol=0.2, timeout=10, wait_mon=False):
         """Set RF phase and wait until it gets there."""
         self.phase = value
-        pv2wait = 'SL:INP:PHS' if wait_mon else 'SL:REF:PHS'
+        pv2wait = 'SLInpPhs-Mon' if wait_mon else 'SLRefPhs-Mon'
         return self._wait_float(pv2wait, value, abs_tol=tol, timeout=timeout)
 
+    def phase_shift_cav_sp(self):
+        """."""
+        return self['PhShCav-SP']
+
+    @property
+    def phase_shift_cav(self):
+        """."""
+        return self['PhShCav-RB']
+
+    @phase_shift_cav.setter
+    def phase_shift_cav(self, value):
+        self['PhShCav-SP'] = self._wrap_phase(value)
+
+    @property
+    def phase_refmin_sp(self):
+        """."""
+        return self['PhsRefMin-SP']
+
+    @property
+    def phase_refmin(self):
+        """."""
+        return self['PhsRefMin-RB']
+
+    @phase_refmin.setter
+    def phase_refmin(self, value):
+        self['PhsRefMin-SP'] = value
+
     @property
     def voltage_mon(self):
         """."""
-        return self['SL:INP:AMP']
+        return self['SLInpAmp-Mon']
 
     @property
     def voltage_ref(self):
         """."""
-        return self['SL:REF:AMP']
+        return self['SLRefAmp-Mon']
+
+    @property
+    def voltage_sp(self):
+        """."""
+        return self['ALRef-SP']
 
     @property
     def voltage(self):
         """."""
-        return self['mV:AL:REF-RB']
+        return self['ALRef-RB']
 
     @voltage.setter
     def voltage(self, value):
-        self['mV:AL:REF-SP'] = value
+        self['ALRef-SP'] = value
 
     @property
     def voltage_incrate_str(self):
-        """."""
-        return self.VoltIncRates._fields[self['AMPREF:INCRATE']]
+        """Voltage increase rate enum string."""
+        return ASLLRF.VoltIncRates._fields[self['AmpIncRate-RB']]
 
     @property
     def voltage_incrate(self):
-        """."""
-        return self['AMPREF:INCRATE']
+        """Voltage increase rate."""
+        return self['AmpIncRate-RB']
 
     @voltage_incrate.setter
     def voltage_incrate(self, value):
-        if int(value) in self.VoltIncRates:
-            self['AMPREF:INCRATE:S'] = int(value)
+        self._enum_setter('AmpIncRate-SP', value, ASLLRF.VoltIncRates)
+
+    def set_voltage_incrate(self, value, timeout=None):
+        """Set and wait voltage increase rate to reach `value`."""
+        self.voltage_incrate = value
+        return self._wait('AmpIncRate-RB', value, timeout=timeout)
 
     def set_voltage(self, value, tol=1, timeout=10, wait_mon=False):
-        """Set RF phase and wait until it gets there."""
+        """Set RF voltage and wait until it gets there."""
         self.voltage = value
-        pv2wait = 'SL:INP:AMP' if wait_mon else 'SL:REF:AMP'
+        pv2wait = 'SLInpAmp-Mon' if wait_mon else 'SLRefAmp-Mon'
+        return self._wait_float(pv2wait, value, abs_tol=tol, timeout=timeout)
+
+    @property
+    def voltage_refmin_sp(self):
+        """."""
+        return self['AmpRefMin-SP']
+
+    @property
+    def voltage_refmin(self):
+        """."""
+        return self['AmpRefMin-RB']
+
+    @voltage_refmin.setter
+    def voltage_refmin(self, value):
+        self['AmpRefMin-SP'] = value
+
+    @property
+    def conditioning_state(self):
+        """."""
+        return self['CondEnbl-Sts']
+
+    @conditioning_state.setter
+    def conditioning_state(self, value):
+        self['CondEnbl-Sel'] = bool(value)
+
+    def cmd_turn_on_conditioning(self, timeout=10):
+        """Turn on conditioning mode."""
+        self.conditioning_state = 1
+        return self._wait('CondEnbl-Sts', 1, timeout=timeout)
+
+    def cmd_turn_off_conditioning(self, timeout=10):
+        """Turn off conditioning mode."""
+        self.conditioning_state = 0
+        return self._wait('CondEnbl-Sts', 0, timeout=timeout)
+
+    @property
+    def conditioning_duty_cycle_mon(self):
+        """Duty cycle in %."""
+        return self['CondDutyCycle-Mon']
+
+    @property
+    def conditioning_duty_cycle(self):
+        """Duty cycle in %."""
+        return self['CondDuty-RB']
+
+    @conditioning_duty_cycle.setter
+    def conditioning_duty_cycle(self, value):
+        self['CondDuty-SP'] = value
+
+    def set_duty_cycle(self, value, tol=1, timeout=10, wait_mon=True):
+        """Set RF phase and wait until it gets there."""
+        self.conditioning_duty_cycle = value
+        pv2wait = 'CondDuty-RB' if wait_mon else 'CondDutyCycle-Mon'
         return self._wait_float(pv2wait, value, abs_tol=tol, timeout=timeout)
 
     @property
     def detune(self):
         """."""
-        return self['DTune-RB']
+        return self['Detune-RB']
 
     @detune.setter
     def detune(self, value):
-        self['DTune-SP'] = value
+        self['Detune-SP'] = value
 
     @property
     def detune_error(self):
         """."""
-        return self['TUNE:DEPHS']
-
-    @property
-    def field_flatness_acting(self):
-        """Return whether the field flatness loop is acting."""
-        return self['FF:ON']
-
-    @property
-    def field_flatness_enable(self):
-        """Return whether the field flatness loop is enabled."""
-        return self['FF']
-
-    @field_flatness_enable.setter
-    def field_flatness_enable(self, value):
-        """Control state of field flatness loop."""
-        self['FF:S'] = bool(value)
-
-    @property
-    def field_flatness_position(self):
-        """Return the field flatness position."""
-        return self['FF:POS']
-
-    @field_flatness_position.setter
-    def field_flatness_position(self, value):
-        """Control the field flatness position."""
-        self['FF:POS:S'] = bool(value)
-
-    @property
-    def field_flatness_gain1(self):
-        """Return the gain of the first cell controlled."""
-        return self['FF:GAIN:CELL2']
-
-    @field_flatness_gain1.setter
-    def field_flatness_gain1(self, value):
-        """Control the gain of the first cell controlled."""
-        self['FF:GAIN:CELL2:S'] = value
-
-    @property
-    def field_flatness_gain2(self):
-        """Return the gain of the second cell controle."""
-        return self['FF:GAIN:CELL4']
-
-    @field_flatness_gain2.setter
-    def field_flatness_gain2(self, value):
-        """Control the gain of the second cell controlled."""
-        self['FF:GAIN:CELL4:S'] = value
-
-    @property
-    def field_flatness_deadband(self):
-        """Return the loop action deadband in [%]."""
-        return self['FF:DEADBAND']
-
-    @field_flatness_deadband.setter
-    def field_flatness_deadband(self, value):
-        """Control the loop action deadband in [%]."""
-        self['FF:DEADBAND:S'] = value
-
-    @property
-    def field_flatness_amp1(self):
-        """Amplitude of the first cell controled in [mV]."""
-        return self['FF:CELL2']
-
-    @property
-    def field_flatness_amp2(self):
-        """Amplitude of the second cell controled in [mV]."""
-        return self['FF:CELL4']
-
-    @property
-    def field_flatness_error(self):
-        """Return the amplitude error in [mV]."""
-        return self['FF:ERR']
+        return self['TuneDephs-Mon']
 
     @staticmethod
     def _wrap_phase(phase):
         """Phase must be in [-180, +180] interval."""
         return (phase + 180) % 360 - 180
 
+    # interlock properties
+    @property
+    def fast_interlock_monitor_orbit(self):
+        """."""
+        return self['FIMLLRF1-Sts']
 
-class BORFCavMonitor(_DeviceNC):
+    @fast_interlock_monitor_orbit.setter
+    def fast_interlock_monitor_orbit(self, value):
+        """."""
+        self['FIMLLRF1-Sel'] = value
+
+    @property
+    def fast_interlock_monitor_manual(self):
+        """."""
+        return self['FIMManual-Sts']
+
+    @fast_interlock_monitor_manual.setter
+    def fast_interlock_monitor_manual(self, value):
+        """."""
+        self['FIMManual-Sel'] = value
+
+    @property
+    def interlock_manual(self):
+        """."""
+        return self['IntlkManual-Sts']
+
+    @interlock_manual.setter
+    def interlock_manual(self, value):
+        """."""
+        self['IntlkManual-Sel'] = value
+
+    @property
+    def interlock_mon(self):
+        """."""
+        return self['IntlkAll-Mon']
+
+    @property
+    def interlock_input1_mon(self):
+        """."""
+        return self['Inp1Intlk-Mon']
+
+    @property
+    def interlock_input2_mon(self):
+        """."""
+        return self['Inp2Intlk-Mon']
+
+    def cmd_reset_interlock(self, wait=1):
+        """Reset interlocks."""
+        self['IntlkReset-Cmd'] = 1
+        _time.sleep(wait)
+        self['IntlkReset-Cmd'] = 0
+
+    def loop_pi_limit_sp(self):
+        return self['SLPILim-SP']
+
+    @property
+    def loop_pi_limit(self):
+        return self['SLPILim-RB']
+
+    @loop_pi_limit.setter
+    def loop_pi_limit(self, value):
+        self['SLPILim-SP'] = value
+
+    def loop_ki_sp(self):
+        return self['SLKI-SP']
+
+    @property
+    def loop_ki(self):
+        return self['SLKI-RB']
+
+    @loop_ki.setter
+    def loop_ki(self, value):
+        self['SLKI-SP'] = value
+
+    def loop_kp_sp(self):
+        return self['SLKP-SP']
+
+    @property
+    def loop_kp(self):
+        return self['SLKP-RB']
+
+    @loop_kp.setter
+    def loop_kp(self, value):
+        self['SLKP-SP'] = value
+
+
+class _BOLLRF(_BaseLLRF):
+    """."""
+
+    # fieldflatness properties that are exclusive for P5Cav at BO
+    PROPERTIES_DEFAULT = _BaseLLRF.PROPERTIES_DEFAULT + (
+        'FFOn-Mon', 'FFEnbl-Sts', 'FFEnbl-Sel', 'FFDir-Sts', 'FFDir-Sel',
+        'FFGainCell2-RB', 'FFGainCell2-SP', 'FFGainCell4-RB', 'FFGainCell4-SP',
+        'FFDeadBand-RB', 'FFDeadBand-SP', 'FFCell2-Mon', 'FFCell4-Mon',
+        'FFError-Mon',
+        )
+
+    @property
+    def field_flatness_acting(self):
+        """Return whether the field flatness loop is acting."""
+        return self['FFOn-Mon']
+
+    @property
+    def field_flatness_enable(self):
+        """Return whether the field flatness loop is enabled."""
+        return self['FFEnbl-Sts']
+
+    @field_flatness_enable.setter
+    def field_flatness_enable(self, value):
+        """Control state of field flatness loop."""
+        self['FFEnbl-Sel'] = bool(value)
+
+    @property
+    def field_flatness_position(self):
+        """Return the field flatness position."""
+        return self['FFDir-Sts']
+
+    @field_flatness_position.setter
+    def field_flatness_position(self, value):
+        """Control the field flatness position."""
+        self['FFDir-Sel'] = bool(value)
+
+    @property
+    def field_flatness_gain1(self):
+        """Return the gain of the first cell controlled."""
+        return self['FFGainCell2-RB']
+
+    @field_flatness_gain1.setter
+    def field_flatness_gain1(self, value):
+        """Control the gain of the first cell controlled."""
+        self['FFGainCell2-SP'] = value
+
+    @property
+    def field_flatness_gain2(self):
+        """Return the gain of the second cell controle."""
+        return self['FFGainCell4-RB']
+
+    @field_flatness_gain2.setter
+    def field_flatness_gain2(self, value):
+        """Control the gain of the second cell controlled."""
+        self['FFGainCell4-SP'] = value
+
+    @property
+    def field_flatness_deadband(self):
+        """Return the loop action deadband in [%]."""
+        return self['FFDeadBand-RB']
+
+    @field_flatness_deadband.setter
+    def field_flatness_deadband(self, value):
+        """Control the loop action deadband in [%]."""
+        self['FFDeadBand-SP'] = value
+
+    @property
+    def field_flatness_amp1(self):
+        """Amplitude of the first cell controled in [mV]."""
+        return self['FFCell2-Mon']
+
+    @property
+    def field_flatness_amp2(self):
+        """Amplitude of the second cell controled in [mV]."""
+        return self['FFCell4-Mon']
+
+    @property
+    def field_flatness_error(self):
+        """Return the amplitude error in [mV]."""
+        return self['FFError-Mon']
+
+
+class _SILLRF(_BaseLLRF):
+    pass
+
+
+class BORFCavMonitor(_Device):
     """."""
 
     class DEVICES:
@@ -818,19 +1072,18 @@ class BORFCavMonitor(_DeviceNC):
 
         BO = 'BO-05D:RF-P5Cav'
 
-    _properties = (
-        'Cell3PwrTop-Mon', 'Cell3PwrBot-Mon', 'PwrRFIntlk-Mon', 'Sts-Mon',
-        'Cell1Pwr-Mon', 'Cell2Pwr-Mon', 'Cell3Pwr-Mon', 'Cell4Pwr-Mon',
-        'Cell5Pwr-Mon', 'Cylin1T-Mon', 'Cylin2T-Mon', 'Cylin3T-Mon',
+    PROPERTIES_DEFAULT = (
+        'FwdPwrW-Mon', 'RevPwrW-Mon',
+        'Cell3TopPwrW-Mon', 'Cell3BotPwrW-Mon', 'PwrRFIntlk-Mon', 'Sts-Mon',
+        'Cell1PwrW-Mon', 'Cell2PwrW-Mon', 'Cell3PwrW-Mon', 'Cell4PwrW-Mon',
+        'Cell5PwrW-Mon', 'Cylin1T-Mon', 'Cylin2T-Mon', 'Cylin3T-Mon',
         'Cylin4T-Mon', 'Cylin5T-Mon', 'CoupT-Mon',
-        'RmpAmpVCavBot-Mon', 'RmpAmpVCavTop-Mon',
+        'Cell3BotVGap-Mon', 'Cell3TopVGap-Mon',
         )
 
-    def __init__(self):
+    def __init__(self, props2init='all'):
         """."""
-        # call base class constructor
-        super().__init__(
-            BORFCavMonitor.DEVICES.BO, properties=BORFCavMonitor._properties)
+        super().__init__(BORFCavMonitor.DEVICES.BO, props2init=props2init)
 
     @property
     def status(self):
@@ -844,48 +1097,48 @@ class BORFCavMonitor(_DeviceNC):
 
     @property
     def power_top(self):
-        """."""
-        return self['Cell3PwrTop-Mon']
+        """Power at Top in [W]."""
+        return self['Cell3TopPwrW-Mon']
 
     @property
     def power_bottom(self):
-        """."""
-        return self['Cell3PwrBot-Mon']
+        """Power at Bottom in [W]."""
+        return self['Cell3BotPwrW-Mon']
 
     @property
     def power_reverse(self):
-        """."""
-        return self['PwrRev-Mon']
+        """Reverse power in [W]."""
+        return self['RevPwrW-Mon']
 
     @property
     def power_forward(self):
-        """."""
-        return self['PwrFwd-Mon']
+        """Forward power in [W]."""
+        return self['FwdPwrW-Mon']
 
     @property
     def power_cell1(self):
-        """."""
-        return self['Cell1Pwr-Mon']
+        """Power on Cell1 in [W]."""
+        return self['Cell1PwrW-Mon']
 
     @property
     def power_cell2(self):
-        """."""
-        return self['Cell2Pwr-Mon']
+        """Power on Cell2 in [W]."""
+        return self['Cell2PwrW-Mon']
 
     @property
     def power_cell3(self):
-        """."""
-        return self['Cell3Pwr-Mon']
+        """Power on Cell3 in [W]."""
+        return self['Cell3PwrW-Mon']
 
     @property
     def power_cell4(self):
-        """."""
-        return self['Cell4Pwr-Mon']
+        """Power on Cell4 in [W]."""
+        return self['Cell4PwrW-Mon']
 
     @property
     def power_cell5(self):
-        """."""
-        return self['Cell5Pwr-Mon']
+        """Power on Cell5 in [W]."""
+        return self['Cell5PwrW-Mon']
 
     @property
     def temp_coupler(self):
@@ -920,154 +1173,153 @@ class BORFCavMonitor(_DeviceNC):
     @property
     def gap_voltage_bottom(self):
         """Gap Voltage in [V]."""
-        return self['RmpAmpVCavBot-Mon']
+        return self['Cell3BotVGap-Mon']
 
     @property
     def gap_voltage_top(self):
         """Gap Voltage in [V]."""
-        return self['RmpAmpVCavTop-Mon']
+        return self['Cell3TopVGap-Mon']
 
 
-class SIRFCavMonitor(_DeviceNC):
-    """."""
+class SIRFCavMonitor(_Device):
+    """TODO: UPDATE WITH PVS FOR SC CAVITIES."""
 
     class DEVICES:
         """Devices names."""
 
-        SI = 'SI-02SB:RF-P7Cav'
+        SIA = 'SI-03SP:RF-SRFCav-A'
+        SIB = 'SI-03SP:RF-SRFCav-B'
+        ALL = (SIA, SIB, )
 
-    _properties = (
-        'PwrRev-Mon', 'PwrFwd-Mon',
-        'PwrCell4Top-Mon', 'PwrCell4Bot-Mon', 'PwrRFIntlk-Mon', 'Sts-Mon',
-        'PwrCell2-Mon', 'PwrCell4-Mon', 'PwrCell6-Mon', 'Cylin1T-Mon',
-        'Cylin2T-Mon', 'Cylin3T-Mon', 'Cylin4T-Mon', 'Cylin5T-Mon',
-        'Cylin6T-Mon', 'Cylin7T-Mon', 'CoupT-Mon', 'AmpVCav-Mon',
+    PROPERTIES_DEFAULT = (
+        'PwrW-Mon', 'PwrdBm-Mon', 'Amp-Mon',
+        'FwdPwrW-Mon', 'FwdPwrdBm-Mon', 'FwdAmp-Mon',
+        'RevPwrW-Mon', 'RevPwrdBm-Mon', 'RevAmp-Mon',
+        'TunerMoveDown-Mon', 'TunerMoveUp-Mon', 'VGap-Mon',
         )
 
-    def __init__(self):
+    def __init__(self, devname, props2init='all'):
         """."""
-        # call base class constructor
-        super().__init__(
-            SIRFCavMonitor.DEVICES.SI, properties=SIRFCavMonitor._properties)
+        if devname not in SIRFCavMonitor.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
 
     @property
-    def status(self):
+    def system_nickname(self):
         """."""
-        return self['Sts-Mon']
-
-    @property
-    def power_interlock(self):
-        """."""
-        return self['PwrRFIntlk-Mon']
-
-    @property
-    def power_top(self):
-        """."""
-        return self['PwrCell4Top-Mon']
-
-    @property
-    def power_bottom(self):
-        """."""
-        return self['PwrCell4Bot-Mon']
-
-    @property
-    def power_reverse(self):
-        """."""
-        return self['PwrRev-Mon']
-
-    @property
-    def power_forward(self):
-        """."""
-        return self['PwrFwd-Mon']
-
-    @property
-    def power_cell2(self):
-        """."""
-        return self['PwrCell2-Mon']
-
-    @property
-    def power_cell4(self):
-        """."""
-        return self['PwrCell4-Mon']
-
-    @property
-    def power_cell6(self):
-        """."""
-        return self['PwrCell6-Mon']
-
-    @property
-    def temp_coupler(self):
-        """."""
-        return self['CoupT-Mon']
-
-    @property
-    def temp_cell1(self):
-        """."""
-        return self['Cylin1T-Mon']
-
-    @property
-    def temp_cell2(self):
-        """."""
-        return self['Cylin2T-Mon']
-
-    @property
-    def temp_cell3(self):
-        """."""
-        return self['Cylin3T-Mon']
-
-    @property
-    def temp_cell4(self):
-        """."""
-        return self['Cylin4T-Mon']
-
-    @property
-    def temp_cell5(self):
-        """."""
-        return self['Cylin5T-Mon']
-
-    @property
-    def temp_cell6(self):
-        """."""
-        return self['Cylin6T-Mon']
-
-    @property
-    def temp_cell7(self):
-        """."""
-        return self['Cylin7T-Mon']
+        if self.devname == ASLLRF.DEVICES.SIA:
+            return "A"
+        if self.devname == ASLLRF.DEVICES.SIB:
+            return "B"
+        return ""
 
     @property
     def gap_voltage(self):
-        """."""
-        return self['AmpVCav-Mon']
+        """Gap voltage in [V]."""
+        return self['VGap-Mon']
+
+    @property
+    def power(self):
+        """Power at cavity in [W]."""
+        return self['PwrW-Mon']
+
+    @property
+    def power_dbm(self):
+        """Power at cavity in [dBm]."""
+        return self['PwrdBm-Mon']
+
+    @property
+    def amplitude(self):
+        """Amplitude at cavity in [mV]."""
+        return self['Amp-Mon']
+
+    @property
+    def power_forward(self):
+        """Forward power in [W]."""
+        return self['FwdPwrW-Mon']
+
+    @property
+    def power_forward_dbm(self):
+        """Forward power in [dBm]."""
+        return self['FwdPwrdBm-Mon']
+
+    @property
+    def power_reverse(self):
+        """Reverse power in [W]."""
+        return self['RevPwrW-Mon']
+
+    @property
+    def power_reverse_dbm(self):
+        """Reverse power in [dBm]."""
+        return self['RevPwrdBm-Mon']
+
+    @property
+    def amplitude_forward(self):
+        """Forward amplitude in [mV]."""
+        return self['FwdAmp-Mon']
+
+    @property
+    def amplitude_reverse(self):
+        """Reverse amplitude in [mV]."""
+        return self['RevAmp-Mon']
+
+    @property
+    def tuner_moving_down(self):
+        """Is tuner moving down?."""
+        return self['TunerMoveDown-Mon']
+
+    @property
+    def tuner_moving_up(self):
+        """Is tuner moving up?."""
+        return self['TunerMoveUp-Mon']
 
 
-class RFCav(_Devices):
+class RFCav(_DeviceSet):
     """."""
 
     class DEVICES:
         """Devices names."""
 
         BO = 'BO-05D:RF-P5Cav'
-        SI = 'SI-02SB:RF-P7Cav'
-        ALL = (BO, SI)
+        SIA = 'SI-03SP:RF-SRFCav-A'
+        SIB = 'SI-03SP:RF-SRFCav-B'
+        ALL = (BO, SIA, SIB)
 
-    def __init__(self, devname):
-        """."""
+    def __init__(self, devname, props2init='all'):
+        """RFCav DeviceSet.
+
+        Args:
+            devname (str): choose the accelerator cavity in RFCav.DEVICES
+            props2init (str, optional): 'all' to connect with all PVs or
+                bool(props2init) == False to initialize without any
+                connection. Defaults to 'all'.
+
+        """
         # check if device exists
         if devname not in RFCav.DEVICES.ALL:
             raise NotImplementedError(devname)
 
-        self.dev_rfgen = RFGen()
-        if devname == RFCav.DEVICES.SI:
-            self.dev_llrf = ASLLRF(ASLLRF.DEVICES.SI)
-            self.dev_cavmon = SIRFCavMonitor()
+        _isall = isinstance(props2init, str) and props2init.lower() == 'all'
+        if not _isall and props2init:
+            raise ValueError(
+                "props2init must be 'all' or bool(props2init) == False")
+
+        self.dev_rfgen = RFGen(props2init=props2init)
+        if devname == RFCav.DEVICES.SIA:
+            self.dev_llrf = ASLLRF(ASLLRF.DEVICES.SIA, props2init=props2init)
+            self.dev_cavmon = SIRFCavMonitor(
+                SIRFCavMonitor.DEVICES.SIA, props2init=props2init)
+        elif devname == RFCav.DEVICES.SIB:
+            self.dev_llrf = ASLLRF(ASLLRF.DEVICES.SIB, props2init=props2init)
+            self.dev_cavmon = SIRFCavMonitor(
+                SIRFCavMonitor.DEVICES.SIB, props2init=props2init)
         elif devname == RFCav.DEVICES.BO:
-            self.dev_llrf = ASLLRF(ASLLRF.DEVICES.BO)
-            self.dev_cavmon = BORFCavMonitor()
+            self.dev_llrf = ASLLRF(ASLLRF.DEVICES.BO, props2init=props2init)
+            self.dev_cavmon = BORFCavMonitor(props2init=props2init)
         devices = (self.dev_rfgen, self.dev_llrf, self.dev_cavmon)
 
         # call base class constructor
-        super().__init__(devname, devices)
+        super().__init__(devices, devname=devname)
 
     @property
     def is_cw(self):
@@ -1094,49 +1346,267 @@ class RFCav(_Devices):
         return self.dev_rfgen.set_frequency(value, tol=tol, timeout=timeout)
 
 
-class RFKillBeam(ASLLRF):
+class RFKillBeam(_DeviceSet):
     """RF Kill Beam Button."""
-    TIMEOUT_WAIT = 20.0  # [s]
-    INCRATE_VALUE = ASLLRF.VoltIncRates.vel_50p0  # [mV/s]
-    REFMIN_VALUE = 60  # Minimum Amplitude Reference [mV]
+
+    TIMEOUT_WAIT = 10  # [s]
 
     def __init__(self):
         """Init."""
-        super().__init__(ASLLRF.DEVICES.SI)
+        props2init = ASLLRF.PROPERTIES_INTERLOCK
+        si_a = ASLLRF(devname=ASLLRF.DEVICES.SIA, props2init=props2init)
+        si_b = ASLLRF(devname=ASLLRF.DEVICES.SIB, props2init=props2init)
+        devs = [si_a, si_b]
+        super().__init__(devices=devs, devname='SI-Glob:RF-KillBeam')
 
     def cmd_kill_beam(self):
         """Kill beam."""
         if not self.wait_for_connection(self.TIMEOUT_WAIT):
             return [False, 'Could not read RF PVs.']
 
-        # get initial values
-        amp_incrate_init = self.voltage_incrate
-        amp_init = self.voltage
-
-        # set Amplitude Increase Rate to 50 mV/s and wait
-        self.voltage_incrate = self.INCRATE_VALUE
-        if not self._wait(
-                'AMPREF:INCRATE', self.INCRATE_VALUE,
-                timeout=self.TIMEOUT_WAIT):
-            return [False, 'Could not set RF Amplitude Increase Rate.']
-
-        # set Amplitude Reference to 60mV and wait
-        self.voltage = self.REFMIN_VALUE
-        if not self._wait_float(
-                'SL:INP:AMP', self.REFMIN_VALUE, abs_tol=1,
-                timeout=self.TIMEOUT_WAIT):
-            return [False, 'Could not set RF Voltage to low value.']
-
-        # set Amplitude Reference to initial value
-        self.voltage = amp_init
-        if not self._wait_float(
-                'SL:INP:AMP', amp_init, abs_tol=1, timeout=self.TIMEOUT_WAIT):
-            return [False, 'Could not set RF Voltage back to original value.']
-
-        # set Amplitude Increase Rate to initial value
-        self.voltage_incrate = amp_incrate_init
-        if not self._wait(
-                'AMPREF:INCRATE', self.INCRATE_VALUE,
-                timeout=self.TIMEOUT_WAIT):
-            return [False, 'Could not set RF Amplitude Increase Rate back.']
+        for llrf in self.devices:
+            llrf.interlock_manual = 1
+        _time.sleep(1)
+        for llrf in self.devices:
+            llrf.interlock_manual = 0
         return [True, '']
+
+
+class SILLRFPreAmp(_Device):
+    """SILLRF Pre Amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+        # Cavity A
+        SSA1 = 'RA-ToSIA01:RF-CtrlPanel'
+        SSA2 = 'RA-ToSIA02:RF-CtrlPanel'
+        # Cavity B
+        SSA3 = 'RA-ToSIB03:RF-CtrlPanel'
+        SSA4 = 'RA-ToSIB04:RF-CtrlPanel'
+        ALL = (SSA1, SSA2, SSA3, SSA4)
+
+    PROPERTIES_DEFAULT = (
+        'PINSwEnbl-Cmd', 'PINSwDsbl-Cmd', 'PINSwSts-Mon',
+    )
+
+    def __init__(self, devname, props2init='all'):
+        if devname not in SILLRFPreAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable_pinsw(self, timeout=None, wait_mon=True):
+        """Enable PINSw 1."""
+        self['PINSwEnbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PINSwEnbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PINSwSts-Mon', 1, timeout=timeout)
+        return True
+
+    def cmd_disable_pinsw(self, timeout=None, wait_mon=True):
+        """Disable PINSw 1."""
+        self['PINSwDsbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PINSwDsbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PINSwSts-Mon', 0, timeout=timeout)
+        return True
+
+
+class BOLLRFPreAmp(_Device):
+    """BOLLRF Pre Amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+
+        BO01 = 'RA-RaBO01:RF-LLRFPreAmp'
+        ALL = (BO01, )
+
+    PROPERTIES_DEFAULT = (
+        'PinSwEnbl-Cmd', 'PinSwDsbl-Cmd', 'PinSw-Mon',
+    )
+
+    def __init__(self, devname='', props2init='all'):
+        if not devname:
+            devname = BOLLRFPreAmp.DEVICES.BO01
+        if devname not in BOLLRFPreAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable_pinsw(self, timeout=None, wait_mon=True):
+        """Enable PINSw."""
+        self['PinSwEnbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PinSwEnbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PinSw-Mon', 1, timeout=timeout)
+        return True
+
+    def cmd_disable_pinsw(self, timeout=None, wait_mon=True):
+        """Disable PINSw."""
+        self['PinSwDsbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PinSwDsbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PinSw-Mon', 0, timeout=timeout)
+        return True
+
+
+class SIRFDCAmp(_Device):
+    """SI RF DC/TDK amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+        # Cavity A
+        SSA1 = 'RA-ToSIA01:RF-TDKSource'
+        SSA2 = 'RA-ToSIA02:RF-TDKSource'
+        # Cavity B
+        SSA3 = 'RA-ToSIB03:RF-TDKSource'
+        SSA4 = 'RA-ToSIB04:RF-TDKSource'
+        ALL = (SSA1, SSA2, SSA3, SSA4)
+
+    PROPERTIES_DEFAULT = (
+        'PwrDCEnbl-Cmd', 'PwrDCDsbl-Cmd', 'PwrDC-Mon',
+    )
+
+    def __init__(self, devname, props2init='all'):
+        if devname not in SIRFDCAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable(self, timeout=None, wait_mon=True):
+        """Enable."""
+        self['PwrDCEnbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PwrDCEnbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PwrDC-Mon', 1, timeout=timeout)
+        return True
+
+    def cmd_disable(self, timeout=None, wait_mon=True):
+        """Disable."""
+        self['PwrDCDsbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PwrDCDsbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PwrDC-Mon', 0, timeout=timeout)
+        return True
+
+
+class BORFDCAmp(_Device):
+    """BO RF DC/DC amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+
+        SSA = 'RA-ToBO:RF-SSAmpTower'
+        ALL = (SSA, )
+
+    PROPERTIES_DEFAULT = (
+        'PwrCnvEnbl-Sel', 'PwrCnvDsbl-Sel', 'PwrCnv-Sts',
+    )
+
+    def __init__(self, devname='', props2init='all'):
+        if not devname:
+            devname = BORFDCAmp.DEVICES.SSA
+        if devname not in BORFDCAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable(self, timeout=None, wait_mon=True):
+        """Enable."""
+        self['PwrCnvEnbl-Sel'] = 1
+        _time.sleep(1)
+        self['PwrCnvEnbl-Sel'] = 0
+        if wait_mon:
+            return self._wait('PwrCnv-Sts', 1, timeout=timeout)
+        return True
+
+    def cmd_disable(self, timeout=None, wait_mon=True):
+        """Disable."""
+        self['PwrCnvDsbl-Sel'] = 1
+        _time.sleep(1)
+        self['PwrCnvDsbl-Sel'] = 0
+        if wait_mon:
+            return self._wait('PwrCnv-Sts', 0, timeout=timeout)
+        return True
+
+
+class SIRFACAmp(_Device):
+    """SI RF AC/TDK amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+        # Cavity A
+        SSA1 = 'RA-ToSIA01:RF-ACPanel'
+        SSA2 = 'RA-ToSIA02:RF-ACPanel'
+        # Cavity B
+        SSA3 = 'RA-ToSIB03:RF-ACPanel'
+        SSA4 = 'RA-ToSIB04:RF-ACPanel'
+        ALL = (SSA1, SSA2, SSA3, SSA4)
+
+    PROPERTIES_DEFAULT = (
+        'PwrACEnbl-Cmd', 'PwrACDsbl-Cmd', 'PwrAC-Mon',
+    )
+
+    def __init__(self, devname, props2init='all'):
+        if devname not in SIRFACAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable(self, timeout=None, wait_mon=True):
+        """Enable."""
+        self['PwrACEnbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PwrACEnbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PwrAC-Mon', 1, timeout=timeout)
+        return True
+
+    def cmd_disable(self, timeout=None, wait_mon=True):
+        """Disable."""
+        self['PwrACDsbl-Cmd'] = 1
+        _time.sleep(1)
+        self['PwrACDsbl-Cmd'] = 0
+        if wait_mon:
+            return self._wait('PwrAC-Mon', 0, timeout=timeout)
+        return True
+
+
+class BORF300VDCAmp(_Device):
+    """BO RF 300VDC amplifier."""
+
+    class DEVICES:
+        """Devices names."""
+
+        SSA = 'RA-ToBO:RF-ACDCPanel'
+        ALL = (SSA, )
+
+    PROPERTIES_DEFAULT = (
+        '300VdcEnbl-Sel', '300VdcDsbl-Sel', '300Vdc-Sts',
+    )
+
+    def __init__(self, devname=None, props2init='all'):
+        if not devname:
+            devname = BORF300VDCAmp.DEVICES.SSA
+        if devname not in BORF300VDCAmp.DEVICES.ALL:
+            raise NotImplementedError(devname)
+        super().__init__(devname, props2init=props2init)
+
+    def cmd_enable(self, timeout=None, wait_mon=True):
+        """Enable."""
+        self['300VdcEnbl-Sel'] = 1
+        _time.sleep(1)
+        self['300VdcEnbl-Sel'] = 0
+        if wait_mon:
+            return self._wait('300Vdc-Sts', 1, timeout=timeout)
+        return True
+
+    def cmd_disable(self, timeout=None, wait_mon=True):
+        """Disable."""
+        self['300VdcDsbl-Sel'] = 1
+        _time.sleep(1)
+        self['300VdcDsbl-Sel'] = 0
+        if wait_mon:
+            return self._wait('300Vdc-Sts', 0, timeout=timeout)
+        return True
