@@ -21,7 +21,7 @@ class SOFB(_Device):
         SI = 'SI-Glob:AP-SOFB'
         ALL = (TB, BO, TS, SI)
 
-    def __new__(cls, devname):
+    def __new__(cls, devname, props2init='all'):
         """."""
         # check if device exists
         if devname not in SOFB.DEVICES.ALL:
@@ -31,18 +31,18 @@ class SOFB(_Device):
         data = SOFBFactory.create(devname[:2])
 
         if data.acc == 'SI':
-            return SISOFB(devname, data)
+            return SISOFB(devname, data, props2init=props2init)
         elif data.isring:
-            return BOSOFB(devname, data)
-        return TLSOFB(devname, data)
+            return BOSOFB(devname, data, props2init=props2init)
+        return TLSOFB(devname, data, props2init=props2init)
 
 
 class TLSOFB(_Device):
     """SOFB Device."""
 
-    _properties = (
+    PROPERTIES_DEFAULT = (
         'TrigAcqChan-Sel', 'TrigAcqChan-Sts', 'OrbStatus-Mon',
-        'RespMat-SP', 'RespMat-RB', 'InvRespMat-Mon',
+        'RespMat-SP', 'RespMat-RB', 'RespMat-Mon', 'InvRespMat-Mon',
         'KickCH-Mon', 'KickCV-Mon',
         'DeltaKickCH-Mon', 'DeltaKickCV-Mon',
         'DeltaKickCH-SP', 'DeltaKickCV-SP',
@@ -78,7 +78,7 @@ class TLSOFB(_Device):
     _default_timeout_respm = 2 * 60 * 60  # [s]
     _default_timeout_kick_apply = 2  # [s]
 
-    def __init__(self, devname, data=None):
+    def __init__(self, devname, data=None, props2init='all'):
         """."""
         # check if device exists
         if devname not in SOFB.DEVICES.ALL:
@@ -87,7 +87,7 @@ class TLSOFB(_Device):
         # SOFB object
         self._data = data or SOFBFactory.create(devname[:2])
         # call base class constructor
-        super().__init__(devname, properties=self._properties)
+        super().__init__(devname, props2init=props2init)
 
     @property
     def data(self):
@@ -106,17 +106,21 @@ class TLSOFB(_Device):
 
     @property
     def respmat(self):
-        """."""
+        """Raw response matrix."""
         return self['RespMat-RB'].reshape(self._data.nr_bpms*2, -1)
 
     @respmat.setter
     def respmat(self, mat):
-        """."""
         self['RespMat-SP'] = _np.array(mat).ravel()
 
+    @property
+    def respmat_mon(self):
+        """Applied response matrix."""
+        return self['RespMat-Mon'].reshape(self._data.nr_bpms*2, -1)
+
     @property
     def invrespmat(self):
-        """."""
+        """Inverse response matrix."""
         return self['InvRespMat-Mon'].reshape(-1, self._data.nr_bpms*2)
 
     @property
@@ -341,48 +345,66 @@ class TLSOFB(_Device):
 
     def cmd_reset(self):
         """."""
-        self['SmoothReset-Cmd'] = 1
-        return True
+        prop = 'SmoothReset-Cmd'
+        val = self[prop]
+        self[prop] = 1
+        return self._wait(prop, val+1)
 
     def cmd_calccorr(self):
         """."""
-        self['CalcDelta-Cmd'] = 1
-        return True
+        prop = 'CalcDelta-Cmd'
+        val = self[prop]
+        self[prop] = 1
+        return self._wait(prop, val+1)
 
     def cmd_applycorr_ch(self):
         """."""
-        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.CH
-        return True
+        prop = 'ApplyDelta-Cmd'
+        val = self[prop]
+        self[prop] = self._data.ApplyDelta.CH
+        return self._wait(prop, val+1)
 
     def cmd_applycorr_cv(self):
         """."""
-        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.CV
-        return True
+        prop = 'ApplyDelta-Cmd'
+        val = self[prop]
+        self[prop] = self._data.ApplyDelta.CV
+        return self._wait(prop, val+1)
 
     def cmd_applycorr_rf(self):
         """."""
-        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.RF
-        return True
+        prop = 'ApplyDelta-Cmd'
+        val = self[prop]
+        self[prop] = self._data.ApplyDelta.RF
+        return self._wait(prop, val+1)
 
     def cmd_applycorr_all(self):
         """."""
-        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.All
-        return True
+        prop = 'ApplyDelta-Cmd'
+        val = self[prop]
+        self[prop] = self._data.ApplyDelta.All
+        return self._wait(prop, val+1)
 
     def cmd_measrespmat_start(self):
         """."""
-        self['MeasRespMat-Cmd'] = 0
-        return True
+        prop = 'MeasRespMat-Cmd'
+        val = self[prop]
+        self[prop] = 0
+        return self._wait(prop, val+1)
 
     def cmd_measrespmat_stop(self):
         """."""
-        self['MeasRespMat-Cmd'] = 1
-        return True
+        prop = 'MeasRespMat-Cmd'
+        val = self[prop]
+        self[prop] = 1
+        return self._wait(prop, val+1)
 
     def cmd_measrespmat_reset(self):
         """."""
-        self['MeasRespMat-Cmd'] = 2
-        return True
+        prop = 'MeasRespMat-Cmd'
+        val = self[prop]
+        self[prop] = 2
+        return self._wait(prop, val+1)
 
     def cmd_trigacq_start(self, timeout=10):
         """."""
@@ -404,16 +426,6 @@ class TLSOFB(_Device):
         _time.sleep(0.6)  # Status PV updates at 2Hz
         return self.wait_orb_status_ok(timeout=timeout)
 
-    def cmd_trigacq_abort(self, timeout=10):
-        """."""
-        self['TrigAcqCtrl-Sel'] = 'Abort'
-        ret = self._wait(
-            'TrigAcqCtrl-Sts', self._data.TrigAcqCtrl.Abort, timeout=timeout)
-        if not ret:
-            return False
-        _time.sleep(0.6)  # Status PV updates at 2Hz
-        return self.wait_orb_status_ok(timeout=timeout)
-
     def cmd_trigacq_config(self, timeout=10):
         """."""
         self['TrigAcqConfig-Cmd'] = 1
@@ -468,13 +480,11 @@ class TLSOFB(_Device):
             resy = _np.sqrt(_np.sum(resy*resy)/resy.size)
             if resx < residue and resy < residue:
                 break
+            self.wait_buffer()
             self.cmd_calccorr()
-            _time.sleep(0.5)
             self.cmd_applycorr_all()
             self.wait_apply_delta_kick()
-            _time.sleep(0.2)
             self.cmd_reset()
-            self.wait_buffer()
         return i, resx, resy
 
     def wait_buffer(self, timeout=None):
@@ -505,18 +515,25 @@ class TLSOFB(_Device):
 
     def cmd_sync_bpms(self):
         """Synchronize BPMs."""
-        self['SyncBPMs-Cmd'] = 1
-        return True
+        prop = 'SyncBPMs-Cmd'
+        val = self[prop]
+        self[prop] = 1
+        return self._wait(prop, val+1)
 
 
 class BOSOFB(TLSOFB):
     """SOFB Device."""
 
-    _properties = TLSOFB._properties + (
+    PROPERTIES_DEFAULT = TLSOFB.PROPERTIES_DEFAULT + (
         'MTurnAcquire-Cmd',
         'MTurnSum-Mon', 'MTurnOrbX-Mon', 'MTurnOrbY-Mon',
         'MTurnIdxOrbX-Mon', 'MTurnIdxOrbY-Mon', 'MTurnIdxSum-Mon',
         'MTurnTime-Mon',
+        'MTurnDownSample-SP', 'MTurnDownSample-RB',
+        'MTurnIdx-SP', 'MTurnIdx-RB', 'MTurnIdxTime-Mon',
+        'MTurnUseMask-Sel', 'MTurnUseMask-Sts',
+        'MTurnMaskSplBeg-SP', 'MTurnMaskSplBeg-RB',
+        'MTurnMaskSplEnd-SP', 'MTurnMaskSplEnd-RB',
         )
 
     @property
@@ -554,6 +571,61 @@ class BOSOFB(TLSOFB):
         """."""
         return self['MTurnIdxSum-Mon']
 
+    @property
+    def mt_downsampling(self):
+        """."""
+        return self['MTurnDownSample-RB']
+
+    @mt_downsampling.setter
+    def mt_downsampling(self, value):
+        """."""
+        self['MTurnDownSample-SP'] = int(value)
+
+    @property
+    def mt_idx(self):
+        """."""
+        return self['MTurnIdx-RB']
+
+    @mt_idx.setter
+    def mt_idx(self, value):
+        """."""
+        self['MTurnIdx-SP'] = int(value)
+
+    @property
+    def mt_idx_time(self):
+        """."""
+        return self['MTurnIdxTime-Mon']
+
+    @property
+    def mt_use_tbt_mask(self):
+        """."""
+        return bool(self['MTurnUseMask-Sts'])
+
+    @mt_use_tbt_mask.setter
+    def mt_use_tbt_mask(self, value):
+        """."""
+        self['MTurnUseMask-Sel'] = value
+
+    @property
+    def mt_tbt_mask_begin(self):
+        """."""
+        return self['MTurnMaskSplBeg-RB']
+
+    @mt_tbt_mask_begin.setter
+    def mt_tbt_mask_begin(self, value):
+        """."""
+        self['MTurnMaskSplBeg-SP'] = int(value)
+
+    @property
+    def mt_tbt_mask_end(self):
+        """."""
+        return self['MTurnMaskSplEnd-RB']
+
+    @mt_tbt_mask_end.setter
+    def mt_tbt_mask_end(self, value):
+        """."""
+        self['MTurnMaskSplEnd-SP'] = int(value)
+
     @property
     def trajx(self):
         """."""
@@ -571,14 +643,16 @@ class BOSOFB(TLSOFB):
 
     def cmd_mturn_acquire(self):
         """."""
-        self['MTurnAcquire-Cmd'] = 1
-        return True
+        prop = 'MTurnAcquire-Cmd'
+        val = self[prop]
+        self[prop] = 1
+        return self._wait(prop, val+1)
 
 
 class SISOFB(BOSOFB):
     """SOFB Device."""
 
-    _properties = BOSOFB._properties + (
+    PROPERTIES_DEFAULT = BOSOFB.PROPERTIES_DEFAULT + (
         'SOFBMode-Sel', 'SOFBMode-Sts',
         'KickRF-Mon',
         'DeltaKickRF-Mon', 'DeltaKickRF-SP',
@@ -588,6 +662,13 @@ class SISOFB(BOSOFB):
         'RFEnbl-Sel', 'RFEnbl-Sts',
         'SlowOrbX-Mon', 'SlowOrbY-Mon',
         'LoopState-Sts', 'LoopState-Sel',
+        'CorrSync-Sts', 'CorrSync-Sel',
+        'LoopPIDKpCH-SP', 'LoopPIDKiCH-SP', 'LoopPIDKdCH-SP',
+        'LoopPIDKpCH-RB', 'LoopPIDKiCH-RB', 'LoopPIDKdCH-RB',
+        'LoopPIDKpCV-SP', 'LoopPIDKiCV-SP', 'LoopPIDKdCV-SP',
+        'LoopPIDKpCV-RB', 'LoopPIDKiCV-RB', 'LoopPIDKdCV-RB',
+        'LoopPIDKpRF-SP', 'LoopPIDKiRF-SP', 'LoopPIDKdRF-SP',
+        'LoopPIDKpRF-RB', 'LoopPIDKiRF-RB', 'LoopPIDKdRF-RB',
         'DriveFreqDivisor-SP', 'DriveFreqDivisor-RB', 'DriveFrequency-Mon',
         'DriveNrCycles-SP', 'DriveNrCycles-RB', 'DriveDuration-Mon',
         'DriveAmplitude-SP', 'DriveAmplitude-RB',
@@ -634,6 +715,87 @@ class SISOFB(BOSOFB):
         _time.sleep(0.6)  # Status PV updates at 2Hz
         return self.wait_orb_status_ok(timeout=timeout)
 
+    @property
+    def loop_pid_ch_kp(self):
+        """Loop PID Kp parameter for CH."""
+        return self['LoopPIDKpCH-RB']
+
+    @loop_pid_ch_kp.setter
+    def loop_pid_ch_kp(self, value):
+        self['LoopPIDKpCH-SP'] = value
+
+    @property
+    def loop_pid_ch_ki(self):
+        """Loop PID Ki parameter for CH."""
+        return self['LoopPIDKiCH-RB']
+
+    @loop_pid_ch_ki.setter
+    def loop_pid_ch_ki(self, value):
+        self['LoopPIDKiCH-SP'] = value
+
+    @property
+    def loop_pid_ch_kd(self):
+        """Loop PID Kd parameter for CH."""
+        return self['LoopPIDKdCH-RB']
+
+    @loop_pid_ch_kd.setter
+    def loop_pid_ch_kd(self, value):
+        self['LoopPIDKdCH-SP'] = value
+
+    @property
+    def loop_pid_cv_kp(self):
+        """Loop PID Kp parameter for CV."""
+        return self['LoopPIDKpCV-RB']
+
+    @loop_pid_cv_kp.setter
+    def loop_pid_cv_kp(self, value):
+        self['LoopPIDKpCV-SP'] = value
+
+    @property
+    def loop_pid_cv_ki(self):
+        """Loop PID Ki parameter for CV."""
+        return self['LoopPIDKiCV-RB']
+
+    @loop_pid_cv_ki.setter
+    def loop_pid_cv_ki(self, value):
+        self['LoopPIDKiCV-SP'] = value
+
+    @property
+    def loop_pid_cv_kd(self):
+        """Loop PID Kd parameter for CV."""
+        return self['LoopPIDKdCV-RB']
+
+    @loop_pid_cv_kd.setter
+    def loop_pid_cv_kd(self, value):
+        self['LoopPIDKdCV-SP'] = value
+
+    @property
+    def loop_pid_rf_kp(self):
+        """Loop PID Kp parameter for RF."""
+        return self['LoopPIDKpRF-RB']
+
+    @loop_pid_rf_kp.setter
+    def loop_pid_rf_kp(self, value):
+        self['LoopPIDKpRF-SP'] = value
+
+    @property
+    def loop_pid_rf_ki(self):
+        """Loop PID Ki parameter for RF."""
+        return self['LoopPIDKiRF-RB']
+
+    @loop_pid_rf_ki.setter
+    def loop_pid_rf_ki(self, value):
+        self['LoopPIDKiRF-SP'] = value
+
+    @property
+    def loop_pid_rf_kd(self):
+        """Loop PID Kd parameter for RF."""
+        return self['LoopPIDKdRF-RB']
+
+    @loop_pid_rf_kd.setter
+    def loop_pid_rf_kd(self, value):
+        self['LoopPIDKdRF-SP'] = value
+
     @property
     def drivests(self):
         """."""
@@ -828,6 +990,29 @@ class SISOFB(BOSOFB):
         return self._wait(
             'LoopState-Sts', self._data.LoopState.Open, timeout=timeout)
 
+    @property
+    def synckicksts(self):
+        """Correction syncronization status."""
+        return self['CorrSync-Sts']
+
+    @synckicksts.setter
+    def synckicksts(self, value):
+        self._enum_setter('CorrSync-Sel', value, self._data.CorrSync)
+
+    @property
+    def synckicksts_str(self):
+        """Correction syncronization status enum string."""
+        return self._data.CorrSync._fields[self['CorrSync-Sts']]
+
+    def cmd_turn_off_synckick(self, timeout=None):
+        """Turn off correction synchronization."""
+        timeout = timeout or self._default_timeout
+        if self.synckicksts == self._data.CorrSync.Off:
+            return True
+        self['CorrSync-Sel'] = self._data.CorrSync.Off
+        return self._wait(
+            'CorrSync-Sts', self._data.CorrSync.Off, timeout=timeout)
+
     def cmd_turn_on_drive(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout
