@@ -1,15 +1,15 @@
 """Synchronized Devices."""
 
 from ..namesys import SiriusPVName as _SiriusPVName
-from .device import DeviceApp as _DeviceApp
+from .device import Device as _Device
 
 
-class DevicesSync(_DeviceApp):
+class DevicesSync(_Device):
     """Synchronized devices."""
 
     def __init__(
             self, devnames, propty_sync, propty_async=None,
-            devname=None, auto_mon=False):
+            devname=None, auto_monitor_mon=False):
         """."""
         self._devnames = [_SiriusPVName(dev) for dev in devnames]
         self._props_sync = list(propty_sync)
@@ -19,7 +19,8 @@ class DevicesSync(_DeviceApp):
         properties, self._prop2prop = self._get_properties()
 
         # call base class constructor
-        super().__init__(properties, devname, auto_mon)
+        super().__init__(
+            devname, auto_monitor_mon=auto_monitor_mon, props2init=properties)
 
     @property
     def devnames(self):
@@ -46,7 +47,7 @@ class DevicesSync(_DeviceApp):
                 return False
         return True
 
-    def value_get(self, propty):
+    def get_value(self, propty):
         """Return property value."""
         if not self.connected:
             return
@@ -60,7 +61,7 @@ class DevicesSync(_DeviceApp):
             return
         return sum(values) / len(values)
 
-    def value_set(self, propty, value):
+    def set_value(self, propty, value):
         """Set property."""
         if not self.connected:
             return
