@@ -1,24 +1,30 @@
 """Injection Control App."""
 
 from .. import csdev as _csdev
-
+from ..util import ClassProperty as _classproperty
+from ..search import PSSearch as _PSSearch
 
 # --- Enumeration Types ---
 
+
 class ETypes(_csdev.ETypes):
     """Local enumerate types."""
 
-    INJMODE = ('Decay', 'TopUp')
+    INJMODE = ('Decay', 'TopUp', 'Accum')
     INJTYPE = ('SingleBunch', 'MultiBunch')
     INJTYPE_MON = INJTYPE + ('Undefined', )
     PUMODE = ('Accumulation', 'Optimization', 'OnAxis')
     PUMODE_MON = PUMODE + ('Undefined', )
     TOPUPSTS = (
         'Off', 'Waiting', 'TurningOn', 'Injecting', 'TurningOff', 'Skipping')
+    ACCUMSTS = ('Off', 'Waiting', 'TurningOn', 'Injecting')
     INJSYSCMDSTS = ('Idle', 'On', 'Off')
     RFKILLBEAMMON = ('Idle', 'Kill')
     IDLERUNNING = ('Idle', 'Running')
+    IDLEINJECTING = ('Idle', 'Injecting')
     BIASFB_MODEL_TYPES = ('Linear', 'GaussianProcess')
+    FIXED_UNFIXED = ('Fixed', 'Unfixed')
+    STANDBY_INJECT = ('Standby', 'Inject')
 
 
 _et = ETypes
@@ -35,11 +41,15 @@ class Const(_csdev.Const):
     PUMode = _csdev.Const.register('PUMode', _et.PUMODE)
     PUModeMon = _csdev.Const.register('PUModeMon', _et.PUMODE_MON)
     TopUpSts = _csdev.Const.register('TopUpSts', _et.TOPUPSTS)
+    AccumSts = _csdev.Const.register('AccumSts', _et.ACCUMSTS)
     InjSysCmdSts = _csdev.Const.register('InjSysCmdSts', _et.INJSYSCMDSTS)
     RFKillBeamMon = _csdev.Const.register('RFKillBeamMon', _et.RFKILLBEAMMON)
     IdleRunning = _csdev.Const.register('IdleRunning', _et.IDLERUNNING)
+    IdleInjecting = _csdev.Const.register('IdleInjecting', _et.IDLEINJECTING)
     BiasFBModelTypes = _csdev.Const.register(
         'ModelTypes', _et.BIASFB_MODEL_TYPES)
+    FixedUnfixed = _csdev.Const.register('FixedUnfixed', _et.FIXED_UNFIXED)
+    StandbyInject = _csdev.Const.register('StandbyInject', _et.STANDBY_INJECT)
 
     GEN_STATUS_LABELS = ('LI', 'TB', 'BO', 'TS', 'SI', 'AS')
     LI_STATUS_LABELS = ('Egun', 'PS', 'PU', 'RF', 'TI')
@@ -69,16 +79,43 @@ class Const(_csdev.Const):
     FILACURR_OPVALUE = 1.39  # [A]
     HV_OPVALUE = 90.0  # [kV]
 
-    INJSYS_DEF_ON_ORDER = ['bo_rf', 'as_pu', 'bo_ps', 'injbo', 'li_rf']
-    INJSYS_DEF_OFF_ORDER = ['bo_rf', 'li_rf', 'injbo', 'as_pu', 'bo_ps']
-
-    PU_VOLTAGE_UP_TIME = 30  # [s]
+    INJSYS_DEF_ON_ORDER = ['bo_rf', 'as_pu', 'bo_ps', 'li_rf']
+    INJSYS_DEF_OFF_ORDER = ['bo_rf', 'li_rf', 'as_pu', 'bo_ps']
 
     BIASFB_AHEADSETIME = 10  # [s]
     BIASFB_MINIMUM_LIFETIME = 1800  # [s]
     BIASFB_PROPTY_PREFIX = 'BiasFB'
     BIASFB_MAX_DATA_SIZE = 1000
 
+    __TOPUP_STANDBY_PUNAMES = None
+    __TOPUP_STANDBY_PUNICKNAMES = None
+
+    @_classproperty
+    def TOPUP_STANDBY_PUNAMES(cls):
+        """Define names of the pulsed magnets controlled in top-up standby."""
+        if cls.__TOPUP_STANDBY_PUNAMES is not None:
+            return cls.__TOPUP_STANDBY_PUNAMES
+
+        names = _PSSearch.get_psnames(
+            {
+                'sec': '(TB|BO|TS|SI)',
+                'dis': 'PU', 'dev': '.*(Kckr|Sept)',
+                'propty_name': '(?!:CCoil).*'
+            })
+        nicknames = [
+            f'{n.sec}{n.dev}{n.idx}' for n in names
+        ]
+        cls.__TOPUP_STANDBY_PUNAMES = names
+        cls.__TOPUP_STANDBY_PUNICKNAMES = nicknames
+
+        return cls.__TOPUP_STANDBY_PUNAMES
+
+    @_classproperty
+    def TOPUP_STANDBY_PUNICKNAMES(cls):
+        """Define nicknames for TOPUP_STANDBY_PUNAMES."""
+        cls.TOPUP_STANDBY_PUNAMES
+        return cls.__TOPUP_STANDBY_PUNICKNAMES
+
 
 _ct = Const
 
@@ -114,14 +151,16 @@ def get_injctrl_propty_database():
 
     dbase = {
         'Version-Cte': {'type': 'str', 'value': 'UNDEF'},
+        'TimestampBoot-Cte': {'type': 'float', 'value': 0,
+                              'prec': 7, 'unit': 'timestamp'},
         'Log-Mon': {'type': 'str', 'value': 'Starting...'},
 
         'Mode-Sel': {
             'type': 'enum', 'value': _ct.InjMode.Decay,
-            'enums': _et.INJMODE, 'unit': 'Decay_Topup'},
+            'enums': _et.INJMODE, 'unit': 'Decay_Topup_Accum'},
         'Mode-Sts': {
             'type': 'enum', 'value': _ct.InjMode.Decay,
-            'enums': _et.INJMODE, 'unit': 'Decay_Topup'},
+            'enums': _et.INJMODE, 'unit': 'Decay_Topup_Accum'},
         'Type-Sel': {
             'type': 'enum', 'value': _ct.InjType.MultiBunch,
             'enums': _et.INJTYPE, 'unit': 'SB_MB'},
@@ -204,11 +243,11 @@ def get_injctrl_propty_database():
             'prec': 3, 'lolim': -8.0, 'low': -8.0, 'lolo': -8.0,
             'hilim': 0.0, 'high': 0.0, 'hihi': 0.0},
         'TargetCurrent-SP': {
-            'type': 'float', 'value': 100.0, 'unit': 'mA',
+            'type': 'float', 'value': 200.0, 'unit': 'mA',
             'prec': 2, 'lolim': 0.0, 'low': 0.0, 'lolo': 0.0,
             'hilim': 500.0, 'high': 500.0, 'hihi': 500.0},
         'TargetCurrent-RB': {
-            'type': 'float', 'value': 100.0, 'unit': 'mA',
+            'type': 'float', 'value': 200.0, 'unit': 'mA',
             'prec': 2, 'lolim': 0.0, 'low': 0.0, 'lolo': 0.0,
             'hilim': 500.0, 'high': 500.0, 'hihi': 500.0},
         'BucketListStart-SP': {
@@ -224,12 +263,54 @@ def get_injctrl_propty_database():
             'type': 'int', 'value': 864, 'unit': 'bucket index',
             'lolim': _ct.MIN_BKT, 'hilim': _ct.MAX_BKT},
         'BucketListStep-SP': {
-            'type': 'int', 'value': 15, 'unit': 'buckets',
-            'lolim': -_ct.MAX_BKT+1, 'hilim': _ct.MAX_BKT-1},
+            'type': 'int', 'value': 29, 'unit': 'buckets',
+            'lolim': -_ct.MAX_BKT, 'hilim': _ct.MAX_BKT},
         'BucketListStep-RB': {
-            'type': 'int', 'value': 15, 'unit': 'buckets',
-            'lolim': -_ct.MAX_BKT+1, 'hilim': _ct.MAX_BKT-1},
+            'type': 'int', 'value': 29, 'unit': 'buckets',
+            'lolim': -_ct.MAX_BKT, 'hilim': _ct.MAX_BKT},
+        "BucketListAllowedMask-SP": {
+            "type": "int",
+            "count": _ct.MAX_BKT,
+            "value": _ct.MAX_BKT * [1],
+            "unit": "Buckets where injection is allowed",
+        },
+        "BucketListAllowedMask-RB": {
+            "type": "int",
+            "count": _ct.MAX_BKT,
+            "value": _ct.MAX_BKT * [1],
+            "unit": "Buckets where injection is allowed",
+        },
+        'IsInjecting-Mon': {
+            'type': 'enum', 'value': _ct.IdleInjecting.Idle,
+            'enums': _et.IDLEINJECTING, 'unit': 'Idle_Inj'},
+        'IsInjDelay-SP': {
+            'type': 'int', 'value': 0, 'unit': 'ms',
+            'lolim': 0, 'hilim': 1000},
+        'IsInjDelay-RB': {
+            'type': 'int', 'value': 0, 'unit': 'ms',
+            'lolim': 0, 'hilim': 1000},
+        'IsInjDuration-SP': {
+            'type': 'int', 'value': 300, 'unit': 'ms',
+            'lolim': 0, 'hilim': 1000},
+        'IsInjDuration-RB': {
+            'type': 'int', 'value': 300, 'unit': 'ms',
+            'lolim': 0, 'hilim': 1000},
 
+        # accumulation settings
+        'AccumState-Sel': {
+            'type': 'enum', 'value': _ct.OffOn.Off,
+            'enums': _et.OFF_ON, 'unit': 'Off_On'},
+        'AccumState-Sts': {
+            'type': 'enum', 'value': _ct.AccumSts.Off, 'enums': _et.ACCUMSTS,
+            'unit': 'Off_Wai_TOn_Inj_TOff_Skip'},
+        'AccumPeriod-SP': {
+            'type': 'int', 'value': 5, 'unit': 's',
+            'lolim': 1, 'hilim': 60*60},
+        'AccumPeriod-RB': {
+            'type': 'int', 'value': 5, 'unit': 's',
+            'lolim': 1, 'hilim': 60*60},
+
+        # topup settings
         'TopUpState-Sel': {
             'type': 'enum', 'value': _ct.OffOn.Off,
             'enums': _et.OFF_ON, 'unit': 'Off_On'},
@@ -237,10 +318,10 @@ def get_injctrl_propty_database():
             'type': 'enum', 'value': _ct.TopUpSts.Off, 'enums': _et.TOPUPSTS,
             'unit': 'Off_Wai_TOn_Inj_TOff_Skip'},
         'TopUpPeriod-SP': {
-            'type': 'int', 'value': 5, 'unit': 'min',
+            'type': 'int', 'value': 1, 'unit': 'min',
             'lolim': 1, 'hilim': 6*60},
         'TopUpPeriod-RB': {
-            'type': 'int', 'value': 5, 'unit': 'min',
+            'type': 'int', 'value': 1, 'unit': 'min',
             'lolim': 1, 'hilim': 6*60},
         'TopUpHeadStartTime-SP': {
             'type': 'float', 'value': 0, 'unit': 's', 'prec': 2,
@@ -248,12 +329,6 @@ def get_injctrl_propty_database():
         'TopUpHeadStartTime-RB': {
             'type': 'float', 'value': 0, 'unit': 's', 'prec': 2,
             'lolim': 0, 'hilim': 2*60},
-        'TopUpPUStandbyEnbl-Sel': {
-            'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
-            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
-        'TopUpPUStandbyEnbl-Sts': {
-            'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
-            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
         'TopUpNextInj-Mon': {
             'type': 'float', 'value': 0.0, 'unit': 's'},
         'TopUpNrPulses-SP': {
@@ -262,6 +337,44 @@ def get_injctrl_propty_database():
         'TopUpNrPulses-RB': {
             'type': 'int', 'value': 1, 'unit': 'pulses',
             'lolim': _ct.MIN_BKT, 'hilim': _ct.MAX_BKT},
+        # LI standby handler
+        'TopUpLIWarmUpEnbl-Sel': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Enbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpLIWarmUpEnbl-Sts': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Enbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpLIWarmUpTime-SP': {
+            'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
+        'TopUpLIWarmUpTime-RB': {
+            'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
+        # bo standby handlers
+        'TopUpBOPSStandbyEnbl-Sel': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpBOPSStandbyEnbl-Sts': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpBOPSWarmUpTime-SP': {
+            'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
+        'TopUpBOPSWarmUpTime-RB': {
+            'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
+        'TopUpBORFStandbyEnbl-Sel': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Enbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpBORFStandbyEnbl-Sts': {
+            'type': 'enum', 'value': _ct.DsblEnbl.Enbl,
+            'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+        'TopUpBORFWarmUpTime-SP': {
+            'type': 'float', 'value': 5, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
+        'TopUpBORFWarmUpTime-RB': {
+            'type': 'float', 'value': 5, 'unit': 's', 'prec': 1,
+            'lolim': 0, 'hilim': 2*60},
 
         'InjSysTurnOn-Cmd': {'type': 'int', 'value': 0},
         'InjSysTurnOff-Cmd': {'type': 'int', 'value': 0},
@@ -329,6 +442,23 @@ def get_injctrl_propty_database():
             'type': 'char', 'count': 1000,
             'value': '\n'.join(_ct.INJ_STATUS_LABELS)},
     }
+    # PU standby handlers
+    for puid in _ct.TOPUP_STANDBY_PUNICKNAMES:
+        dbase.update({
+            f'TopUp{puid}StandbyEnbl-Sel': {
+                'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
+                'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+            f'TopUp{puid}StandbyEnbl-Sts': {
+                'type': 'enum', 'value': _ct.DsblEnbl.Dsbl,
+                'enums': _et.DSBL_ENBL, 'unit': 'Dsbl_Enbl'},
+            f'TopUp{puid}WarmUpTime-SP': {
+                'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+                'lolim': 0, 'hilim': 2*60},
+            f'TopUp{puid}WarmUpTime-RB': {
+                'type': 'float', 'value': 10, 'unit': 's', 'prec': 1,
+                'lolim': 0, 'hilim': 2*60},
+        })
+    dbase.update(get_biasfb_database())
     dbase = _csdev.add_pvslist_cte(dbase)
     return dbase
 
@@ -337,27 +467,23 @@ def get_biasfb_database():
     """."""
     dbase = {
         'LoopState-Sel': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'LoopState-Sts': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'MinVoltage-SP': {
-            'type': 'float', 'value': -52, 'unit': 'V',
-            'prec': 1, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -44, 'unit': 'V',
+            'prec': 1, 'lolim': -120, 'hilim': -30.0},
         'MinVoltage-RB': {
-            'type': 'float', 'value': -52, 'unit': 'V',
-            'prec': 1, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -44, 'unit': 'V',
+            'prec': 1, 'lolim': -120, 'hilim': -30.0},
         'MaxVoltage-SP': {
-            'type': 'float', 'value': -40, 'unit': 'V',
-            'prec': 1, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -37, 'unit': 'V',
+            'prec': 1, 'lolim': -120, 'hilim': -30.0},
         'MaxVoltage-RB': {
-            'type': 'float', 'value': -40, 'unit': 'V',
-            'prec': 1, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -37, 'unit': 'V',
+            'prec': 1, 'lolim': -120, 'hilim': -30.0},
 
         'ModelType-Sel': {
             'type': 'enum', 'value': _ct.BiasFBModelTypes.GaussianProcess,
@@ -366,53 +492,35 @@ def get_biasfb_database():
             'type': 'enum', 'value': _ct.BiasFBModelTypes.GaussianProcess,
             'enums': _et.BIASFB_MODEL_TYPES, 'unit': 'Lin_GP'},
         'ModelMaxNrPts-SP': {
-            'type': 'int', 'value': 20, 'unit': '#',
-            'lolim': 2, 'low': 2, 'lolo': 2,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 30, 'unit': '#',
+            'lolim': 2, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelMaxNrPts-RB': {
-            'type': 'int', 'value': 20, 'unit': '#',
-            'lolim': 0, 'low': 0, 'lolo': 0,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 30, 'unit': '#',
+            'lolim': 2, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelNrPts-Mon': {
-            'type': 'int', 'value': 20, 'unit': '#',
-            'lolim': 2, 'low': 2, 'lolo': 2,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 30, 'unit': '#',
+            'lolim': 2, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelFitParamsNow-Cmd': {'type': 'int', 'value': 0},
         'ModelAutoFitParams-Sel': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'ModelAutoFitParams-Sts': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'ModelAutoFitEveryNrPts-SP': {
-            'type': 'int', 'value': 10, 'unit': '#',
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 1, 'unit': '#',
+            'lolim': 1, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelAutoFitEveryNrPts-RB': {
-            'type': 'int', 'value': 10, 'unit': '#',
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 1, 'unit': '#',
+            'lolim': 1, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelNrPtsAfterFit-Mon': {
-            'type': 'int', 'value': 10, 'unit': '#',
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': _ct.BIASFB_MAX_DATA_SIZE,
-            'high': _ct.BIASFB_MAX_DATA_SIZE,
-            'hihi': _ct.BIASFB_MAX_DATA_SIZE},
+            'type': 'int', 'value': 1, 'unit': '#',
+            'lolim': 1, 'hilim': _ct.BIASFB_MAX_DATA_SIZE},
         'ModelUpdateData-Sel': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'ModelUpdateData-Sts': {
-            'type': 'enum', 'value': _ct.OffOn.Off, 'enums': _et.OFF_ON,
+            'type': 'enum', 'value': _ct.OffOn.On, 'enums': _et.OFF_ON,
             'unit': 'Off_On'},
         'ModelDataBias-SP': {
             'type': 'float', 'count': _ct.BIASFB_MAX_DATA_SIZE,
@@ -435,28 +543,22 @@ def get_biasfb_database():
 
         'LinModAngCoeff-SP': {
             'type': 'float', 'value': 10, 'unit': 'V/mA',
-            'prec': 2, 'lolim': 0.1, 'low': 0.1, 'lolo': 0.1,
-            'hilim': 30.0, 'high': 30.0, 'hihi': 30.0},
+            'prec': 2, 'lolim': 0.1, 'hilim': 30.0},
         'LinModAngCoeff-RB': {
             'type': 'float', 'value': 10, 'unit': 'V/mA',
-            'prec': 2, 'lolim': 0.1, 'low': 0.1, 'lolo': 0.1,
-            'hilim': 30.0, 'high': 30.0, 'hihi': 30.0},
+            'prec': 2, 'lolim': 0.1, 'hilim': 30.0},
         'LinModAngCoeff-Mon': {
             'type': 'float', 'value': 10, 'unit': 'V/mA',
-            'prec': 2, 'lolim': 0.1, 'low': 0.1, 'lolo': 0.1,
-            'hilim': 30.0, 'high': 30.0, 'hihi': 30.0},
+            'prec': 2, 'lolim': 0.1, 'hilim': 30.0},
         'LinModOffCoeff-SP': {
-            'type': 'float', 'value': -52, 'unit': 'V/mA',
-            'prec': 2, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -44, 'unit': 'V/mA',
+            'prec': 2, 'lolim': -120, 'hilim': -30.0},
         'LinModOffCoeff-RB': {
-            'type': 'float', 'value': -52, 'unit': 'V/mA',
-            'prec': 2, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -44, 'unit': 'V/mA',
+            'prec': 2, 'lolim': -120, 'hilim': -30.0},
         'LinModOffCoeff-Mon': {
-            'type': 'float', 'value': -52, 'unit': 'V/mA',
-            'prec': 2, 'lolim': -120, 'low': -120, 'lolo': -120,
-            'hilim': -30.0, 'high': -30.0, 'hihi': -30.0},
+            'type': 'float', 'value': -44, 'unit': 'V/mA',
+            'prec': 2, 'lolim': -120, 'hilim': -30.0},
 
         # These are used to give the model inference about the bias
         # Generally ploted in Injcurr X Bias graphs
@@ -473,41 +575,54 @@ def get_biasfb_database():
             'type': 'float', 'count': 100, 'value': [0]*100, 'unit': 'mA'},
 
         'GPModNoiseStd-SP': {
-            'type': 'float', 'value': 0.05, 'unit': 'mA', 'prec': 4,
-            'lolim': 0.005, 'low': 0.005, 'lolo': 0.005,
-            'hilim': 0.5, 'high': 0.5, 'hihi': 0.5},
+            'type': 'float', 'value': 0.0316, 'unit': 'mA', 'prec': 4,
+            'lolim': 0.005, 'hilim': 0.5},
         'GPModNoiseStd-RB': {
-            'type': 'float', 'value': 0.05, 'unit': 'mA', 'prec': 4,
-            'lolim': 0.005, 'low': 0.005, 'lolo': 0.005,
-            'hilim': 0.5, 'high': 0.5, 'hihi': 0.5},
+            'type': 'float', 'value': 0.0316, 'unit': 'mA', 'prec': 4,
+            'lolim': 0.005, 'hilim': 0.5},
         'GPModNoiseStd-Mon': {
-            'type': 'float', 'value': 0.05, 'unit': 'mA', 'prec': 4,
-            'lolim': 0.005, 'low': 0.005, 'lolo': 0.005,
-            'hilim': 0.5, 'high': 0.5, 'hihi': 0.5},
+            'type': 'float', 'value': 0.0316, 'unit': 'mA', 'prec': 4,
+            'lolim': 0.005, 'hilim': 0.5},
         'GPModKernStd-SP': {
-            'type': 'float', 'value': 0.4, 'unit': 'mA', 'prec': 3,
-            'lolim': 0.05, 'low': 0.05, 'lolo': 0.05,
-            'hilim': 1, 'high': 1, 'hihi': 1},
+            'type': 'float', 'value': 0.432, 'unit': 'mA', 'prec': 3,
+            'lolim': 0.05, 'hilim': 1},
         'GPModKernStd-RB': {
-            'type': 'float', 'value': 0.4, 'unit': 'mA', 'prec': 3,
-            'lolim': 0.05, 'low': 0.05, 'lolo': 0.05,
-            'hilim': 1, 'high': 1, 'hihi': 1},
+            'type': 'float', 'value': 0.432, 'unit': 'mA', 'prec': 3,
+            'lolim': 0.05, 'hilim': 1},
         'GPModKernStd-Mon': {
-            'type': 'float', 'value': 0.4, 'unit': 'mA', 'prec': 3,
-            'lolim': 0.05, 'low': 0.05, 'lolo': 0.05,
-            'hilim': 1, 'high': 1, 'hihi': 1},
+            'type': 'float', 'value': 0.432, 'unit': 'mA', 'prec': 3,
+            'lolim': 0.05, 'hilim': 1},
         'GPModKernLenScl-SP': {
-            'type': 'float', 'value': 5, 'unit': 'V', 'prec': 3,
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': 10, 'high': 10, 'hihi': 10},
+            'type': 'float', 'value': 4, 'unit': 'V', 'prec': 3,
+            'lolim': 2, 'hilim': 10},
         'GPModKernLenScl-RB': {
-            'type': 'float', 'value': 5, 'unit': 'V', 'prec': 3,
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': 10, 'high': 10, 'hihi': 10},
+            'type': 'float', 'value': 4, 'unit': 'V', 'prec': 3,
+            'lolim': 2, 'hilim': 10},
         'GPModKernLenScl-Mon': {
-            'type': 'float', 'value': 5, 'unit': 'V', 'prec': 3,
-            'lolim': 1, 'low': 1, 'lolo': 1,
-            'hilim': 10, 'high': 10, 'hihi': 10},
+            'type': 'float', 'value': 4, 'unit': 'V', 'prec': 3,
+            'lolim': 2, 'hilim': 10},
+
+        # These properties are used to fix or unfix the fitting of the
+        # gp parameters.
+        'GPModNoiseStdFit-Sel': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Unfixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+        'GPModNoiseStdFit-Sts': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Unfixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+        'GPModKernStdFit-Sel': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Unfixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+        'GPModKernStdFit-Sts': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Unfixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+        'GPModKernLenSclFit-Sel': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Fixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+        'GPModKernLenSclFit-Sts': {
+            'type': 'enum', 'value': _ct.FixedUnfixed.Fixed,
+            'enums': _et.FIXED_UNFIXED, 'unit': 'Fixed_Unfixed'},
+
 
         # These are used to give the model inference about the bias
         # Generally ploted in Injcurr X Bias graphs
