@@ -5,7 +5,10 @@ import time as _time
 import logging as _log
 
 import numpy as _np
-from matplotlib import pyplot as _plt
+try:
+    from matplotlib import pyplot as _plt
+except:
+    _plt = None
 
 from ..search import PSSearch as _PSSearch
 from ..clientarch import ClientArchiver as _CltArch, Time as _Time, \
@@ -253,6 +256,14 @@ class MacReport:
         'Users',
     ]
 
+    FAILURES_MANUAL = [
+        # hydraulic failure, wrong machine shift for recovery
+        [_Time(2023, 3, 3, 22, 56, 0, 0), _Time(2023, 3, 3, 23, 0, 0, 0)],
+        # power grid failure, archiver was down
+        [_Time(2023, 5, 18, 5, 55, 0, 0), _Time(2023, 5, 18, 9, 8, 0, 0)],
+        [_Time(2024, 1, 18, 14, 0, 0, 0), _Time(2024, 1, 18, 19, 45, 0, 0)],
+    ]
+
     def __init__(self, connector=None, logger=None):
         """Initialize object."""
         # client archiver connector
@@ -1075,7 +1086,7 @@ class MacReport:
 
         datetimes = _np.array([_Time(t) for t in self._raw_data['Timestamp']])
 
-        fig, axs = _plt.subplots(16, 1, sharex=True)
+        fig, axs = _plt.subplots(17, 1, sharex=True)
         fig.set_size_inches(9, 10)
         fig.subplots_adjust(top=0.96, left=0.08, bottom=0.05, right=0.96)
         axs[0].set_title('Raw data', fontsize=12)
@@ -1152,40 +1163,47 @@ class MacReport:
 
         axs[10].xaxis.axis_date()
         axs[10].plot(
-            datetimes, self._raw_data['Distortions']['SOFBLoop'], '-',
-            color='orangered', label='Distortions - SOFB Loop Open')
+            datetimes, self._raw_data['Failures']['ManualAnnotated'], '-',
+            color='red', label='Failures - Manual Annotated')
         axs[10].legend(loc='upper left', fontsize=9)
         axs[10].grid()
 
         axs[11].xaxis.axis_date()
         axs[11].plot(
-            datetimes, self._raw_data['Distortions']['FOFBLoop'], '-',
-            color='orangered', label='Distortions - FOFB Loop Open')
+            datetimes, self._raw_data['Distortions']['SOFBLoop'], '-',
+            color='orangered', label='Distortions - SOFB Loop Open')
         axs[11].legend(loc='upper left', fontsize=9)
         axs[11].grid()
 
         axs[12].xaxis.axis_date()
         axs[12].plot(
-            datetimes[0], 0, '.', color='white',
-            label='Distortions - BbB Instabilities:')
-        axs[12].plot(
-            datetimes, self._raw_data['Distortions']['BbBHStab'], '-',
-            color='blue', label='H')
-        axs[12].plot(
-            datetimes, self._raw_data['Distortions']['BbBVStab'], '-',
-            color='red', label='V')
-        axs[12].plot(
-            datetimes, self._raw_data['Distortions']['BbBLStab'], '-',
-            color='green', label='L')
-        axs[12].legend(loc='upper left', fontsize=9, ncol=4)
+            datetimes, self._raw_data['Distortions']['FOFBLoop'], '-',
+            color='orangered', label='Distortions - FOFB Loop Open')
+        axs[12].legend(loc='upper left', fontsize=9)
         axs[12].grid()
 
         axs[13].xaxis.axis_date()
         axs[13].plot(
+            datetimes[0], 0, '.', color='white',
+            label='Distortions - BbB Instabilities:')
+        axs[13].plot(
+            datetimes, self._raw_data['Distortions']['BbBHStab'], '-',
+            color='blue', label='H')
+        axs[13].plot(
+            datetimes, self._raw_data['Distortions']['BbBVStab'], '-',
+            color='red', label='V')
+        axs[13].plot(
+            datetimes, self._raw_data['Distortions']['BbBLStab'], '-',
+            color='green', label='L')
+        axs[13].legend(loc='upper left', fontsize=9, ncol=4)
+        axs[13].grid()
+
+        axs[14].xaxis.axis_date()
+        axs[14].plot(
             datetimes, self._raw_data['Shift']['Injection'], '-',
             color='lightsalmon', label='Injection Shifts')
-        axs[13].legend(loc='upper left', fontsize=9)
-        axs[13].grid()
+        axs[14].legend(loc='upper left', fontsize=9)
+        axs[14].grid()
 
         shift2color = {
             'MachineStudy': ['MacStudy', 'skyblue'],
@@ -1195,26 +1213,26 @@ class MacReport:
         for shift, auxdata in shift2color.items():
             ydata = self._raw_data['Shift'][shift]
 
-            axs[14].xaxis.axis_date()
-            axs[14].plot(
+            axs[15].xaxis.axis_date()
+            axs[15].plot(
                 datetimes, ydata, '-',
                 color=auxdata[1], label=auxdata[0])
-        axs[14].legend(loc='upper left', ncol=4, fontsize=9)
-        axs[14].set_ylim(0.0, 2.0)
-        axs[14].grid()
+        axs[15].legend(loc='upper left', ncol=4, fontsize=9)
+        axs[15].set_ylim(0.0, 2.0)
+        axs[15].grid()
 
         egmodes2color = {
             'MultiBunch': 'orangered', 'SingleBunch': 'orange'}
         for egmode, color in egmodes2color.items():
             ydata = self._raw_data['EgunModes'][egmode]
 
-            axs[15].xaxis.axis_date()
-            axs[15].plot(
+            axs[16].xaxis.axis_date()
+            axs[16].plot(
                 datetimes, ydata, '-',
                 color=color, label=egmode)
-        axs[15].legend(loc='upper left', ncol=2, fontsize=9)
-        axs[15].set_ylim(0.0, 2.0)
-        axs[15].grid()
+        axs[16].legend(loc='upper left', ncol=2, fontsize=9)
+        axs[16].set_ylim(0.0, 2.0)
+        axs[16].grid()
 
         return fig
 
@@ -1339,6 +1357,10 @@ class MacReport:
         # get pvs data and calculate failures
         self._raw_data['Failures'] = dict()
 
+        # - manual annotated failures
+        self._raw_data['Failures']['ManualAnnotated'] = \
+            self._get_man_annotated_fails_data()
+
         # - subsystems status
         self._ps_fail_values, self._mps_fail_values = \
             self._get_subsystems_status_data()
@@ -1416,21 +1438,36 @@ class MacReport:
 
     def _get_egunmode_data(self):
         # single/multi bunch mode data
+        # get EVG injection data and oversample considering current data
         inj_ts, inj_vs = self._get_pv_data(self._injevt_pv)
         inj_vs = _interp1d_previous(inj_ts, inj_vs, self._curr_times)
+        # get egun trigger data and oversample considering current data
         trig_ts, trig_vs = self._get_pv_data(self._egtrgen_pv)
         trig_vs = _interp1d_previous(trig_ts, trig_vs, self._curr_times)
+        # get single bunch data and oversample considering current data
         sb_ts, sb_vs = self._get_pv_data(self._egpusel_pv)
         sb_vs = _interp1d_previous(sb_ts, sb_vs, self._curr_times)
+        # find points where the injection was with single bunch mode,
+        # store 1 for single bunch
         idcs1 = _np.where(inj_vs*trig_vs*sb_vs)[0]
         mode_ts = self._curr_times[idcs1]
         mode_vs = [1]*len(idcs1)
+        # find points where the injection was with multi bunch mode,
+        # considering this is complementary to single bunch injections,
+        # store 0 for multi bunch
         idcs2 = _np.where(inj_vs*trig_vs*1*_np.logical_not(sb_vs))[0]
         mode_ts = _np.r_[mode_ts, self._curr_times[idcs2]]
         mode_vs = _np.r_[mode_vs, [0]*len(idcs2)]
+        # sort results
         ind = mode_ts.argsort()
         mode_ts, mode_vs = mode_ts[ind], mode_vs[ind]
-        mode_vs = _interp1d_previous(mode_ts, mode_vs, self._curr_times)
+        # if there is injections in the period, oversample
+        if len(mode_ts):
+            mode_vs = _interp1d_previous(mode_ts, mode_vs, self._curr_times)
+        else:
+            # else, generate an array considering with multi bunch mode
+            mode_vs = _np.zeros(len(self._curr_times))
+        # build sb and mb values
         sbvals = mode_vs
         mbvals = _np.logical_not(mode_vs)
         return sbvals, mbvals
@@ -1559,8 +1596,8 @@ class MacReport:
             beg_val += [beg_idcs.size-1]
             beg1, beg2 = beg_idcs[beg_val], beg_idcs[beg_val] + 15
             if beg2[-1] > self._users_shift_delivd_values.size-1:
-                beg1.pop()
-                beg2.pop()
+                beg1 = _np.delete(beg1, -1)
+                beg2 = _np.delete(beg2, -1)
             beg_val = [i for i in range(beg1.size) if not
                        any([beg1[i] < e < beg2[i] for e in end_idcs])]
             beg1, beg2 = beg1[beg_val], beg2[beg_val]
@@ -1577,8 +1614,8 @@ class MacReport:
                              if end_idcs[i]-end_idcs[i-1] > 15]
             end1, end2 = end_idcs[end_val] - 15, end_idcs[end_val]
             if end1[0] < 0:
-                end1.pop(0)
-                end2.pop(0)
+                end1 = _np.delete(end1, 0)
+                end2 = _np.delete(end2, 0)
             end_val = [i for i in range(end1.size-1) if not
                        any([end1[i] < b < end2[i] for b in beg_idcs])]
             end1, end2 = end1[end_val], end2[end_val]
@@ -1591,9 +1628,13 @@ class MacReport:
             self._usershift_current_end_stddev = 0
 
         # # # ----- failures -----
-        beam_dump_values = _np.logical_not(
-            self._raw_data['Failures']['WrongShift']) * \
-            self._raw_data['Failures']['NoEBeam']
+        beam_dump_values = 1 * _np.logical_and(
+            _np.logical_not(
+                self._raw_data['Failures']['WrongShift']
+            ), _np.logical_or(
+                self._raw_data['Failures']['ManualAnnotated'],
+                self._raw_data['Failures']['NoEBeam']
+            ))
         self._usershift_beam_dump_count = _np.sum(
             _np.diff(beam_dump_values) > 0)
 
@@ -1754,6 +1795,20 @@ class MacReport:
             values[idcsdefv] = defv
         return times, values
 
+    def _get_man_annotated_fails_data(self):
+        # get all annotated failures
+        t2vs = _np.array([[], []])
+        for ini, end in MacReport.FAILURES_MANUAL:
+            t2vs = _np.c_[t2vs, [ini.timestamp(), 1], [end.timestamp(), 0]]
+        # insert initial point indicating not failure
+        t2vs = _np.c_[t2vs, [t2vs[0, 0]-1, 0]]
+        # sort by dates
+        t2vs = t2vs[:, t2vs[0, :].argsort()]
+        # calculate failures data in current timestamp base
+        failures = _interp1d_previous(
+            t2vs[0, :], t2vs[1, :], self._curr_times)
+        return failures
+
     def _calc_current_stats(self, dtimes):
         interval = _np.sum(dtimes)
         if not interval:
