@@ -29,8 +29,11 @@ class Keysight:
     wavet, waved = scope.wfm_get_data()
     plt.plot(wavet, waved)
     plt.show()
+
     """
 
+    SOCKET_TIMEOUT = 10  # [s]
+
     def __init__(self, scope=None, scopesignal=None):
         """."""
         if scopesignal and isinstance(scopesignal, tuple):
@@ -48,14 +51,14 @@ class Keysight:
     @property
     def scope_name(self):
         """."""
-        return _scopes.ScopeSignals.get_scope_name(self.host)
+        return _scopes.ScopeSignals.get_scope_name(scope_hostname=self.host)
 
     def connect(self):
         """."""
         self._socket = _socket.socket(
             _socket.AF_INET,  # Internet
             _socket.SOCK_STREAM)  # TCP
-        self._socket.settimeout(10)
+        self._socket.settimeout(Keysight.SOCKET_TIMEOUT)
         self._socket.connect((self.host, self.port))
 
     def close(self):
@@ -66,7 +69,7 @@ class Keysight:
         """Enable scope waveform acquisition."""
         return self.send_command(b'*IDN?\r\n')
 
-    def wfm_config(self):
+    def wfm_config(self, wait_trigger=False):
         """Set scope waveform format."""
         self.send_command(b":WAVeform:FORMat WORD\n", get_res=False)
         dataformat = self.send_command(b":WAVeform:FORMat?\n")
@@ -74,7 +77,10 @@ class Keysight:
         # Set bit order to MSB First
         self.send_command(b":WAVeform:BYTeorder MSBF\n", get_res=False)
         # Acquire
-        self.send_command(b':DIG\n', get_res=False)
+        if wait_trigger:
+            self.send_command(b':DIG\n', get_res=False)
+        else:
+            self.send_command(b':RUN\n', get_res=False)
 
     def wfm_acquire(self, channel):
         """Acquire scope waveform."""
@@ -101,9 +107,12 @@ class Keysight:
 
         # Get scales
         xinc = self.send_command(b":WAVeform:XINCrement?\n")
+        xinc = float(xinc)
         print('Horizontal Scale:', xinc)
         yinc = self.send_command(b":WAVeform:YINCrement?\n")
         yor = self.send_command(b":WAVeform:YORigin?\n")
+        yinc = float(yinc)
+        yor = float(yor)
         print('Vertical Scale:', xinc, yor)
 
         # Data aquisition
@@ -122,12 +131,14 @@ class Keysight:
 
         va1 = _np.array(list(dataraw)[0::2])
         va0 = _np.array(list(dataraw)[1::2])
+        va1 = va1[:va0.size]
+
         datay = ((va1 << 8) + va0 - 2**16*(va1 >> 7)) * yinc + yor
 
         datax = _np.arange(datay.size)*xinc
         return datax, datay, srate, bdw
 
-    def wfm_get_data(self, channel=None):
+    def wfm_get_data(self, channel=None, wait_trigger=False):
         """Enable and get sccope waveform data."""
         channel = channel or self.chan
         self.connect()
@@ -135,15 +146,16 @@ class Keysight:
         waved = None
         try:
             self.wfm_enable()
-            self.wfm_config()
+            self.wfm_config(wait_trigger)
             tini = _time.time()
             print('Acquiring ' + self.chan)
             wavet, waved, srate1, bdw1 = self.wfm_acquire(channel)
             print('Total acquisition time:', _time.time() - tini)
             # self.send_command(b":WAVeform:STReaming ON\n", get_res=False)
         except Exception:
-            print("Unexpected error:", _sys.exc_info()[0])
-            print('Close connetion by exception')
+            print('Close connection by exception')
+            raise
+
         finally:
             self.close()
 
@@ -158,6 +170,36 @@ class Keysight:
     def stats_acquire(self):
         """Return a dictionary of scope measurement statistics."""
         meas = self.send_command(b":MEASure:RESults?\n")
+        data = Keysight.process_stats_meas(meas)
+        return data
+
+    def stats_get_data(self):
+        """."""
+        self.connect()
+        try:
+            self.stats_enable()
+            tini = _time.time()
+            print('Acquiring ' + self.chan)
+            data = self.stats_acquire()
+            print('Total acquisition time:', _time.time() - tini)
+        except Exception:
+            print('Close connection by exception')
+            raise
+        finally:
+            self.close()
+        return data
+
+    def send_command(self, cmd, get_res=True):
+        """."""
+        self._socket.sendall(cmd)
+        if get_res:
+            return self._socket.recv(1024).decode('ascii')
+        return
+
+    @staticmethod
+    def process_stats_meas(meas):
+        """Process stats measurents."""
+        # TODO: generalize or instantiate for each scope type
         meas = meas.split(',')
         data = dict()
         for i in range(0, len(meas) - len(meas) % 8, 8):
@@ -172,26 +214,3 @@ class Keysight:
             datum['num_of_meas'] = int(float(meas[i+7]))
             data[label] = datum
         return data
-
-    def stats_get_data(self):
-        """."""
-        self.connect()
-        try:
-            self.stats_enable()
-            tini = _time.time()
-            print('Acquiring ' + self.chan)
-            data = self.stats_acquire()
-            print('Total acquisition time:', _time.time() - tini)
-        except Exception:
-            print("Unexpected error:", _sys.exc_info()[0])
-            print('Close connetion by exception')
-        finally:
-            self.close()
-        return data
-
-    def send_command(self, cmd, get_res=True):
-        """."""
-        self._socket.sendall(cmd)
-        if get_res:
-            return self._socket.recv(1024).decode('ascii')
-        return
