@@ -6,7 +6,6 @@ Documentation:
 
 https://wiki-sirius.lnls.br/mediawiki/index.php/Machine:Power_Supplies
 """
-
 from ...bsmp import constants as _const_bsmp
 
 # version of the BSMP implementation of power supplies that is compatible
@@ -314,17 +313,17 @@ class ConstFAC_DCDC(ConstPSBSMP):
     V_I_LOAD2 = 35
     V_V_CAPBANK = 36
     V_DUTY_CYCLE = 37
-    V_V_INPUT_IIB = 38
-    V_I_INPUT_IIB = 39
-    V_I_OUTPUT_IIB = 40
-    V_TEMP_IGBT_1_IIB = 41
-    V_TEMP_IGBT_2_IIB = 42
-    V_TEMP_INDUCTOR_IIB = 43
-    V_TEMP_HEATSINK_IIB = 44
-    V_V_DRIVER_IIB = 45
-    V_I_DRIVER_1_IIB = 46
-    V_I_DRIVER_2_IIB = 47
-    V_I_LEAKAGE_IIB = 48
+    V_I_LEAKAGE = 38
+    V_V_INPUT_IIB = 39
+    V_I_INPUT_IIB = 40
+    V_I_OUTPUT_IIB = 41
+    V_TEMP_IGBT_1_IIB = 42
+    V_TEMP_IGBT_2_IIB = 43
+    V_TEMP_INDUCTOR_IIB = 44
+    V_TEMP_HEATSINK_IIB = 45
+    V_V_DRIVER_IIB = 46
+    V_I_DRIVER_1_IIB = 47
+    V_I_DRIVER_2_IIB = 48
     V_TEMP_BOARD_IIB = 49
     V_RH_IIB = 50
     V_IIB_INTERLOCKS = 51
@@ -499,18 +498,18 @@ class ConstFAP_4P(ConstPSBSMP):
     V_DUTY_CYCLE_2_3 = 55
     V_DUTY_CYCLE_1_4 = 56
     V_DUTY_CYCLE_2_4 = 57
-    V_V_INPUT_IIB_1 = 58
-    V_V_OUTPUT_IIB_1 = 59
-    V_I_IGBT_1_IIB_1 = 60
-    V_I_IGBT_2_IIB_1 = 61
-    V_TEMP_IGBT_1_IIB_1 = 62
-    V_TEMP_IGBT_2_IIB_1 = 63
-    V_V_DRIVER_IIB_1 = 64
-    V_I_DRIVER_1_IIB_1 = 65
-    V_I_DRIVER_2_IIB_1 = 66
-    V_TEMP_INDUCTOR_IIB_1 = 67
-    V_TEMP_HEATSINK_IIB_1 = 68
-    V_I_LEAKAGE_IIB_1 = 69
+    V_I_LEAKAGE = 58
+    V_V_INPUT_IIB_1 = 59
+    V_V_OUTPUT_IIB_1 = 60
+    V_I_IGBT_1_IIB_1 = 61
+    V_I_IGBT_2_IIB_1 = 62
+    V_TEMP_IGBT_1_IIB_1 = 63
+    V_TEMP_IGBT_2_IIB_1 = 64
+    V_V_DRIVER_IIB_1 = 65
+    V_I_DRIVER_1_IIB_1 = 66
+    V_I_DRIVER_2_IIB_1 = 67
+    V_TEMP_INDUCTOR_IIB_1 = 68
+    V_TEMP_HEATSINK_IIB_1 = 69
     V_TEMP_BOARD_IIB_1 = 70
     V_RH_IIB_1 = 71
     V_IIB_INTERLOCKS_1 = 72
@@ -526,44 +525,41 @@ class ConstFAP_4P(ConstPSBSMP):
     V_I_DRIVER_2_IIB_2 = 82
     V_TEMP_INDUCTOR_IIB_2 = 83
     V_TEMP_HEATSINK_IIB_2 = 84
-    V_I_LEAKAGE_IIB_2 = 85
-    V_TEMP_BOARD_IIB_2 = 86
-    V_RH_IIB_2 = 87
-    V_IIB_INTERLOCKS_2 = 88
-    V_IIB_ALARMS_2 = 89
-    V_V_INPUT_IIB_3 = 90
-    V_V_OUTPUT_IIB_3 = 91
-    V_I_IGBT_1_IIB_3 = 92
-    V_I_IGBT_2_IIB_3 = 93
-    V_TEMP_IGBT_1_IIB_3 = 94
-    V_TEMP_IGBT_2_IIB_3 = 95
-    V_V_DRIVER_IIB_3 = 96
-    V_I_DRIVER_1_IIB_3 = 97
-    V_I_DRIVER_2_IIB_3 = 98
-    V_TEMP_INDUCTOR_IIB_3 = 99
-    V_TEMP_HEATSINK_IIB_3 = 100
-    V_I_LEAKAGE_IIB_3 = 101
-    V_TEMP_BOARD_IIB_3 = 102
-    V_RH_IIB_3 = 103
-    V_IIB_INTERLOCKS_3 = 104
-    V_IIB_ALARMS_3 = 105
-    V_V_INPUT_IIB_4 = 106
-    V_V_OUTPUT_IIB_4 = 107
-    V_I_IGBT_1_IIB_4 = 108
-    V_I_IGBT_2_IIB_4 = 109
-    V_TEMP_IGBT_1_IIB_4 = 110
-    V_TEMP_IGBT_2_IIB_4 = 111
-    V_V_DRIVER_IIB_4 = 112
-    V_I_DRIVER_1_IIB_4 = 113
-    V_I_DRIVER_2_IIB_4 = 114
-    V_TEMP_INDUCTOR_IIB_4 = 115
-    V_TEMP_HEATSINK_IIB_4 = 116
-    V_I_LEAKAGE_IIB_4 = 117
-    V_TEMP_BOARD_IIB_4 = 118
-    V_RH_IIB_4 = 119
-    V_IIB_INTERLOCKS_4 = 120
-    V_IIB_ALARMS_4 = 121
-    V_PS_ALARMS = 122
+    V_TEMP_BOARD_IIB_2 = 85
+    V_RH_IIB_2 = 86
+    V_IIB_INTERLOCKS_2 = 87
+    V_IIB_ALARMS_2 = 88
+    V_V_INPUT_IIB_3 = 89
+    V_V_OUTPUT_IIB_3 = 90
+    V_I_IGBT_1_IIB_3 = 91
+    V_I_IGBT_2_IIB_3 = 92
+    V_TEMP_IGBT_1_IIB_3 = 93
+    V_TEMP_IGBT_2_IIB_3 = 94
+    V_V_DRIVER_IIB_3 = 95
+    V_I_DRIVER_1_IIB_3 = 96
+    V_I_DRIVER_2_IIB_3 = 97
+    V_TEMP_INDUCTOR_IIB_3 = 98
+    V_TEMP_HEATSINK_IIB_3 = 99
+    V_TEMP_BOARD_IIB_3 = 100
+    V_RH_IIB_3 = 101
+    V_IIB_INTERLOCKS_3 = 102
+    V_IIB_ALARMS_3 = 103
+    V_V_INPUT_IIB_4 = 104
+    V_V_OUTPUT_IIB_4 = 105
+    V_I_IGBT_1_IIB_4 = 106
+    V_I_IGBT_2_IIB_4 = 107
+    V_TEMP_IGBT_1_IIB_4 = 108
+    V_TEMP_IGBT_2_IIB_4 = 109
+    V_V_DRIVER_IIB_4 = 110
+    V_I_DRIVER_1_IIB_4 = 111
+    V_I_DRIVER_2_IIB_4 = 112
+    V_TEMP_INDUCTOR_IIB_4 = 113
+    V_TEMP_HEATSINK_IIB_4 = 114
+    V_TEMP_BOARD_IIB_4 = 115
+    V_RH_IIB_4 = 116
+    V_IIB_INTERLOCKS_4 = 117
+    V_IIB_ALARMS_4 = 118
+    V_PS_ALARMS = 119
 
 
 class ConstFAP_2P2S(ConstPSBSMP):
@@ -603,18 +599,18 @@ class ConstFAP_2P2S(ConstPSBSMP):
     V_DUTY_CYCLE_2_3 = 61
     V_DUTY_CYCLE_1_4 = 62
     V_DUTY_CYCLE_2_4 = 63
-    V_V_INPUT_IIB_1 = 64
-    V_V_OUTPUT_IIB_1 = 65
-    V_I_IGBT_1_IIB_1 = 66
-    V_I_IGBT_2_IIB_1 = 67
-    V_TEMP_IGBT_1_IIB_1 = 68
-    V_TEMP_IGBT_2_IIB_1 = 69
-    V_V_DRIVER_IIB_1 = 70
-    V_I_DRIVER_1_IIB_1 = 71
-    V_I_DRIVER_2_IIB_1 = 72
-    V_TEMP_INDUCTOR_IIB_1 = 73
-    V_TEMP_HEATSINK_IIB_1 = 74
-    V_I_LEAKAGE_IIB_1 = 75
+    V_I_LEAKAGE = 64
+    V_V_INPUT_IIB_1 = 65
+    V_V_OUTPUT_IIB_1 = 66
+    V_I_IGBT_1_IIB_1 = 67
+    V_I_IGBT_2_IIB_1 = 68
+    V_TEMP_IGBT_1_IIB_1 = 69
+    V_TEMP_IGBT_2_IIB_1 = 70
+    V_V_DRIVER_IIB_1 = 71
+    V_I_DRIVER_1_IIB_1 = 72
+    V_I_DRIVER_2_IIB_1 = 73
+    V_TEMP_INDUCTOR_IIB_1 = 74
+    V_TEMP_HEATSINK_IIB_1 = 75
     V_TEMP_BOARD_IIB_1 = 76
     V_RH_IIB_1 = 77
     V_IIB_INTERLOCKS_1 = 78
@@ -630,43 +626,41 @@ class ConstFAP_2P2S(ConstPSBSMP):
     V_I_DRIVER_2_IIB_2 = 88
     V_TEMP_INDUCTOR_IIB_2 = 89
     V_TEMP_HEATSINK_IIB_2 = 90
-    V_PS_ALARMS = 91
-    V_TEMP_BOARD_IIB_2 = 92
-    V_RH_IIB_2 = 93
-    V_IIB_INTERLOCKS_2 = 94
-    V_IIB_ALARMS_2 = 95
-    V_V_INPUT_IIB_3 = 96
-    V_V_OUTPUT_IIB_3 = 97
-    V_I_IGBT_1_IIB_3 = 98
-    V_I_IGBT_2_IIB_3 = 99
-    V_TEMP_IGBT_1_IIB_3 = 100
-    V_TEMP_IGBT_2_IIB_3 = 101
-    V_V_DRIVER_IIB_3 = 102
-    V_I_DRIVER_1_IIB_3 = 103
-    V_I_DRIVER_2_IIB_3 = 104
-    V_TEMP_INDUCTOR_IIB_3 = 105
-    V_TEMP_HEATSINK_IIB_3 = 106
-    V_I_LEAKAGE_IIB_3 = 107
-    V_TEMP_BOARD_IIB_3 = 108
-    V_RH_IIB_3 = 109
-    V_IIB_INTERLOCKS_3 = 110
-    V_IIB_ALARMS_3 = 111
-    V_V_INPUT_IIB_4 = 112
-    V_V_OUTPUT_IIB_4 = 113
-    V_I_IGBT_1_IIB_4 = 114
-    V_I_IGBT_2_IIB_4 = 115
-    V_TEMP_IGBT_1_IIB_4 = 116
-    V_TEMP_IGBT_2_IIB_4 = 117
-    V_V_DRIVER_IIB_4 = 118
-    V_I_DRIVER_1_IIB_4 = 119
-    V_I_DRIVER_2_IIB_4 = 120
-    V_TEMP_INDUCTOR_IIB_4 = 121
-    V_TEMP_HEATSINK_IIB_4 = 122
-    V_I_LEAKAGE_IIB_4 = 123
-    V_TEMP_BOARD_IIB_4 = 124
-    V_RH_IIB_4 = 125
-    V_IIB_INTERLOCKS_4 = 126
-    V_IIB_ALARMS_4 = 127
+    V_TEMP_BOARD_IIB_2 = 91
+    V_RH_IIB_2 = 92
+    V_IIB_INTERLOCKS_2 = 93
+    V_IIB_ALARMS_2 = 94
+    V_V_INPUT_IIB_3 = 95
+    V_V_OUTPUT_IIB_3 = 96
+    V_I_IGBT_1_IIB_3 = 97
+    V_I_IGBT_2_IIB_3 = 98
+    V_TEMP_IGBT_1_IIB_3 = 99
+    V_TEMP_IGBT_2_IIB_3 = 100
+    V_V_DRIVER_IIB_3 = 101
+    V_I_DRIVER_1_IIB_3 = 102
+    V_I_DRIVER_2_IIB_3 = 103
+    V_TEMP_INDUCTOR_IIB_3 = 104
+    V_TEMP_HEATSINK_IIB_3 = 105
+    V_TEMP_BOARD_IIB_3 = 106
+    V_RH_IIB_3 = 107
+    V_IIB_INTERLOCKS_3 = 108
+    V_IIB_ALARMS_3 = 109
+    V_V_INPUT_IIB_4 = 110
+    V_V_OUTPUT_IIB_4 = 111
+    V_I_IGBT_1_IIB_4 = 112
+    V_I_IGBT_2_IIB_4 = 113
+    V_TEMP_IGBT_1_IIB_4 = 114
+    V_TEMP_IGBT_2_IIB_4 = 115
+    V_V_DRIVER_IIB_4 = 116
+    V_I_DRIVER_1_IIB_4 = 117
+    V_I_DRIVER_2_IIB_4 = 118
+    V_TEMP_INDUCTOR_IIB_4 = 119
+    V_TEMP_HEATSINK_IIB_4 = 120
+    V_TEMP_BOARD_IIB_4 = 121
+    V_RH_IIB_4 = 122
+    V_IIB_INTERLOCKS_4 = 123
+    V_IIB_ALARMS_4 = 124
+    V_PS_ALARMS = 125
 
 
 # --- Const ACDC ---
