@@ -1,11 +1,8 @@
 """Power Supply Model classes."""
 
-from . import pscreaders as _readers
-from . import pscwriters as _writers
-from . import pscontroller as _controller
-
-from ..bsmp import constants as _const_psbsmp
-from ..bsmp import entities as _etity_psbsmp
+from ..bsmp import constants as _const_psbsmp, entities as _etity_psbsmp
+from . import pscontroller as _controller, pscreaders as _readers, \
+    pscwriters as _writers
 
 
 class _PSModel:
@@ -31,6 +28,12 @@ class _PSModel:
         'CycleAmpl-RB': _c.V_SIGGEN_AMPLITUDE,
         'CycleOffset-RB': _c.V_SIGGEN_OFFSET,
         'CycleAuxParam-RB': _c.V_SIGGEN_AUX_PARAM,
+        # Wfm
+        'WfmSelected-Mon': _c.V_WFMREF_SELECTED,
+        'WfmSyncMode-Sts': _c.V_WFMREF_SYNC_MODE,
+        'WfmFreq-RB': _c.V_WFMREF_FREQUENCY,
+        'WfmGain-RB': _c.V_WFMREF_GAIN,
+        'WfmOffset-RB': _c.V_WFMREF_OFFSET,
         # Scope
         'ScopeSrcAddr-RB': _c.V_SCOPE_SRC_DATA,
         'ScopeFreq-RB': _c.V_SCOPE_FREQUENCY,
@@ -239,7 +242,8 @@ class _PSModel:
         elif epics_field == 'ScopeFreq-SP':
             return _writers.ScopeFreq(device_ids, pru_controller, setpoints)
         elif epics_field == 'ScopeDuration-SP':
-            return _writers.ScopeDuration(device_ids, pru_controller, setpoints)
+            return _writers.ScopeDuration(
+                device_ids, pru_controller, setpoints)
         elif epics_field == 'ScopeSrcAddr-SP':
             return _writers.ScopeSrcAddr(device_ids, pru_controller, setpoints)
         return None
@@ -263,20 +267,13 @@ class _PSModel:
     @staticmethod
     def _writer_cfgsiggen(
             device_ids, epics_field, pru_controller, setpoints):
-        p2i = {
-            'CycleType-Sel': 0,
-            'CycleNrCycles-SP': 1,
-            'CycleFreq-SP': 2,
-            'CycleAmpl-SP': 3,
-            'CycleOffset-SP': 4,
-            'CycleAuxParam-SP': 5,
-            }
-        _c = _const_psbsmp.ConstPSBSMP
-        if epics_field in p2i:
-            idx = p2i[epics_field]
+        params_siggen = _controller.StandardPSController.PARMS_SIGGEN
+        if epics_field in params_siggen:
+            idx = params_siggen.index(epics_field)
             return _writers.CfgSiggen(
                 device_ids, pru_controller, idx, setpoints)
         if epics_field == 'CycleDsbl-Cmd':
+            _c = _const_psbsmp.ConstPSBSMP
             return _writers.Command(
                 device_ids, pru_controller, _c.F_DISABLE_SIGGEN, setpoints)
         return None
@@ -296,6 +293,11 @@ class _PSModel:
         if epics_field == 'WfmMonAcq-Sel':
             return _writers.WfmMonAcq(
                 device_ids, pru_controller, setpoints)
+        params_wfm = _controller.StandardPSController.PARMS_WFM
+        if epics_field in params_wfm:
+            idx = params_wfm.index(epics_field)
+            return _writers.CfgWfm(
+                device_ids, pru_controller, idx, setpoints)
         return None
 
 
@@ -323,6 +325,7 @@ class PSModelFBP(_PSModel):
 
     _pruc_properties = {
         'SOFBMode-Sts': 'sofb_mode',
+        'IDFFMode-Sts': 'idff_mode',
         'SOFBCurrent-RB': 'sofb_current_rb',
         'SOFBCurrentRef-Mon': 'sofb_current_refmon',
         'SOFBCurrent-Mon': 'sofb_current_mon',
@@ -335,13 +338,15 @@ class PSModelFBP(_PSModel):
                 device_ids, pru_controller, setpoints)
         if epics_field == 'SOFBMode-Sel':
             return _writers.SOFBMode(pru_controller, setpoints)
+        if epics_field == 'IDFFMode-Sel':
+            return _writers.IDFFMode(pru_controller, setpoints)
         if epics_field == 'SOFBUpdate-Cmd':
             return _writers.SOFBUpdate(pru_controller, setpoints)
         return super().writer(
             device_ids, epics_field, pru_controller, setpoints)
 
 
-class PSModelFAC_DCDC(_PSModel):
+class PSModelFAC_DCDC(_PSModel):  # noqa: N801
     """FAC power supply model."""
 
     _n = 'FAC_DCDC'
@@ -359,6 +364,7 @@ class PSModelFAC_DCDC(_PSModel):
         'Current2-Mon': _c.V_I_LOAD2,
         'CapacitorBankVoltage-Mon': _c.V_V_CAPBANK,
         'PWMDutyCycle-Mon': _c.V_DUTY_CYCLE,
+        'LeakCurrent-Mon': _c.V_I_LEAKAGE,
         'VoltageInputIIB-Mon': _c.V_V_INPUT_IIB,
         'CurrentInputIIB-Mon': _c.V_I_INPUT_IIB,
         'CurrentOutputIIB-Mon': _c.V_I_OUTPUT_IIB,
@@ -369,7 +375,6 @@ class PSModelFAC_DCDC(_PSModel):
         'IGBTDriverVoltageIIB-Mon': _c.V_V_DRIVER_IIB,
         'IGBT1DriverCurrentIIB-Mon': _c.V_I_DRIVER_1_IIB,
         'IGBT2DriverCurrentIIB-Mon': _c.V_I_DRIVER_2_IIB,
-        'LeakCurrentIIB-Mon': _c.V_I_LEAKAGE_IIB,
         'BoardTemperatureIIB-Mon': _c.V_TEMP_BOARD_IIB,
         'RelativeHumidityIIB-Mon': _c.V_RH_IIB,
         'IntlkIIB-Mon': _c.V_IIB_INTERLOCKS,
@@ -378,7 +383,7 @@ class PSModelFAC_DCDC(_PSModel):
     }
 
 
-class PSModelFAC_2S_DCDC(_PSModel):
+class PSModelFAC_2S_DCDC(_PSModel):  # noqa: N801
     """FAC_2S_DCDC power supply model."""
 
     _n = 'FAC_2S_DCDC'
@@ -430,7 +435,7 @@ class PSModelFAC_2S_DCDC(_PSModel):
         }
 
 
-class PSModelFAC_2P4S_DCDC(PSModelFAC_DCDC):
+class PSModelFAC_2P4S_DCDC(PSModelFAC_DCDC):  # noqa: N801
     """FAC_2P4S_DCDC power supply model (BO Dipoles)."""
 
     _n = 'FAC_2P4S_DCDC'
@@ -538,7 +543,7 @@ class PSModelFAP(_PSModel):
         }
 
 
-class PSModelFAP_4P(_PSModel):
+class PSModelFAP_4P(_PSModel):  # noqa: N801, N801
     """FAP_4P power supply model."""
 
     _n = 'FAP_4P'
@@ -576,6 +581,7 @@ class PSModelFAP_4P(_PSModel):
         'IGBT2PWMDutyCycleMod3-Mon': _c.V_DUTY_CYCLE_2_3,
         'IGBT1PWMDutyCycleMod4-Mon': _c.V_DUTY_CYCLE_1_4,
         'IGBT2PWMDutyCycleMod4-Mon': _c.V_DUTY_CYCLE_2_4,
+        'LeakCurrent-Mon': _c.V_I_LEAKAGE,
         'VoltageInputIIBMod1-Mon': _c.V_V_INPUT_IIB_1,
         'VoltageOutputIIBMod1-Mon': _c.V_V_OUTPUT_IIB_1,
         'IGBT1CurrentIIBMod1-Mon': _c.V_I_IGBT_1_IIB_1,
@@ -587,7 +593,6 @@ class PSModelFAP_4P(_PSModel):
         'IGBT2DriverCurrentIIBMod1-Mon': _c.V_I_DRIVER_2_IIB_1,
         'InductorTemperatureIIBMod1-Mon': _c.V_TEMP_INDUCTOR_IIB_1,
         'HeatSinkTemperatureIIBMod1-Mon': _c.V_TEMP_HEATSINK_IIB_1,
-        'LeakageCurrentIIBMod1-Mon': _c.V_I_LEAKAGE_IIB_1,
         'TemperatureIIBMod1-Mon': _c.V_TEMP_BOARD_IIB_1,
         'RelativeHumidityIIBMod1-Mon': _c.V_RH_IIB_1,
         'IntlkIIBMod1-Mon': _c.V_IIB_INTERLOCKS_1,
@@ -603,7 +608,6 @@ class PSModelFAP_4P(_PSModel):
         'IGBT2DriverCurrentIIBMod2-Mon': _c.V_I_DRIVER_2_IIB_2,
         'InductorTemperatureIIBMod2-Mon': _c.V_TEMP_INDUCTOR_IIB_2,
         'HeatSinkTemperatureIIBMod2-Mon': _c.V_TEMP_HEATSINK_IIB_2,
-        'LeakageCurrentIIBMod2-Mon': _c.V_I_LEAKAGE_IIB_2,
         'TemperatureIIBMod2-Mon': _c.V_TEMP_BOARD_IIB_2,
         'RelativeHumidityIIBMod2-Mon': _c.V_RH_IIB_2,
         'IntlkIIBMod2-Mon': _c.V_IIB_INTERLOCKS_2,
@@ -619,7 +623,6 @@ class PSModelFAP_4P(_PSModel):
         'IGBT2DriverCurrentIIBMod3-Mon': _c.V_I_DRIVER_2_IIB_3,
         'InductorTemperatureIIBMod3-Mon': _c.V_TEMP_INDUCTOR_IIB_3,
         'HeatSinkTemperatureIIBMod3-Mon': _c.V_TEMP_HEATSINK_IIB_3,
-        'LeakageCurrentIIBMod3-Mon': _c.V_I_LEAKAGE_IIB_3,
         'TemperatureIIBMod3-Mon': _c.V_TEMP_BOARD_IIB_3,
         'RelativeHumidityIIBMod3-Mon': _c.V_RH_IIB_3,
         'IntlkIIBMod3-Mon': _c.V_IIB_INTERLOCKS_3,
@@ -635,7 +638,6 @@ class PSModelFAP_4P(_PSModel):
         'IGBT2DriverCurrentIIBMod4-Mon': _c.V_I_DRIVER_2_IIB_4,
         'InductorTemperatureIIBMod4-Mon': _c.V_TEMP_INDUCTOR_IIB_4,
         'HeatSinkTemperatureIIBMod4-Mon': _c.V_TEMP_HEATSINK_IIB_4,
-        'LeakageCurrentIIBMod4-Mon': _c.V_I_LEAKAGE_IIB_4,
         'TemperatureIIBMod4-Mon': _c.V_TEMP_BOARD_IIB_4,
         'RelativeHumidityIIBMod4-Mon': _c.V_RH_IIB_4,
         'IntlkIIBMod4-Mon': _c.V_IIB_INTERLOCKS_4,
@@ -644,7 +646,7 @@ class PSModelFAP_4P(_PSModel):
         }
 
 
-class PSModelFAP_2P2S(_PSModel):
+class PSModelFAP_2P2S(_PSModel):  # noqa: N801
     """FAP_2P2S power supply model."""
 
     _n = 'FAP_2P2S'
@@ -688,6 +690,7 @@ class PSModelFAP_2P2S(_PSModel):
         'IGBT2PWMDutyCycleMod3-Mon': _c.V_DUTY_CYCLE_2_3,
         'IGBT1PWMDutyCycleMod4-Mon': _c.V_DUTY_CYCLE_1_4,
         'IGBT2PWMDutyCycleMod4-Mon': _c.V_DUTY_CYCLE_2_4,
+        'LeakCurrent-Mon': _c.V_I_LEAKAGE,
         'VoltageInputIIBMod1-Mon': _c.V_V_INPUT_IIB_1,
         'VoltageOutputIIBMod1-Mon': _c.V_V_OUTPUT_IIB_1,
         'IGBT1CurrentIIBMod1-Mon': _c.V_I_IGBT_1_IIB_1,
@@ -699,7 +702,6 @@ class PSModelFAP_2P2S(_PSModel):
         'IGBT2DriverCurrentIIBMod1-Mon': _c.V_I_DRIVER_2_IIB_1,
         'InductorTemperatureIIBMod1-Mon': _c.V_TEMP_INDUCTOR_IIB_1,
         'HeatSinkTemperatureIIBMod1-Mon': _c.V_TEMP_HEATSINK_IIB_1,
-        'LeakageCurrentIIBMod1-Mon': _c.V_I_LEAKAGE_IIB_1,
         'TemperatureIIBMod1-Mon': _c.V_TEMP_BOARD_IIB_1,
         'RelativeHumidityIIBMod1-Mon': _c.V_RH_IIB_1,
         'IntlkIIBMod1-Mon': _c.V_IIB_INTERLOCKS_1,
@@ -730,7 +732,6 @@ class PSModelFAP_2P2S(_PSModel):
         'IGBT2DriverCurrentIIBMod3-Mon': _c.V_I_DRIVER_2_IIB_3,
         'InductorTemperatureIIBMod3-Mon': _c.V_TEMP_INDUCTOR_IIB_3,
         'HeatSinkTemperatureIIBMod3-Mon': _c.V_TEMP_HEATSINK_IIB_3,
-        'LeakageCurrentIIBMod3-Mon': _c.V_I_LEAKAGE_IIB_3,
         'TemperatureIIBMod3-Mon': _c.V_TEMP_BOARD_IIB_3,
         'RelativeHumidityIIBMod3-Mon': _c.V_RH_IIB_3,
         'IntlkIIBMod3-Mon': _c.V_IIB_INTERLOCKS_3,
@@ -746,7 +747,6 @@ class PSModelFAP_2P2S(_PSModel):
         'IGBT2DriverCurrentIIBMod4-Mon': _c.V_I_DRIVER_2_IIB_4,
         'InductorTemperatureIIBMod4-Mon': _c.V_TEMP_INDUCTOR_IIB_4,
         'HeatSinkTemperatureIIBMod4-Mon': _c.V_TEMP_HEATSINK_IIB_4,
-        'LeakageCurrentIIBMod4-Mon': _c.V_I_LEAKAGE_IIB_4,
         'TemperatureIIBMod4-Mon': _c.V_TEMP_BOARD_IIB_4,
         'RelativeHumidityIIBMod4-Mon': _c.V_RH_IIB_4,
         'IntlkIIBMod4-Mon': _c.V_IIB_INTERLOCKS_4,
@@ -758,7 +758,7 @@ class PSModelFAP_2P2S(_PSModel):
 # --- ACDC ---
 
 
-class PSModelFBP_DCLink(_PSModel):
+class PSModelFBP_DCLink(_PSModel):  # noqa: N801
     """FBP_DCLink model."""
 
     _n = 'FBP_DCLink'
@@ -809,7 +809,7 @@ class PSModelFBP_DCLink(_PSModel):
             readers, writers, pru_controller, devices)
 
 
-class PSModelFAC_2S_ACDC(_PSModel):
+class PSModelFAC_2S_ACDC(_PSModel):  # noqa: N801
     """FAC_2S_ACDC model."""
 
     _n = 'FAC_2S_ACDC'
@@ -879,7 +879,7 @@ class PSModelFAC_2S_ACDC(_PSModel):
             readers, writers, pru_controller, devices)
 
 
-class PSModelFAC_2P4S_ACDC(PSModelFAC_2S_ACDC):
+class PSModelFAC_2P4S_ACDC(PSModelFAC_2S_ACDC):  # noqa: N801
     """FAC_2P4S_ACDC model."""
 
     _n = 'FAC_2P4S_ACDC'
