@@ -11,6 +11,7 @@ from socket import timeout as _socket_timeout
 
 from ..thread import AsyncWorker as _AsyncWorker
 from ..search import PSSearch as _PSSearch
+from ..search import IDSearch as _IDSearch
 from ..bsmp import SerialError as _SerialError
 from ..bsmp import constants as _const_bsmp
 from ..devices import StrengthConv as _StrengthConv
@@ -44,7 +45,9 @@ class PSNamesSOFB:
 
     @staticmethod
     def get_psnames_ch(acc):
-        """Return horizontal corrector psnames of a given sector."""
+        """Return horizontal corrector psnames of a given sector/ID."""
+        if 'ID-' in acc:
+            return _IDSearch.conv_idname_2_idff_chnames(acc)
         if PSNamesSOFB._sofb_factory is None:
             from ..sofb.csdev import SOFBFactory
             PSNamesSOFB._sofb_factory = SOFBFactory
@@ -54,7 +57,9 @@ class PSNamesSOFB:
 
     @staticmethod
     def get_psnames_cv(acc):
-        """Return vertical corrector psnames of a given sector."""
+        """Return vertical corrector psnames of a given sector/ID."""
+        if 'ID-' in acc:
+            return _IDSearch.conv_idname_2_idff_cvnames(acc)
         if PSNamesSOFB._sofb_factory is None:
             from ..sofb.csdev import SOFBFactory
             PSNamesSOFB._sofb_factory = SOFBFactory
@@ -62,6 +67,26 @@ class PSNamesSOFB:
             PSNamesSOFB._sofb[acc] = PSNamesSOFB._sofb_factory.create(acc)
         return PSNamesSOFB._sofb[acc].cv_names
 
+    @staticmethod
+    def get_psnames_qs(acc):
+        """Return skew corrector psnames of a ID."""
+        if 'ID-' in acc:
+            return _IDSearch.conv_idname_2_idff_qsnames(acc)
+        return []
+
+    @staticmethod
+    def get_bbbnames(acc):
+        """Return bbbnames."""
+        corrnames = \
+            PSNamesSOFB.get_psnames_ch(acc) + \
+            PSNamesSOFB.get_psnames_cv(acc) + \
+            PSNamesSOFB.get_psnames_qs(acc)
+        bbbnames = set()
+        for corrname in corrnames:
+            bbbname = _PSSearch.conv_psname_2_bbbname(corrname)
+            bbbnames.add(bbbname)
+        return list(bbbnames)
+
 
 class UnitConverter:
     """."""
@@ -88,7 +113,7 @@ class UnitConverter:
             if pstype not in pstype_2_sconv:
                 # sconv = _NormFact.create(psname.replace(':PS', ':MA'))
                 sconv = _StrengthConv(
-                    psname, UnitConverter.DIPOLE_PROPTY, auto_mon=True)
+                    psname, UnitConverter.DIPOLE_PROPTY, auto_monitor_mon=True)
                 pstype_2_sconv[pstype] = sconv
 
         # convert index to numpy array
@@ -149,7 +174,7 @@ class PSConnSOFB:
 
     def __init__(
             self, ethbridgeclnt_class, bbbnames=None, mproc=None,
-            sofb_update_iocs=False, dipoleoff=False):
+            sofb_update_iocs=False, dipoleoff=False, acc=None):
         """."""
         # check arguments
         if mproc is not None and \
@@ -158,7 +183,7 @@ class PSConnSOFB:
             raise ValueError('Invalid mproc dictionary!')
 
         self._dipoleoff = dipoleoff
-        self._acc = 'SI'
+        self._acc = acc or 'SI'
         self._pru = None
         self._udc = None
         self.bbbnames = bbbnames or _dcopy(PSSOFB.BBBNAMES)
@@ -169,7 +194,8 @@ class PSConnSOFB:
 
         self._sofb_psnames = \
             PSNamesSOFB.get_psnames_ch(self._acc) + \
-            PSNamesSOFB.get_psnames_cv(self._acc)
+            PSNamesSOFB.get_psnames_cv(self._acc) + \
+            PSNamesSOFB.get_psnames_qs(self._acc)
 
         # snapshot of sofb current values
         ncorrs = len(self._sofb_psnames)
@@ -205,10 +231,12 @@ class PSConnSOFB:
             self.converter = UnitConverter(
                 self._sofb_psnames, dipoleoff=dipoleoff)
 
+    @property
     def pru(self):
         """Return Beagle-name to PRU-object dictionary."""
         return self._pru
 
+    @property
     def udc(self):
         """Return Beagle-name to UDC-object dictionary."""
         return self._udc
