@@ -324,6 +324,15 @@ class PSSearch:
         dict_aux = PSSearch.get_dclink_2_psname_dict(filter_auxps)
         return dict_aux[dclink]
 
+    @staticmethod
+    def conv_psname_2_udcindex(psname):
+        """Return power supply index in associated UDC."""
+        with PSSearch._lock:
+            PSSearch._reload_udc_2_bsmp_dict()
+        udc = PSSearch._bsmp_2_udc_dict[psname]
+        udcmap = PSSearch._udc_2_bsmp_dict[udc]
+        return [ps[0] for ps in udcmap].index(psname)
+
     @staticmethod
     def get_linac_ps_sinap2sirius_dict():
         """Return PS name convertion dict."""
@@ -359,7 +368,7 @@ class PSSearch:
             return PSSearch._splims_ps_unit
         elif psmodel in ('APU', ):
             return 'mm'
-        elif psmodel in ('FP_SEPT', 'FP_KCKR', 'FP_PINGER'):
+        elif psmodel in ('FP_SEPT', 'FP_KCKR', 'FP_KCKRCCOIL', 'FP_PINGER'):
             return PSSearch._splims_pu_unit
         else:
             raise ValueError(psmodel)
